searchData={"items":[{"type":"module","title":"Membrane.Debug.Filter","doc":"Membrane Filter, that can be used to create a child that will be used to debug data flowing thouth pipeline.\n\nAny buffers, stream formats and events arriving to Elixir.Membrane.Debug.Filter will be forwarded by it to the opposite\nside than the one from which they came.\n\nUsage example:\n```elixir\nchild(:source, CustomSource)\n|> child(:filter, %Membrane.Debug.Filter{\n  handle_buffer: &IO.inspect(&1, label: \"buffer\"),\n  handle_stream_format: &IO.inspect(&1, label: \"stream format\")\n})\n|> child(:sink, CustomSink)\n```","ref":"Membrane.Debug.Filter.html"},{"type":"module","title":"Element options - Membrane.Debug.Filter","doc":"Passed via struct `t:Membrane.Debug.Filter.t/0`\n\n- `handle_buffer`  \n\n  ```\n  (Buffer.t() -> any())\n  ```\n  \n  Default value: `&Membrane.Debug.Filter.noop/1`  \n  Function with arity 1, that will be called with all buffers handled by this sink.\n  Result of this function is ignored.\n\n\n- `handle_event`  \n\n  ```\n  (Event.t() -> any())\n  ```\n  \n  Default value: `&Membrane.Debug.Filter.noop/1`  \n  Function with arity 1, that will be called with all events handled by this sink.\n  Result of this function is ignored.\n\n\n\n- `handle_stream_format`  \n\n  ```\n  (StreamFormat.t() -> any())\n  ```\n  \n  Default value: `&Membrane.Debug.Filter.noop/1`  \n  Function with arity 1, that will be called with all stream formats handled by this sink.\n  Result of this function is ignored.","ref":"Membrane.Debug.Filter.html#module-element-options"},{"type":"module","title":"Pads - Membrane.Debug.Filter","doc":"### `:input`\n\nAccepted formats:\n```\n_any\n```\n\nDirection: | `:input`\nAvailability: | `:always`\nFlow control: | `:auto`\n\n\n### `:output`\n\nAccepted formats:\n```\n_any\n```\n\nDirection: | `:output`\nAvailability: | `:always`\nFlow control: | `:auto`","ref":"Membrane.Debug.Filter.html#module-pads"},{"type":"function","title":"Membrane.Debug.Filter.noop/1","doc":"","ref":"Membrane.Debug.Filter.html#noop/1"},{"type":"function","title":"Membrane.Debug.Filter.options/0","doc":"Returns description of options available for this module","ref":"Membrane.Debug.Filter.html#options/0"},{"type":"type","title":"Membrane.Debug.Filter.t/0","doc":"Struct containing options for `Membrane.Debug.Filter`","ref":"Membrane.Debug.Filter.html#t:t/0"},{"type":"module","title":"Membrane.Debug.Sink","doc":"Membrane Sink, that can be used to create a child that will be used to debug data flowing thouth pipeline.\n\nUsage example:\n```elixir\nchild(:source, CustomSource)\n|> child(:sink, %Membrane.Debug.Sink{\n  handle_buffer: &IO.inspect(&1, label: \"buffer\"),\n  handle_event: &IO.inspect(&1, label: \"event\")\n})\n```","ref":"Membrane.Debug.Sink.html"},{"type":"module","title":"Element options - Membrane.Debug.Sink","doc":"Passed via struct `t:Membrane.Debug.Sink.t/0`\n\n- `handle_buffer`  \n\n  ```\n  (Buffer.t() -> any())\n  ```\n  \n  Default value: `&Membrane.Debug.Sink.noop/1`  \n  Function with arity 1, that will be called with all buffers handled by this sink.\n  Result of this function is ignored.\n\n\n- `handle_event`  \n\n  ```\n  (Event.t() -> any())\n  ```\n  \n  Default value: `&Membrane.Debug.Sink.noop/1`  \n  Function with arity 1, that will be called with all events handled by this sink.\n  Result of this function is ignored.\n\n\n\n- `handle_stream_format`  \n\n  ```\n  (StreamFormat.t() -> any())\n  ```\n  \n  Default value: `&Membrane.Debug.Sink.noop/1`  \n  Function with arity 1, that will be called with all stream formats handled by this sink.\n  Result of this function is ignored.","ref":"Membrane.Debug.Sink.html#module-element-options"},{"type":"module","title":"Pads - Membrane.Debug.Sink","doc":"### `:input`\n\nAccepted formats:\n```\n_any\n```\n\nDirection: | `:input`\nAvailability: | `:always`\nFlow control: | `:auto`","ref":"Membrane.Debug.Sink.html#module-pads"},{"type":"function","title":"Membrane.Debug.Sink.noop/1","doc":"","ref":"Membrane.Debug.Sink.html#noop/1"},{"type":"function","title":"Membrane.Debug.Sink.options/0","doc":"Returns description of options available for this module","ref":"Membrane.Debug.Sink.html#options/0"},{"type":"type","title":"Membrane.Debug.Sink.t/0","doc":"Struct containing options for `Membrane.Debug.Sink`","ref":"Membrane.Debug.Sink.html#t:t/0"},{"type":"behaviour","title":"Membrane.Pipeline","doc":"A behaviour module for implementing pipelines.\n\n`Membrane.Pipeline` contains the callbacks and functions for constructing and supervising pipelines.\nPipelines facilitate the convenient instantiation, linking, and management of elements and bins.\\\nLinking pipeline children together enables them to pass and process data.\n\nTo create a pipeline, use `use Membrane.Pipeline` and implement callbacks of `Membrane.Pipeline`  behaviour.\nSee `Membrane.ChildrenSpec` for details on instantiating and linking children.","ref":"Membrane.Pipeline.html"},{"type":"behaviour","title":"Starting and supervision - Membrane.Pipeline","doc":"Start a pipeline with `start_link/2` or `start/2`. Pipelines always spawn under a dedicated supervisor, so\nin the case of success, either function will return `{:ok, supervisor_pid, pipeline_pid}` .\n\nThe supervisor never restarts the pipeline, but it does ensure that the pipeline and its children terminate properly.\nIf the pipeline needs to be restarted, it should be spawned under a different supervisor with the appropriate strategy.\n\n#","ref":"Membrane.Pipeline.html#module-starting-and-supervision"},{"type":"behaviour","title":"Starting under a supervision tree - Membrane.Pipeline","doc":"A pipeline can be spawned under a supervision tree like any other `GenServer`.\\\n `use Membrane.Pipeline` injects a `child_spec/1` function. A simple scenario could look like this:\n\n    defmodule MyPipeline do\n      use Membrane.Pipeline\n\n      def start_link(options) do\n        Membrane.Pipeline.start_link(__MODULE__, options, name: MyPipeline)\n      end\n\n      # ...\n    end\n\n    Supervisor.start_link([{MyPipeline, option: :value}], strategy: :one_for_one)\n    send(MyPipeline, :message)\n\n#","ref":"Membrane.Pipeline.html#module-starting-under-a-supervision-tree"},{"type":"behaviour","title":"Starting outside of a supervision tree - Membrane.Pipeline","doc":"When starting a pipeline outside a supervision tree, use the `pipeline_pid` pid to interact with the pipeline.\n A simple scenario could look like this:\n\n    {:ok, _supervisor_pid, pipeline_pid} = Membrane.Pipeline.start_link(MyPipeline, option: :value)\n    send(pipeline_pid, :message)\n\n#","ref":"Membrane.Pipeline.html#module-starting-outside-of-a-supervision-tree"},{"type":"behaviour","title":"Visualizing the supervision tree - Membrane.Pipeline","doc":"Use the [Applications tab](https://www.erlang.org/doc/apps/observer/observer_ug#applications-tab) in Erlang's Observer GUI\n(or the `Kino` library in Livebook) to visualize a pipeline's internal supervision tree. Use the following configuration for debugging purposes only:\n\n      config :membrane_core, unsafely_name_processes_for_stalker: [:components]\n\nThis improves the readability of the Observer's process tree graph by naming the pipeline descendants, as demonstrated here:\n\n![Observer graph](assets/images/observer_graph.png).","ref":"Membrane.Pipeline.html#module-visualizing-the-supervision-tree"},{"type":"macro","title":"Membrane.Pipeline.__using__/1","doc":"Brings all the stuff necessary to implement a pipeline.\n\nOptions:\n  - `:bring_spec?` - if true (default) imports and aliases `Membrane.ChildrenSpec`\n  - `:bring_pad?` - if true (default) requires and aliases `Membrane.Pad`","ref":"Membrane.Pipeline.html#__using__/1"},{"type":"function","title":"Membrane.Pipeline.call/3","doc":"Calls the pipeline with a message.\n\nReturns the result of the pipeline call.","ref":"Membrane.Pipeline.html#call/3"},{"type":"callback","title":"Membrane.Pipeline.handle_call/3","doc":"Callback invoked when the pipeline is called using a synchronous call.\n\nContext passed to this callback contains an additional field `:from`.\nBy default, it does nothing.","ref":"Membrane.Pipeline.html#c:handle_call/3"},{"type":"callback","title":"Membrane.Pipeline.handle_child_notification/4","doc":"Callback invoked when a notification comes in from a child.\n\nBy default, it ignores the notification.","ref":"Membrane.Pipeline.html#c:handle_child_notification/4"},{"type":"callback","title":"Membrane.Pipeline.handle_child_pad_removed/4","doc":"Callback invoked when a child removes its pad.\n\nThe callback won't be invoked, when you have initiated the pad removal,\neg. when you have returned `t:Membrane.Pipeline.Action.remove_link()`\naction which made one of your children's pads be removed.\nBy default, it does nothing.","ref":"Membrane.Pipeline.html#c:handle_child_pad_removed/4"},{"type":"callback","title":"Membrane.Pipeline.handle_child_playing/3","doc":"Callback invoked when a child enters `playing` playback.\n\nBy default, it does nothing.","ref":"Membrane.Pipeline.html#c:handle_child_playing/3"},{"type":"callback","title":"Membrane.Pipeline.handle_child_setup_completed/3","doc":"Callback invoked when a child completes its setup.\n\nBy default, it does nothing.","ref":"Membrane.Pipeline.html#c:handle_child_setup_completed/3"},{"type":"callback","title":"Membrane.Pipeline.handle_crash_group_down/3","doc":"Callback invoked when a crash group crashes.\n\nContext passed to this callback contains 2 additional fields: `:members` and `:crash_initiator`.\nBy default, it does nothing.","ref":"Membrane.Pipeline.html#c:handle_crash_group_down/3"},{"type":"callback","title":"Membrane.Pipeline.handle_element_end_of_stream/4","doc":"Callback invoked when a child element finishes processing a stream via the given pad.\n\nBy default, it does nothing.","ref":"Membrane.Pipeline.html#c:handle_element_end_of_stream/4"},{"type":"callback","title":"Membrane.Pipeline.handle_element_start_of_stream/4","doc":"Callback invoked when a child element starts processing a stream via the given pad.\n\nBy default, it does nothing.","ref":"Membrane.Pipeline.html#c:handle_element_start_of_stream/4"},{"type":"callback","title":"Membrane.Pipeline.handle_info/3","doc":"Callback invoked when the pipeline receives a message that is not recognized\nas an internal Membrane message.\n\nUseful for receiving data sent from NIFs or other external sources.\nBy default, it logs and ignores the received message.","ref":"Membrane.Pipeline.html#c:handle_info/3"},{"type":"callback","title":"Membrane.Pipeline.handle_init/2","doc":"Callback invoked on initialization of the pipeline.\n\nThis callback is synchronous: the process that started the pipeline waits until `handle_init`\nfinishes, so it's important to do any long-lasting or complex work in `c:handle_setup/2`.\n`handle_init` should be used for things, like parsing options, initializing state, or spawning\nchildren. By default, `handle_init` converts `opts` to a map if they're a struct and sets them as the pipeline state.","ref":"Membrane.Pipeline.html#c:handle_init/2"},{"type":"callback","title":"Membrane.Pipeline.handle_playing/2","doc":"Callback invoked when the pipeline switches the playback to `:playing`.\nBy default, it does nothing.","ref":"Membrane.Pipeline.html#c:handle_playing/2"},{"type":"callback","title":"Membrane.Pipeline.handle_setup/2","doc":"Callback invoked on pipeline startup, right after `c:handle_init/2`.\n\nAny long-lasting or complex initialization should happen here.\nBy default, it does nothing.","ref":"Membrane.Pipeline.html#c:handle_setup/2"},{"type":"callback","title":"Membrane.Pipeline.handle_spec_started/3","doc":"This callback is deprecated since v1.1.0-rc0.\n\nCallback invoked when children of `Membrane.ChildrenSpec` are started.\n\nIt is invoked, only if pipeline module contains its definition. Otherwise, nothing happens.","ref":"Membrane.Pipeline.html#c:handle_spec_started/3"},{"type":"callback","title":"Membrane.Pipeline.handle_terminate_request/2","doc":"Callback invoked when the pipeline is requested to terminate with `terminate/2`.\nBy default, it returns `t:Membrane.Pipeline.Action.terminate/0` with reason `:normal`.","ref":"Membrane.Pipeline.html#c:handle_terminate_request/2"},{"type":"callback","title":"Membrane.Pipeline.handle_tick/3","doc":"Callback invoked upon each timer tick. A timer can be started with `Membrane.Pipeline.Action.start_timer`\naction.","ref":"Membrane.Pipeline.html#c:handle_tick/3"},{"type":"function","title":"Membrane.Pipeline.list_pipelines/0","doc":"Returns list of pipeline PIDs currently running on the current node.\n\nUse for debugging only.","ref":"Membrane.Pipeline.html#list_pipelines/0"},{"type":"function","title":"Membrane.Pipeline.list_pipelines/1","doc":"Returns list of pipeline PIDs currently running on the passed node. \\\nCompare to `list_pipelines/0`.","ref":"Membrane.Pipeline.html#list_pipelines/1"},{"type":"function","title":"Membrane.Pipeline.pipeline?/1","doc":"Checks whether the module is a pipeline.","ref":"Membrane.Pipeline.html#pipeline?/1"},{"type":"function","title":"Membrane.Pipeline.start/3","doc":"Starts the pipeline outside a supervision tree. Compare to `start_link/3`.","ref":"Membrane.Pipeline.html#start/3"},{"type":"function","title":"Membrane.Pipeline.start_link/3","doc":"Starts the pipeline based on the given module and links it to the current process.\n\n\nPipeline options are passed to the `c:handle_init/2` callback.\nNote that this function returns `{:ok, supervisor_pid, pipeline_pid}` in case of\nsuccess. Check the 'Starting and supervision' section of the moduledoc for details.","ref":"Membrane.Pipeline.html#start_link/3"},{"type":"function","title":"Membrane.Pipeline.terminate/2","doc":"Terminates the pipeline.\n\nAccepts three options:\n* `asynchronous?` - if set to `true`, pipeline termination won't be blocking and\n  will be executed in the process whose pid is returned as a function result.\n  If set to `false`, pipeline termination will be blocking and will be executed in\n  the process that called this function. Defaults to `false`.\n* `timeout` - specifies how much time (ms) to wait for the pipeline to gracefully\n  terminate. Defaults to 5000.\n* `force?` - determines how to handle a pipeline still alive after `timeout`.\n  If set to `true`, `Process.exit/2` kills the pipeline with reason `:kill` and returns\n  `{:error, :timeout}`.\n  If set to `false`, it raises an error. Defaults to `false`.\n\nReturns:\n* `{:ok, pid}` - option `asynchronous?: true` was passed.\n* `:ok` - pipeline gracefully terminated within `timeout`.\n* `{:error, :timeout}` - pipeline was killed after `timeout`.","ref":"Membrane.Pipeline.html#terminate/2"},{"type":"type","title":"Membrane.Pipeline.callback_return/0","doc":"Defines return values from Pipeline callback functions.","ref":"Membrane.Pipeline.html#t:callback_return/0"},{"type":"type","title":"Return values - Membrane.Pipeline.callback_return/0","doc":"* `{[action], state}` - Returns a list of actions that will be performed within the\n    pipeline, e.g., starting new children, sending messages to specific children, etc.\n    Actions are tuples of `{type, arguments}`, so they can be expressed as a keyword list.\n    See `Membrane.Pipeline.Action` for more info.","ref":"Membrane.Pipeline.html#t:callback_return/0-return-values"},{"type":"type","title":"Membrane.Pipeline.config/0","doc":"List of configurations used by `start/3` and `start_link/3`.","ref":"Membrane.Pipeline.html#t:config/0"},{"type":"type","title":"Membrane.Pipeline.config_entry/0","doc":"Defines configuration value used by the `start/3` and `start_link/3`.","ref":"Membrane.Pipeline.html#t:config_entry/0"},{"type":"type","title":"Membrane.Pipeline.name/0","doc":"The Pipeline name","ref":"Membrane.Pipeline.html#t:name/0"},{"type":"type","title":"Membrane.Pipeline.on_start/0","doc":"Defines the return value of the `start/3` and `start_link/3`.\"","ref":"Membrane.Pipeline.html#t:on_start/0"},{"type":"type","title":"Membrane.Pipeline.pipeline_options/0","doc":"Defines options passed to the `start/3` and `start_link/3` and subsequently received\nin the `c:handle_init/2` callback.","ref":"Membrane.Pipeline.html#t:pipeline_options/0"},{"type":"type","title":"Membrane.Pipeline.state/0","doc":"The pipeline state.","ref":"Membrane.Pipeline.html#t:state/0"},{"type":"module","title":"Membrane.Pipeline.Action","doc":"This module contains type specifications of actions that can be returned\nfrom pipeline callbacks.\n\nReturning actions is a way of pipeline interaction with\nother components and parts of framework. Each action may be returned by any\ncallback unless explicitly stated otherwise.","ref":"Membrane.Pipeline.Action.html"},{"type":"type","title":"Membrane.Pipeline.Action.common_actions/0","doc":"Type describing the actions that can be returned from any pipeline callback.","ref":"Membrane.Pipeline.Action.html#t:common_actions/0"},{"type":"type","title":"Membrane.Pipeline.Action.notify_child/0","doc":"Action that sends a message to a child identified by name.","ref":"Membrane.Pipeline.Action.html#t:notify_child/0"},{"type":"type","title":"Membrane.Pipeline.Action.remove_children/0","doc":"Action that stops, unlinks and removes specified child/children from the pipeline.\n\nA child name, list of children names, children group id or a list of children group ids can be specified\nas an argument.","ref":"Membrane.Pipeline.Action.html#t:remove_children/0"},{"type":"type","title":"Membrane.Pipeline.Action.remove_link/0","doc":"Action that removes link, which relates to specified child and pad.\n\nRemoved link has to have dynamic pads on both ends.","ref":"Membrane.Pipeline.Action.html#t:remove_link/0"},{"type":"type","title":"Membrane.Pipeline.Action.reply/0","doc":"Action that replies to a `Membrane.Pipeline.call/3`. Can be returned only from the `c:Membrane.Pipeline.handle_call/3` callback, in\nwhich context the caller reference is available, under the `:from` key.","ref":"Membrane.Pipeline.Action.html#t:reply/0"},{"type":"type","title":"Membrane.Pipeline.Action.reply_to/0","doc":"Action that replies to a `Membrane.Pipeline.call/3`. Useful when one does not want to reply in\n`c:Membrane.Pipeline.handle_call/3` callback. A caller reference is required to be passed, so one needs to save this\nreference from the `Membrane.Pipeline.CallbackContext.Call`, where it is available under the `:from` key.","ref":"Membrane.Pipeline.Action.html#t:reply_to/0"},{"type":"type","title":"Membrane.Pipeline.Action.setup/0","doc":"Action that manages the end of the component setup.\n\nBy default, component setup ends with the end of `c:Membrane.Pipeline.handle_setup/2` callback.\nIf `{:setup, :incomplete}` is returned there, setup lasts until `{:setup, :complete}`\nis returned from antoher callback.\n\nUntils the setup lasts, the component won't enter `:playing` playback.","ref":"Membrane.Pipeline.Action.html#t:setup/0"},{"type":"type","title":"Membrane.Pipeline.Action.spec/0","doc":"Action that instantiates children and links them according to `Membrane.ChildrenSpec`.\n\nChildren's playback is changed to the current pipeline state.\n`c:Membrane.Pipeline.handle_spec_started/3` callback is executed once it happens.","ref":"Membrane.Pipeline.Action.html#t:spec/0"},{"type":"type","title":"Membrane.Pipeline.Action.start_timer/0","doc":"Starts a timer that will invoke `c:Membrane.Pipeline.handle_tick/3` callback\nevery `interval` according to the given `clock`.\n\nThe timer's `id` is passed to the `c:Membrane.Pipeline.handle_tick/3`\ncallback and can be used for changing its interval via `t:timer_interval/0`\nor stopping it via `t:stop_timer/0`.\n\nIf `interval` is set to `:no_interval`, the timer won't issue any ticks until\nthe interval is set with `t:timer_interval/0` action.\n\nIf no `clock` is passed, pipeline clock is chosen.\n\nTimers use `Process.send_after/3` under the hood.","ref":"Membrane.Pipeline.Action.html#t:start_timer/0"},{"type":"type","title":"Membrane.Pipeline.Action.stop_timer/0","doc":"Stops a timer started with `t:start_timer/0` action.\n\nThis action is atomic: stopping timer guarantees that no ticks will arrive from it.","ref":"Membrane.Pipeline.Action.html#t:stop_timer/0"},{"type":"type","title":"Membrane.Pipeline.Action.t/0","doc":"Type describing the union of all actions that can be returned from pipeline callbacks.\n\nReturning actions is a way of pipeline interaction with its children and\nother parts of framework.","ref":"Membrane.Pipeline.Action.html#t:t/0"},{"type":"type","title":"Membrane.Pipeline.Action.terminate/0","doc":"Terminates the pipeline with given reason.\n\nTermination reason follows the OTP semantics:\n- Use `:normal` for graceful termination. If the pipeline has no children, it terminates\nimmediately. Otherwise, it switches to the zombie mode, requests all the children to terminate,\nwaits for them to terminate and then terminates itself. In the zombie mode, no pipeline callbacks\nare called and all messages and calls to the pipeline are ignored (apart from Membrane internal\nmessages)\n- If the reason is other than `:normal`, the pipeline terminates immediately. The pipeline supervisor\nterminates all the children with the reason `:shutdown`\n- If the reason is neither `:normal`, `:shutdown` nor `{:shutdown, term}`, an error is logged","ref":"Membrane.Pipeline.Action.html#t:terminate/0"},{"type":"type","title":"Membrane.Pipeline.Action.timer_interval/0","doc":"Changes interval of a timer started with `t:start_timer/0`.\n\nPermitted only from `c:Membrane.Pipeline.handle_tick/3`, unless the interval\nwas previously set to `:no_interval`.\n\nIf the `interval` is `:no_interval`, the timer won't issue any ticks until\nanother `t:timer_interval/0` action. Otherwise, the timer will issue ticks every\nnew `interval`. The next tick after interval change is scheduled at\n`new_interval + previous_time`, where previous_time is the time of the latest\ntick or the time of returning `t:start_timer/0` action if no tick has been\nsent yet. Note that if `current_time - previous_time > new_interval`, a burst\nof `div(current_time - previous_time, new_interval)` ticks is issued immediately.","ref":"Membrane.Pipeline.Action.html#t:timer_interval/0"},{"type":"module","title":"Membrane.Pipeline.CallbackContext","doc":"Module describing context passed to the `Membrane.Pipeline` callbacks.","ref":"Membrane.Pipeline.CallbackContext.html"},{"type":"type","title":"Membrane.Pipeline.CallbackContext.t/0","doc":"Type describing context passed to the `Membrane.Pipeline` callbacks.\n\nField `:from` is present only in `c:Membrane.Pipeline.handle_call/3`.\n\nField `:start_of_stream_received?` is present only in\n`c:Membrane.Pipeline.handle_element_end_of_stream/4`.\n\nFields `:members`, `:crash_initiator` and `:crash_reason` are present only in\n`c:Membrane.Pipeline.handle_crash_group_down/3`.","ref":"Membrane.Pipeline.CallbackContext.html#t:t/0"},{"type":"module","title":"Membrane.RCMessage","doc":"An abstract module aggregating all the messages that can be sent by the `RCPipeline`.\n\nCheck `t:t/0` for available messages.","ref":"Membrane.RCMessage.html"},{"type":"type","title":"Membrane.RCMessage.t/0","doc":"The type describing all possible `Membrane.RCMessage`s.","ref":"Membrane.RCMessage.html#t:t/0"},{"type":"module","title":"Membrane.RCPipeline","doc":"Remote controlled pipeline - a basic `Membrane.Pipeline` implementation that can be remotely controlled from an external process.\n\nThe easiest way to start this pipeline is to use `start_link!/1`\n```\n  pipeline = Membrane.RCPipeline.start_link!()\n```\n\nThe controlling process can request the execution of arbitrary\nvalid `Membrane.Pipeline.Action`:\n```\n  children = ...\n  links = ...\n  actions = [{:spec, children++links}]\n  Pipeline.exec_actions(pipeline, actions)\n```\n\nThe controlling process can also subscribe to the messages\nsent by the pipeline and later on synchronously await for these messages:\n```\n# subscribes to message which is sent when the pipeline enters `playing`\nMembrane.RCPipeline.subscribe(pipeline, %Message.Playing{})\n...\n# awaits for the message sent when the pipeline enters :playing playback\nMembrane.RCPipeline.await_playing(pipeline)\n...\n```\n\n`Membrane.RCPipeline` can be used when there is no need for introducing a custom\nlogic in the `Membrane.Pipeline` callbacks implementation. An example of usage could be running a\npipeline from the elixir script. `Membrane.RCPipeline` sends the following messages:\n* `t:Membrane.RCMessage.Playing.t/0` sent when pipeline enters `playing` playback,\n* `t:Membrane.RCMessage.StartOfStream.t/0` sent\nwhen one of direct pipeline children informs the pipeline about start of a stream,\n* `t:Membrane.RCMessage.EndOfStream.t/0` sent\nwhen one of direct pipeline children informs the pipeline about end of a stream,\n* `t:Membrane.RCMessage.Notification.t/0` sent when pipeline\nreceives notification from one of its children,\n* `t:Membrane.RCMessage.Terminated.t/0` sent when the pipeline gracefully terminates.","ref":"Membrane.RCPipeline.html"},{"type":"function","title":"Membrane.RCPipeline.await_end_of_stream/1","doc":"Awaits for the first `t:Membrane.RCMessage.t/0` wrapping the `t:Membrane.RCMessage.EndOfStream.t/0` message\nwith no further constraints, sent by the process with `pipeline` pid.\nIt is required to firstly use the `subscribe/2` to subscribe to a given message before awaiting\nfor that message.\n\nUsage example:\n  1) awaiting for the first `end_of_stream` occuring on any pad of any element in the pipeline:\n  ```\n  Pipeline.await_end_of_stream(pipeline)\n  ```","ref":"Membrane.RCPipeline.html#await_end_of_stream/1"},{"type":"function","title":"Membrane.RCPipeline.await_end_of_stream/2","doc":"Awaits for the first `t:Membrane.RCMessage.t/0` wrapping the `t:Membrane.RCMessage.EndOfStream.t/0` message\nconcerning the given `element`, sent by the process with `pipeline` pid.\nIt is required to firstly use the `subscribe/2` to subscribe to a given message before awaiting\nfor that message.\n\nUsage example:\n  1) awaiting for the first `end_of_stream` occuring on any pad of the `:element_id` element in the pipeline:\n  ```\n  Pipeline.await_end_of_stream(pipeline, :element_id)\n  ```","ref":"Membrane.RCPipeline.html#await_end_of_stream/2"},{"type":"function","title":"Membrane.RCPipeline.await_end_of_stream/3","doc":"Awaits for the first `t:Membrane.RCMessage.t/0` wrapping the `t:Membrane.RCMessage.EndOfStream.t/0` message\nconcerning the given `element` and the `pad`, sent by the process with `pipeline` pid.\nIt is required to firstly use the `subscribe/2` to subscribe to a given message before awaiting\nfor that message.\n\nUsage example:\n  1) awaiting for the first `end_of_stream` occuring on the `:pad_id` of the `:element_id` element in the pipeline:\n  ```\n  Pipeline.await_end_of_stream(pipeline, :element_id, :pad_id)\n  ```","ref":"Membrane.RCPipeline.html#await_end_of_stream/3"},{"type":"function","title":"Membrane.RCPipeline.await_notification/1","doc":"Awaits for the first `t:Membrane.RCMessage.t/0` wrapping the `t:Membrane.RCMessage.Notification.t/0`\nmessage with no further constraints, sent by the process with `pipeline` pid.\nIt is required to firstly use the `subscribe/2` to subscribe to a given message before awaiting\nfor that message.\n\nUsage example:\n  1) awaiting for the first notification send to any element in the pipeline:\n  ```\n  Pipeline.await_notification(pipeline)\n  ```","ref":"Membrane.RCPipeline.html#await_notification/1"},{"type":"function","title":"Membrane.RCPipeline.await_notification/2","doc":"Awaits for the first `t:Membrane.RCMessage.t/0` wrapping the `t:Membrane.RCMessage.Notification.t/0` message\nconcerning the given `element`, sent by the process with `pipeline` pid.\nIt is required to firstly use the `subscribe/2` to subscribe to a given message before awaiting\nfor that message.\n\nUsage example:\n  1) awaiting for the first notification send to the `:element_id` element in the pipeline:\n  ```\n  Pipeline.await_notification(pipeline, :element_id)\n  ```","ref":"Membrane.RCPipeline.html#await_notification/2"},{"type":"function","title":"Membrane.RCPipeline.await_playing/1","doc":"Awaits for the first `t:Membrane.RCMessage.t/0` wrapping the `t:Membrane.RCMessage.Playing.t/0`\nIt is required to firstly use the `subscribe/2` to subscribe to a given message before awaiting\nfor that message.\n\nUsage example:\n  1) awaiting until the pipeline starts playing:\n  ```\n  Pipeline.await_playing(pipeline)\n  ```","ref":"Membrane.RCPipeline.html#await_playing/1"},{"type":"function","title":"Membrane.RCPipeline.await_start_of_stream/1","doc":"Awaits for the first `t:Membrane.RCMessage.t/0` wrapping the `t:Membrane.RCMessage.StartOfStream.t/0` message\nwith no further constraints, sent by the process with `pipeline` pid.\nIt is required to firstly use the `subscribe/2` to subscribe to a given message before awaiting\nfor that message.\n\nUsage example:\n  1) awaiting for the first `start_of_stream` occuring on any pad of any element in the pipeline:\n  ```\n  Pipeline.await_start_of_stream(pipeline)\n  ```","ref":"Membrane.RCPipeline.html#await_start_of_stream/1"},{"type":"function","title":"Membrane.RCPipeline.await_start_of_stream/2","doc":"Awaits for the first `t:Membrane.RCMessage.t/0` wrapping the `t:Membrane.RCMessage.StartOfStream.t/0` message\nconcerning the given `element`, sent by the process with `pipeline` pid.\nIt is required to firstly use the `subscribe/2` to subscribe to a given message before awaiting\nfor that message.\n\nUsage example:\n  1) awaiting for the first `start_of_stream` occuring on any pad of the `:element_id` element in the pipeline:\n  ```\n  Pipeline.await_start_of_stream(pipeline, :element_id)\n  ```","ref":"Membrane.RCPipeline.html#await_start_of_stream/2"},{"type":"function","title":"Membrane.RCPipeline.await_start_of_stream/3","doc":"Awaits for the first `t:Membrane.RCMessage.t/0` wrapping the `t:Membrane.RCMessage.StartOfStream.t/0` message\nconcerning the given `element` and the `pad`, sent by the process with `pipeline` pid.\nIt is required to firstly use the `subscribe/2` to subscribe to a given message before awaiting\nfor that message.\n\nUsage example:\n  1) awaiting for the first `start_of_stream` occuring on the `:pad_id` pad of the `:element_id` element in the pipeline:\n  ```\n  Pipeline.await_start_of_stream(pipeline, :element_id, :pad_id)\n  ```","ref":"Membrane.RCPipeline.html#await_start_of_stream/3"},{"type":"function","title":"Membrane.RCPipeline.await_termination/1","doc":"Awaits for the `t:Membrane.RCMessage.t/0` wrapping the `Membrane.RCMessage.Terminated` message,\nwhich is send when the pipeline gracefully terminates.\nIt is required to firstly use the `subscribe/2` to subscribe to a given message before awaiting\nfor that message.\n\nUsage example:\n  1) awaiting for the pipeline termination:\n  ```\n  Pipeline.await_termination(pipeline)\n  ```","ref":"Membrane.RCPipeline.html#await_termination/1"},{"type":"function","title":"Membrane.RCPipeline.child_spec/1","doc":"Returns child specification for spawning under a supervisor","ref":"Membrane.RCPipeline.html#child_spec/1"},{"type":"function","title":"Membrane.RCPipeline.exec_actions/2","doc":"Sends a list of `t:Pipeline.Action.t/0` to the given `Membrane.RCPipeline` for execution.\n\nUsage example:\n  1) making the `Membrane.RCPipeline` start the `Membrane.ChildrenSpec`\n     specified in the action.\n  ```\n  children = ...\n  links = ...\n  actions = [{:spec, children++links}]\n  Pipeline.exec_actions(pipeline, actions)\n  ```","ref":"Membrane.RCPipeline.html#exec_actions/2"},{"type":"function","title":"Membrane.RCPipeline.start/1","doc":"Does the same as the `start_link/1` but starts the process outside of the current supervision tree.","ref":"Membrane.RCPipeline.html#start/1"},{"type":"function","title":"Membrane.RCPipeline.start!/1","doc":"","ref":"Membrane.RCPipeline.html#start!/1"},{"type":"function","title":"Membrane.RCPipeline.start_link/1","doc":"Starts the `Membrane.RCPipeline` and links it to the current process. The process\nthat makes the call to the `start_link/1` automatically becomes the controller process.","ref":"Membrane.RCPipeline.html#start_link/1"},{"type":"function","title":"Membrane.RCPipeline.start_link!/1","doc":"","ref":"Membrane.RCPipeline.html#start_link!/1"},{"type":"macro","title":"Membrane.RCPipeline.subscribe/2","doc":"Subscribes to a given `subscription_pattern`. The `subscription_pattern` should describe some subset\nof elements of `t:Membrane.RCPipeline.Message.t/0` type. The `subscription_pattern`\nmust be a match pattern.\n\n\nUsage examples:\n1) making the `Membrane.RCPipeline` send to the controlling process `Message.StartOfStream` message\n  when any pad of the `:element_id` receives `:start_of_stream` event.\n\n  ```\n  subscribe(pipeline, %Message.StartOfStream{element: :element_id, pad: _})\n  ```\n\n2) making the `Membrane.RCPipeline` send to the controlling process `Message.Playing` message when the pipeline playback changes to `:playing`\n\n  ```\n  subscribe(pipeline, %Message.Playing{})\n  ```","ref":"Membrane.RCPipeline.html#subscribe/2"},{"type":"function","title":"Membrane.RCPipeline.terminate/2","doc":"","ref":"Membrane.RCPipeline.html#terminate/2"},{"type":"module","title":"Membrane.RCMessage.EndOfStream","doc":"Message sent when some element of the pipeline receives the start of stream event on some pad.","ref":"Membrane.RCMessage.EndOfStream.html"},{"type":"type","title":"Membrane.RCMessage.EndOfStream.t/0","doc":"Message sent when some element of the pipeline receives the start of stream event on some pad.","ref":"Membrane.RCMessage.EndOfStream.html#t:t/0"},{"type":"module","title":"Membrane.RCMessage.Notification","doc":"Message sent when the some element of the pipeline receives a notification.","ref":"Membrane.RCMessage.Notification.html"},{"type":"type","title":"Membrane.RCMessage.Notification.t/0","doc":"Message sent when the some element of the pipeline receives a notification.","ref":"Membrane.RCMessage.Notification.html#t:t/0"},{"type":"module","title":"Membrane.RCMessage.Playing","doc":"Message sent when the pipeline starts playing","ref":"Membrane.RCMessage.Playing.html"},{"type":"type","title":"Membrane.RCMessage.Playing.t/0","doc":"Message sent when the pipeline starts playing","ref":"Membrane.RCMessage.Playing.html#t:t/0"},{"type":"module","title":"Membrane.RCMessage.StartOfStream","doc":"Message sent when some element of the pipeline receives the start of stream event on some pad.","ref":"Membrane.RCMessage.StartOfStream.html"},{"type":"type","title":"Membrane.RCMessage.StartOfStream.t/0","doc":"Message sent when some element of the pipeline receives the start of stream event on some pad.","ref":"Membrane.RCMessage.StartOfStream.html#t:t/0"},{"type":"module","title":"Membrane.RCMessage.Terminated","doc":"Message sent when the pipeline gracefully terminates.","ref":"Membrane.RCMessage.Terminated.html"},{"type":"type","title":"Membrane.RCMessage.Terminated.t/0","doc":"Message sent when the pipeline gracefully terminates.","ref":"Membrane.RCMessage.Terminated.html#t:t/0"},{"type":"behaviour","title":"Membrane.Bin","doc":"Bins, similarly to pipelines, are containers for elements.\nHowever, at the same time, they can be placed and linked within pipelines.\nAlthough bin is a separate Membrane entity, it can be perceived as a pipeline within an element.\nBins can also be nested within one another.\n\nThere are two main reasons why bins are useful:\n* they enable creating reusable element groups\n* they allow managing their children, for instance by dynamically spawning or replacing them as the stream changes.\n\nIn order to create bin `use Membrane.Bin` in your callback module.","ref":"Membrane.Bin.html"},{"type":"macro","title":"Membrane.Bin.__using__/1","doc":"Brings all the stuff necessary to implement a bin.\n\nOptions:\n  - `:bring_spec?` - if true (default) imports and aliases `Membrane.ChildrenSpec`\n  - `:bring_pad?` - if true (default) requires and aliases `Membrane.Pad`","ref":"Membrane.Bin.html#__using__/1"},{"type":"function","title":"Membrane.Bin.bin?/1","doc":"Checks whether module is a bin.","ref":"Membrane.Bin.html#bin?/1"},{"type":"macro","title":"Membrane.Bin.def_clock/1","doc":"Defines that bin exposes a clock which is a proxy to one of its children.\n\nIf this macro is not called, no ticks will be forwarded to parent, regardless\nof clock definitions in its children.","ref":"Membrane.Bin.html#def_clock/1"},{"type":"macro","title":"Membrane.Bin.def_input_pad/2","doc":"Macro that defines input pad for the bin.\n\nIt automatically generates documentation from the given definition\nand adds compile-time stream format specs validation.\n\nThe type `t:Membrane.Pad.bin_spec/0` describes how the definition of pads should look.","ref":"Membrane.Bin.html#def_input_pad/2"},{"type":"macro","title":"Membrane.Bin.def_options/1","doc":"Macro defining options that parametrize bin.\n\nIt automatically generates appropriate struct and documentation.\n\nOptions are defined by a keyword list, where each key is an option name and\nis described by another keyword list with following fields:\n\n  * `spec:` typespec for value in struct\n  * `default:` default value for option. If not present, value for this option\n    will have to be provided each time options struct is created\n  * `inspector:` function converting fields' value to a string. Used when\n    creating documentation instead of `inspect/1`, eg. `inspector: &Membrane.Time.inspect/1`\n  * `description:` string describing an option. It will be used for generating the docs","ref":"Membrane.Bin.html#def_options/1"},{"type":"macro","title":"Membrane.Bin.def_output_pad/2","doc":"Macro that defines output pad for the bin.\n\nIt automatically generates documentation from the given definition\nand adds compile-time stream format specs validation.\n\nThe type `t:Membrane.Pad.bin_spec/0` describes how the definition of pads should look.","ref":"Membrane.Bin.html#def_output_pad/2"},{"type":"callback","title":"Membrane.Bin.handle_child_notification/4","doc":"Callback invoked when a notification comes in from an element.\nBy default, it ignores the received message.","ref":"Membrane.Bin.html#c:handle_child_notification/4"},{"type":"callback","title":"Membrane.Bin.handle_child_pad_removed/4","doc":"Callback invoked when a child removes its pad.\n\nThe callback won't be invoked, when you have initiated the pad removal,\neg. when you have returned `t:Membrane.Bin.Action.remove_link()` action\nwhich made one of your children's pads be removed.\nBy default, it does nothing.","ref":"Membrane.Bin.html#c:handle_child_pad_removed/4"},{"type":"callback","title":"Membrane.Bin.handle_child_playing/3","doc":"Callback invoked when a child enters `playing` playback.\n\nBy default, it does nothing.","ref":"Membrane.Bin.html#c:handle_child_playing/3"},{"type":"callback","title":"Membrane.Bin.handle_child_setup_completed/3","doc":"Callback invoked when a child complete its setup.\n\nBy default, it does nothing.","ref":"Membrane.Bin.html#c:handle_child_setup_completed/3"},{"type":"callback","title":"Membrane.Bin.handle_crash_group_down/3","doc":"Callback invoked when crash of the crash group happens.\n\nContext passed to this callback contains 2 additional fields: `:members` and `:crash_initiator`.","ref":"Membrane.Bin.html#c:handle_crash_group_down/3"},{"type":"callback","title":"Membrane.Bin.handle_element_end_of_stream/4","doc":"Callback invoked when a child element finishes processing stream via given pad.\n\nBy default, it does nothing.","ref":"Membrane.Bin.html#c:handle_element_end_of_stream/4"},{"type":"callback","title":"Membrane.Bin.handle_element_start_of_stream/4","doc":"Callback invoked when a child element starts processing stream via given pad.\nBy default, it does nothing.","ref":"Membrane.Bin.html#c:handle_element_start_of_stream/4"},{"type":"callback","title":"Membrane.Bin.handle_info/3","doc":"Callback invoked when bin receives a message that is not recognized\nas an internal membrane message.\n\nCan be used for receiving data from non-membrane processes.\nBy default, it logs and ignores the received message.","ref":"Membrane.Bin.html#c:handle_info/3"},{"type":"callback","title":"Membrane.Bin.handle_init/2","doc":"Callback invoked on initialization of bin.\n\nThis callback is synchronous: the parent waits until it finishes. Also, any failures\nthat happen in this callback crash the parent as well, regardless of crash groups.\nFor these reasons, it's important to do any long-lasting or complex work in `c:handle_setup/2`,\nwhile `handle_init` should be used for things like parsing options, initializing state or\nspawning children.\nBy default, it converts the opts struct to a map and sets them as the bin's state.","ref":"Membrane.Bin.html#c:handle_init/2"},{"type":"callback","title":"Membrane.Bin.handle_pad_added/3","doc":"Callback that is called when new pad has been added to bin. Executed\nONLY for dynamic pads.\n\nContext passed to this callback contains additional field `:pad_options`.\nBy default, it does nothing.","ref":"Membrane.Bin.html#c:handle_pad_added/3"},{"type":"callback","title":"Membrane.Bin.handle_pad_removed/3","doc":"Callback that is called when some pad of the bin has been removed. Executed\nONLY for dynamic pads.\n\nContext passed to this callback contains additional field `:pad_options`.\nBy default, it does nothing.","ref":"Membrane.Bin.html#c:handle_pad_removed/3"},{"type":"callback","title":"Membrane.Bin.handle_parent_notification/3","doc":"Callback invoked when a notification comes in from an parent.\nBy default, it ignores the received message.","ref":"Membrane.Bin.html#c:handle_parent_notification/3"},{"type":"callback","title":"Membrane.Bin.handle_playing/2","doc":"Callback invoked when bin switches the playback to `:playing`.\nBy default, it does nothing.","ref":"Membrane.Bin.html#c:handle_playing/2"},{"type":"callback","title":"Membrane.Bin.handle_setup/2","doc":"Callback invoked on bin startup, right after `c:handle_init/2`.\n\nAny long-lasting or complex initialization should happen here.\nBy default, it does nothing.","ref":"Membrane.Bin.html#c:handle_setup/2"},{"type":"callback","title":"Membrane.Bin.handle_spec_started/3","doc":"This callback is deprecated since v1.1.0-rc0\n\nCallback invoked when children of `Membrane.ChildrenSpec` are started.\n\nIt is invoked, only if pipeline module contains its definition. Otherwise, nothing happens.","ref":"Membrane.Bin.html#c:handle_spec_started/3"},{"type":"callback","title":"Membrane.Bin.handle_terminate_request/2","doc":"A callback invoked when the bin is being removed by its parent.\n\nBy default, it returns `t:Membrane.Bin.Action.terminate/0` with reason `:normal`.","ref":"Membrane.Bin.html#c:handle_terminate_request/2"},{"type":"callback","title":"Membrane.Bin.handle_tick/3","doc":"Callback invoked upon each timer tick. A timer can be started with `t:Membrane.Bin.Action.start_timer/0`\naction.","ref":"Membrane.Bin.html#c:handle_tick/3"},{"type":"macro","title":"Membrane.Bin.is_bin_name?/1","doc":"","ref":"Membrane.Bin.html#is_bin_name?/1"},{"type":"type","title":"Membrane.Bin.callback_return/0","doc":"","ref":"Membrane.Bin.html#t:callback_return/0"},{"type":"type","title":"Membrane.Bin.name/0","doc":"Type that defines a bin name by which it is identified.","ref":"Membrane.Bin.html#t:name/0"},{"type":"type","title":"Membrane.Bin.options/0","doc":"Defines options that can be passed to `start_link/3` and received\nin `c:handle_init/2` callback.","ref":"Membrane.Bin.html#t:options/0"},{"type":"type","title":"Membrane.Bin.state/0","doc":"","ref":"Membrane.Bin.html#t:state/0"},{"type":"module","title":"Membrane.Bin.Action","doc":"This module contains type specifications of actions that can be returned\nfrom bin callbacks.\n\nReturning actions is a way of bin interaction with\nother components and parts of framework. Each action may be returned by any\ncallback unless explicitly stated otherwise.","ref":"Membrane.Bin.Action.html"},{"type":"type","title":"Membrane.Bin.Action.notify_child/0","doc":"Action that sends a message to a child identified by name.","ref":"Membrane.Bin.Action.html#t:notify_child/0"},{"type":"type","title":"Membrane.Bin.Action.notify_parent/0","doc":"Sends a message to the parent.","ref":"Membrane.Bin.Action.html#t:notify_parent/0"},{"type":"type","title":"Membrane.Bin.Action.remove_children/0","doc":"Action that stops, unlinks and removes specified child/children from the bin.\n\nA child ref, list of children refs, children group id or a list of children group ids can be specified\nas an argument.","ref":"Membrane.Bin.Action.html#t:remove_children/0"},{"type":"type","title":"Membrane.Bin.Action.remove_link/0","doc":"Action that removes link, which relates to specified child and pad.\n\nRemoved link has to have dynamic pads on both ends.","ref":"Membrane.Bin.Action.html#t:remove_link/0"},{"type":"type","title":"Membrane.Bin.Action.setup/0","doc":"Action that manages the end of the component setup.\n\nBy default, component setup ends with the end of `c:Membrane.Bin.handle_setup/2` callback.\nIf `{:setup, :incomplete}` is returned there, setup lasts until `{:setup, :complete}`\nis returned from antoher callback. You cannot return `{:setup, :incomplete}` from any other\ncallback than `c:Membrane.Bin.handle_setup/2`.\n\nUntils the setup lasts, the component won't enter `:playing` playback.","ref":"Membrane.Bin.Action.html#t:setup/0"},{"type":"type","title":"Membrane.Bin.Action.spec/0","doc":"Action that instantiates children and links them according to `Membrane.ChildrenSpec`.\n\nChildren's playback is changed to the current bin playback.\n`c:Membrane.Parent.handle_spec_started/3` callback is executed once the children are spawned.","ref":"Membrane.Bin.Action.html#t:spec/0"},{"type":"type","title":"Membrane.Bin.Action.start_timer/0","doc":"Starts a timer that will invoke `c:Membrane.Bin.handle_tick/3` callback\nevery `interval` according to the given `clock`.\n\nThe timer's `id` is passed to the `c:Membrane.Bin.handle_tick/3`\ncallback and can be used for changing its interval via `t:timer_interval/0`\nor stopping it via `t:stop_timer/0`.\n\nIf `interval` is set to `:no_interval`, the timer won't issue any ticks until\nthe interval is set with `t:timer_interval/0` action.\n\nIf no `clock` is passed, parent clock is chosen.\n\nTimers use `Process.send_after/3` under the hood.","ref":"Membrane.Bin.Action.html#t:start_timer/0"},{"type":"type","title":"Membrane.Bin.Action.stop_timer/0","doc":"Stops a timer started with `t:start_timer/0` action.\n\nThis action is atomic: stopping timer guarantees that no ticks will arrive from it.","ref":"Membrane.Bin.Action.html#t:stop_timer/0"},{"type":"type","title":"Membrane.Bin.Action.t/0","doc":"Type describing actions that can be returned from bin callbacks.\n\nReturning actions is a way of bin interaction with its children and\nother parts of framework.","ref":"Membrane.Bin.Action.html#t:t/0"},{"type":"type","title":"Membrane.Bin.Action.terminate/0","doc":"Terminates bin with given reason.\n\nTermination reason follows the OTP semantics:\n- Use `:normal` for graceful termination. Allowed only when the parent already requested termination,\ni.e. after `c:Membrane.Bin.handle_terminate_request/2` is called. If the bin has no children, it\nterminates immediately. Otherwise, it switches to the zombie mode, requests all the children to terminate,\nwaits for them to terminate and then terminates itself. In the zombie mode, no bin callbacks\nare called and all messages and calls to the bin are ignored (apart from Membrane internal\nmessages)\n- If the reason is other than `:normal`, the bin terminates immediately. The bin supervisor\nterminates all the children with the reason `:shutdown`\n- If the reason is neither `:normal`, `:shutdown` nor `{:shutdown, term}`, an error is logged","ref":"Membrane.Bin.Action.html#t:terminate/0"},{"type":"type","title":"Membrane.Bin.Action.timer_interval/0","doc":"Changes interval of a timer started with `t:start_timer/0`.\n\nPermitted only from `c:Membrane.Bin.handle_tick/3`, unless the interval\nwas previously set to `:no_interval`.\n\nIf the `interval` is `:no_interval`, the timer won't issue any ticks until\nanother `t:timer_interval/0` action. Otherwise, the timer will issue ticks every\nnew `interval`. The next tick after interval change is scheduled at\n`new_interval + previous_time`, where previous_time is the time of the latest\ntick or the time of returning `t:start_timer/0` action if no tick has been\nsent yet. Note that if `current_time - previous_time > new_interval`, a burst\nof `div(current_time - previous_time, new_interval)` ticks is issued immediately.","ref":"Membrane.Bin.Action.html#t:timer_interval/0"},{"type":"module","title":"Membrane.Bin.CallbackContext","doc":"Module describing context passed to the `Membrane.Bin` callbacks.","ref":"Membrane.Bin.CallbackContext.html"},{"type":"type","title":"Membrane.Bin.CallbackContext.t/0","doc":"Type describing context passed to the `Membrane.Bin` callbacks.\n\nField `:pad_options` is present only in `c:Membrane.Bin.handle_pad_added/3`\nand `c:Membrane.Bin.handle_pad_removed/3`.\n\nField `:start_of_stream_received?` is present only in\n`c:Membrane.Bin.handle_element_end_of_stream/4`.\n\nFields `:members`, `:crash_initiator` and `crash_reason` and  are present only in\n`c:Membrane.Bin.handle_crash_group_down/3`.","ref":"Membrane.Bin.CallbackContext.html#t:t/0"},{"type":"module","title":"Membrane.Bin.PadData","doc":"Struct describing current pad state.\n\nThe public fields are:\n  - `:availability` - see `t:Membrane.Pad.availability/0`\n  - `:direction` - see `t:Membrane.Pad.direction/0`\n  - `:name` - see `t:Membrane.Pad.name/0`. Do not mistake with `:ref`\n  - `:options` - options passed in `Membrane.ChildrenSpec` when linking pad\n  - `:ref` - see `t:Membrane.Pad.ref/0`\n\nOther fields in the struct ARE NOT PART OF THE PUBLIC API and should not be\naccessed or relied on.","ref":"Membrane.Bin.PadData.html"},{"type":"type","title":"Membrane.Bin.PadData.private_field/0","doc":"","ref":"Membrane.Bin.PadData.html#t:private_field/0"},{"type":"type","title":"Membrane.Bin.PadData.t/0","doc":"Struct describing current pad state.\n\nThe public fields are:\n  - `:availability` - see `t:Membrane.Pad.availability/0`\n  - `:direction` - see `t:Membrane.Pad.direction/0`\n  - `:name` - see `t:Membrane.Pad.name/0`. Do not mistake with `:ref`\n  - `:options` - options passed in `Membrane.ChildrenSpec` when linking pad\n  - `:ref` - see `t:Membrane.Pad.ref/0`\n\nOther fields in the struct ARE NOT PART OF THE PUBLIC API and should not be\naccessed or relied on.","ref":"Membrane.Bin.PadData.html#t:t/0"},{"type":"module","title":"Membrane.Element","doc":"Module containing types and functions for operating on elements.\n\nFor behaviours for elements check `Membrane.Source`, `Membrane.Filter`,\n`Membrane.Endpoint` and `Membrane.Sink`.","ref":"Membrane.Element.html"},{"type":"module","title":"Behaviours - Membrane.Element","doc":"Element-specific behaviours are specified in modules:\n- `Membrane.Element.WithOutputPads` - behaviour common to sources,\nfilters and endpoints\n- `Membrane.Element.WithInputPads` - behaviour common to sinks,\nfilters and endpoints\n- Base modules (`Membrane.Source`, `Membrane.Filter`, `Membrane.Endpoint`,\n`Membrane.Sink`) - behaviours specific to each element type.","ref":"Membrane.Element.html#module-behaviours"},{"type":"module","title":"Callbacks - Membrane.Element","doc":"Modules listed above provide specifications of callbacks that define elements\nlifecycle. All of these callbacks have names with the `handle_` prefix.\nThey are used to define reaction to certain events that happen during runtime,\nand indicate what actions framework should undertake as a result, besides\nexecuting element-specific code.\n\nFor actions that can be returned by each callback, see `Membrane.Element.Action`\nmodule.","ref":"Membrane.Element.html#module-callbacks"},{"type":"function","title":"Membrane.Element.element?/1","doc":"Checks whether module is an element.","ref":"Membrane.Element.html#element?/1"},{"type":"macro","title":"Membrane.Element.is_element_name?/1","doc":"","ref":"Membrane.Element.html#is_element_name?/1"},{"type":"type","title":"Membrane.Element.name/0","doc":"Type that defines an element name by which it is identified.","ref":"Membrane.Element.html#t:name/0"},{"type":"type","title":"Membrane.Element.options/0","doc":"Defines options that can be received in `c:Membrane.Element.Base.handle_init/2`\ncallback.","ref":"Membrane.Element.html#t:options/0"},{"type":"type","title":"Membrane.Element.state/0","doc":"Type of user-managed state of element.","ref":"Membrane.Element.html#t:state/0"},{"type":"type","title":"Membrane.Element.type/0","doc":"Defines possible element types:\n- source, producing buffers\n- filter, processing buffers\n- endpoint, producing and consuming buffers\n- sink, consuming buffers","ref":"Membrane.Element.html#t:type/0"},{"type":"module","title":"Membrane.Endpoint","doc":"Module defining behaviour for endpoints - elements consuming and producing data.\n\nBehaviours for endpoints are specified, besides this place, in modules\n`Membrane.Element.Base`,\n`Membrane.Element.WithOutputPads`,\nand `Membrane.Element.WithInputPads`.\n\nEndpoint can have both input and output pads. Job of usual endpoint is both, to\nreceive some data on such pad and consume it (write to a soundcard, send through\nTCP etc.) and to produce some data (read from soundcard, download through HTTP,\netc.) and send it through such pad. If the pad has the flow control set to\n`:manual`, then endpoint is also responsible for receiving demands on the output\npad and requesting them on the input pad (for more details, see\n`c:Membrane.Element.WithOutputPads.handle_demand/5` callback).\nEndpoints, like all elements, can of course have multiple pads if needed to\nprovide more complex solutions.","ref":"Membrane.Endpoint.html"},{"type":"module","title":"List of available callbacks - Membrane.Endpoint","doc":"Below there is a list of all the callbacks available in a module, that implements `Membrane.Endpoint` behaviour.\nWe have put it for your convenience, as some of these callbacks aren't directly defined in that module and\ntheir specification is available in different modules.\n\nThe callbacks available in `Membrane.Endpoint` behaviour:\n\n`Membrane.Element.Base`\n* `c:Membrane.Element.Base.__struct__/0`\n* `c:Membrane.Element.Base.__struct__/1`\n* `c:Membrane.Element.Base.handle_event/4`\n* `c:Membrane.Element.Base.handle_info/3`\n* `c:Membrane.Element.Base.handle_init/2`\n* `c:Membrane.Element.Base.handle_pad_added/3`\n* `c:Membrane.Element.Base.handle_pad_removed/3`\n* `c:Membrane.Element.Base.handle_parent_notification/3`\n* `c:Membrane.Element.Base.handle_playing/2`\n* `c:Membrane.Element.Base.handle_setup/2`\n* `c:Membrane.Element.Base.handle_terminate_request/2`\n* `c:Membrane.Element.Base.handle_tick/3`\n\n`Membrane.Element.WithInputPads`\n* `c:Membrane.Element.WithInputPads.handle_buffer/4`\n* `c:Membrane.Element.WithInputPads.handle_end_of_stream/3`\n* `c:Membrane.Element.WithInputPads.handle_start_of_stream/3`\n* `c:Membrane.Element.WithInputPads.handle_stream_format/4`\n\n`Membrane.Element.WithOutputPads`\n* `c:Membrane.Element.WithOutputPads.handle_demand/5`","ref":"Membrane.Endpoint.html#module-list-of-available-callbacks"},{"type":"macro","title":"Membrane.Endpoint.__using__/1","doc":"Brings all the stuff necessary to implement a endpoint element.\n\nOptions:\n  - `:bring_pad?` - if true (default) requires and aliases `Membrane.Pad`","ref":"Membrane.Endpoint.html#__using__/1"},{"type":"module","title":"Membrane.Filter","doc":"Module defining behaviour for filters - elements processing data.\n\nBehaviours for filters are specified, besides this place, in modules\n`Membrane.Element.Base`,\n`Membrane.Element.WithOutputPads`,\nand `Membrane.Element.WithInputPads`.\n\nFilters can have both input and output pads. Job of a usual filter is to\nreceive some data on a input pad, process the data and send it through the\noutput pad. If the pad has the flow control set to `:manual`, then filter\nis also responsible for receiving demands on the output pad and requesting\nthem on the input pad (for more details, see\n`c:Membrane.Element.WithOutputPads.handle_demand/5` callback).\nFilters, like all elements, can of course have multiple pads if needed to\nprovide more complex solutions.","ref":"Membrane.Filter.html"},{"type":"module","title":"List of available callbacks - Membrane.Filter","doc":"Below there is a list of all the callbacks available in a module, that implements `Membrane.Filter` behaviour.\nWe have put it for your convenience, as some of these callbacks aren't directly defined in that module and\ntheir specification is available in different modules.\n\nThe callbacks available in `Membrane.Filter` behaviour:\n\n`Membrane.Element.Base`\n* `c:Membrane.Element.Base.__struct__/0`\n* `c:Membrane.Element.Base.__struct__/1`\n* `c:Membrane.Element.Base.handle_event/4`\n* `c:Membrane.Element.Base.handle_info/3`\n* `c:Membrane.Element.Base.handle_init/2`\n* `c:Membrane.Element.Base.handle_pad_added/3`\n* `c:Membrane.Element.Base.handle_pad_removed/3`\n* `c:Membrane.Element.Base.handle_parent_notification/3`\n* `c:Membrane.Element.Base.handle_playing/2`\n* `c:Membrane.Element.Base.handle_setup/2`\n* `c:Membrane.Element.Base.handle_terminate_request/2`\n* `c:Membrane.Element.Base.handle_tick/3`\n\n`Membrane.Element.WithInputPads`\n* `c:Membrane.Element.WithInputPads.handle_buffer/4`\n* `c:Membrane.Element.WithInputPads.handle_end_of_stream/3`\n* `c:Membrane.Element.WithInputPads.handle_start_of_stream/3`\n* `c:Membrane.Element.WithInputPads.handle_stream_format/4`\n\n`Membrane.Element.WithOutputPads`\n* `c:Membrane.Element.WithOutputPads.handle_demand/5`","ref":"Membrane.Filter.html#module-list-of-available-callbacks"},{"type":"macro","title":"Membrane.Filter.__using__/1","doc":"Brings all the stuff necessary to implement a filter element.\n\nOptions:\n  - `:bring_pad?` - if true (default) requires and aliases `Membrane.Pad`","ref":"Membrane.Filter.html#__using__/1"},{"type":"module","title":"Membrane.FilterAggregator","doc":"An element allowing to aggregate many filters within one Elixir process.\n\nWarning: This element is still in experimental phase\n\nThis element supports only filters with one input and one output\nwith following restrictions:\n* not using timers\n* not relying on received messages\n* not expecting any events coming from downstream elements\n* their pads have to be named `:input` and `:output`\n* their pads cannot use manual demands\n* the first filter must make demands in buffers","ref":"Membrane.FilterAggregator.html"},{"type":"module","title":"Element options - Membrane.FilterAggregator","doc":"Passed via struct `t:Membrane.FilterAggregator.t/0`\n\n- `filters`  \n\n  ```\n  [{Membrane.Child.name(), module() | struct()}]\n  ```\n  \n  ***Required***  \n  A list of filters applied to incoming stream","ref":"Membrane.FilterAggregator.html#module-element-options"},{"type":"module","title":"Pads - Membrane.FilterAggregator","doc":"### `:input`\n\nAccepted formats:\n```\n_any\n```\n\nDirection: | `:input`\nAvailability: | `:always`\nFlow control: | `:auto`\n\n\n### `:output`\n\nAccepted formats:\n```\n_any\n```\n\nDirection: | `:output`\nAvailability: | `:always`\nFlow control: | `:auto`","ref":"Membrane.FilterAggregator.html#module-pads"},{"type":"function","title":"Membrane.FilterAggregator.options/0","doc":"Returns description of options available for this module","ref":"Membrane.FilterAggregator.html#options/0"},{"type":"type","title":"Membrane.FilterAggregator.t/0","doc":"Struct containing options for `Membrane.FilterAggregator`","ref":"Membrane.FilterAggregator.html#t:t/0"},{"type":"module","title":"Membrane.Sink","doc":"Module defining behaviour for sinks - elements consuming data.\n\nBehaviours for sinks are specified, besides this place, in modules\n`Membrane.Element.Base`,\nand `Membrane.Element.WithInputPads`.\n\nSink elements can define only input pads. Job of a usual sink is to receive some\ndata on such pad and consume it (write to a soundcard, send through TCP etc.).\nIf the pad has the flow control set to `:manual`, then element is also responsible\nfor requesting demands when it is able and willing to consume data (for more details,\nsee `t:Membrane.Element.Action.demand/0`). Sinks, like all elements, can of course\nhave multiple pads if needed to provide more complex solutions.","ref":"Membrane.Sink.html"},{"type":"module","title":"List of available callbacks - Membrane.Sink","doc":"Below there is a list of all the callbacks available in a module, that implements `Membrane.Sink` behaviour.\nWe have put it for your convenience, as some of these callbacks aren't directly defined in that module and\ntheir specification is available in different modules.\n\nThe callbacks available in `Membrane.Sink` behaviour:\n\n`Membrane.Element.Base`\n* `c:Membrane.Element.Base.__struct__/0`\n* `c:Membrane.Element.Base.__struct__/1`\n* `c:Membrane.Element.Base.handle_event/4`\n* `c:Membrane.Element.Base.handle_info/3`\n* `c:Membrane.Element.Base.handle_init/2`\n* `c:Membrane.Element.Base.handle_pad_added/3`\n* `c:Membrane.Element.Base.handle_pad_removed/3`\n* `c:Membrane.Element.Base.handle_parent_notification/3`\n* `c:Membrane.Element.Base.handle_playing/2`\n* `c:Membrane.Element.Base.handle_setup/2`\n* `c:Membrane.Element.Base.handle_terminate_request/2`\n* `c:Membrane.Element.Base.handle_tick/3`\n\n`Membrane.Element.WithInputPads`\n* `c:Membrane.Element.WithInputPads.handle_buffer/4`\n* `c:Membrane.Element.WithInputPads.handle_end_of_stream/3`\n* `c:Membrane.Element.WithInputPads.handle_start_of_stream/3`\n* `c:Membrane.Element.WithInputPads.handle_stream_format/4`","ref":"Membrane.Sink.html#module-list-of-available-callbacks"},{"type":"macro","title":"Membrane.Sink.__using__/1","doc":"Brings all the stuff necessary to implement a sink element.\n\nOptions:\n  - `:bring_pad?` - if true (default) requires and aliases `Membrane.Pad`","ref":"Membrane.Sink.html#__using__/1"},{"type":"module","title":"Membrane.Source","doc":"Module that should be used in sources - elements producing data. Declares\nappropriate behaviours implementation and provides default callbacks implementation.\n\nBehaviours for sources are specified in modules\n`Membrane.Element.Base` and\n`Membrane.Element.WithOutputPads`.\n\nSource elements can define only output pads. Job of a usual source is to produce\nsome data (read from soundcard, download through HTTP, etc.) and send it through\nsuch pad. If the pad has the flow control set to `:manual`, then element is also\nresponsible for receiving demands and send buffers only if they have previously\nbeen demanded (for more details, see `c:Membrane.Element.WithOutputPads.handle_demand/5`\ncallback). Sources, like all elements, can of course have multiple pads if needed to\nprovide more complex solutions.","ref":"Membrane.Source.html"},{"type":"module","title":"List of available callbacks - Membrane.Source","doc":"Below there is a list of all the callbacks available in a module, that implements `Membrane.Source` behaviour.\nWe have put it for your convenience, as some of these callbacks aren't directly defined in that module and\ntheir specification is available in different modules.\n\nThe callbacks available in `Membrane.Source` behaviour:\n\n`Membrane.Element.Base`\n* `c:Membrane.Element.Base.__struct__/0`\n* `c:Membrane.Element.Base.__struct__/1`\n* `c:Membrane.Element.Base.handle_event/4`\n* `c:Membrane.Element.Base.handle_info/3`\n* `c:Membrane.Element.Base.handle_init/2`\n* `c:Membrane.Element.Base.handle_pad_added/3`\n* `c:Membrane.Element.Base.handle_pad_removed/3`\n* `c:Membrane.Element.Base.handle_parent_notification/3`\n* `c:Membrane.Element.Base.handle_playing/2`\n* `c:Membrane.Element.Base.handle_setup/2`\n* `c:Membrane.Element.Base.handle_terminate_request/2`\n* `c:Membrane.Element.Base.handle_tick/3`\n\n`Membrane.Element.WithOutputPads`\n* `c:Membrane.Element.WithOutputPads.handle_demand/5`","ref":"Membrane.Source.html#module-list-of-available-callbacks"},{"type":"macro","title":"Membrane.Source.__using__/1","doc":"Brings all the stuff necessary to implement a source element.\n\nOptions:\n  - `:bring_pad?` - if true (default) requires and aliases `Membrane.Pad`","ref":"Membrane.Source.html#__using__/1"},{"type":"module","title":"Membrane.Element.Action","doc":"This module contains type specifications of actions that can be returned\nfrom element callbacks.\n\nReturning actions is a way of element interaction with\nother elements and parts of framework. Each action may be returned by any\ncallback unless explicitly stated otherwise.","ref":"Membrane.Element.Action.html"},{"type":"type","title":"Membrane.Element.Action.buffer/0","doc":"Sends buffers through a pad.\n\nThe pad must have output direction.\n\nAllowed only when playback is playing.","ref":"Membrane.Element.Action.html#t:buffer/0"},{"type":"type","title":"Membrane.Element.Action.common_actions/0","doc":"Action that can be always returned from each of the callbacks.","ref":"Membrane.Element.Action.html#t:common_actions/0"},{"type":"type","title":"Membrane.Element.Action.demand/0","doc":"Makes a demand on a pad.\n\nThe pad must have input direction and work in `:manual` flow control mode. This action does NOT\nentail _sending_ demand through the pad, but just _requesting_ some amount\nof data from pad's internal queue, which _sends_ demands automatically when it\nruns out of data.\nIf there is any data available at the pad, the data is passed to\n`c:Membrane.WithInputPads.handle_buffer/4` callback. Invoked callback is\nguaranteed not to receive more data than demanded.\n\nDemand size can be either a non-negative integer, that overrides existing demand,\nor a function that is passed current demand, and is to return the new demand. In case only pad\nis specified, the demand size defaults to 1.\n\nAllowed only when playback is playing.","ref":"Membrane.Element.Action.html#t:demand/0"},{"type":"type","title":"Membrane.Element.Action.demand_size/0","doc":"","ref":"Membrane.Element.Action.html#t:demand_size/0"},{"type":"type","title":"Membrane.Element.Action.end_of_stream/0","doc":"Marks that processing via a pad (output) has been finished and the pad instance\nwon't be used anymore.\n\nTriggers `end_of_stream/3` callback at the receiver element.\nAllowed only when playback is in playing state.","ref":"Membrane.Element.Action.html#t:end_of_stream/0"},{"type":"type","title":"Membrane.Element.Action.event/0","doc":"Sends an event through a pad (input or output).\n\nAllowed only when playback is `playing`.","ref":"Membrane.Element.Action.html#t:event/0"},{"type":"type","title":"Membrane.Element.Action.forward/0","doc":"Sends buffers/stream format/event/end of stream to all output pads of element (or to input\npads when event occurs on the output pad).\n\nUsed by default implementations of\n`c:Membrane.Element.WithInputPads.handle_stream_format/4`,\n`c:Membrane.Element.Base.handle_event/4` and\n`c:Membrane.Element.WithInputPads.handle_end_of_stream/3` callbacks in filter.\n\nAllowed only when _all_ below conditions are met:\n- element is filter,\n- callback is `c:Membrane.Element.WithInputPads.handle_buffer/4`,\n`c:Membrane.Element.WithInputPads.handle_stream_format/4`,\n`c:Membrane.Element.Base.handle_event/4` or `c:Membrane.Element.WithInputPads.handle_end_of_stream/3`\n- playback is `playing`\n\nKeep in mind that `c:Membrane.WithInputPads.handle_buffer/4` can only\nforward buffers, `c:Membrane.Element.WithInputPads.handle_stream_format/4` - stream formats.\n`c:Membrane.Element.Base.handle_event/4` - events and\n`c:Membrane.Element.WithInputPads.handle_end_of_stream/3` - ends of streams.","ref":"Membrane.Element.Action.html#t:forward/0"},{"type":"type","title":"Membrane.Element.Action.latency/0","doc":"This action sets the latency for the element.\n\nThis action is permitted only in callback `c:Membrane.Element.Base.handle_init/2`.","ref":"Membrane.Element.Action.html#t:latency/0"},{"type":"type","title":"Membrane.Element.Action.notify_parent/0","doc":"Sends a message to the parent.","ref":"Membrane.Element.Action.html#t:notify_parent/0"},{"type":"type","title":"Membrane.Element.Action.pause_auto_demand/0","doc":"Pauses auto-demanding on the specific pad.\n\nThe pad must have input direction and work in `:auto` flow control mode.\n\nThis action does not guarantee that no more buffers will arrive on the specific pad,\nbut ensures, that demand on this pad will not increase until returning\n`Membrane.Element.Action.resume_auto_demand()` action. Number of buffers, that will\narrive on the pad, depends on the behaviour of the elements earlier in the pipeline.\n\nWhen auto-demanding is already paused, this action has no effect.","ref":"Membrane.Element.Action.html#t:pause_auto_demand/0"},{"type":"type","title":"Membrane.Element.Action.redemand/0","doc":"Executes `c:Membrane.Element.WithOutputPads.handle_demand/5` callback\nfor the given pad (or pads), that have demand greater than 0.\n\nThe pad must have output direction and work in pull mode.","ref":"Membrane.Element.Action.html#t:redemand/0"},{"type":"type","title":"Redemand in Sources and Endpoints - Membrane.Element.Action.redemand/0","doc":"In case of Sources and Endpoints, `:redemand` is just a helper that simplifies element's code.\nThe element doesn't need to generate the whole demand synchronously at `handle_demand`\nor store current demand size in its state, but it can just generate one buffer\nand return `:redemand` action.\nIf there is still one or more buffers to produce, returning `:redemand` triggers\nthe next invocation of `handle_demand`. In such case, the element is to produce\nnext buffer and call `:redemand` again.\nIf there are no more buffers demanded, `handle_demand` is not invoked and\nthe loop ends.\nOne more advantage of the approach with `:redemand` action is that produced buffers\nare sent one after another in separate messages and this can possibly improve\nthe latency.","ref":"Membrane.Element.Action.html#t:redemand/0-redemand-in-sources-and-endpoints"},{"type":"type","title":"Redemand in Filters - Membrane.Element.Action.redemand/0","doc":"Redemand in Filters is useful in a situation where not the entire demand of\noutput pad has been satisfied and there is a need to send a demand for additional\nbuffers through the input pad.\nA typical example of this situation is a parser that has not demanded enough\nbytes to parse the whole frame.","ref":"Membrane.Element.Action.html#t:redemand/0-redemand-in-filters"},{"type":"type","title":"Usage limitations - Membrane.Element.Action.redemand/0","doc":"Allowed only when playback is playing.","ref":"Membrane.Element.Action.html#t:redemand/0-usage-limitations"},{"type":"type","title":"Membrane.Element.Action.resume_auto_demand/0","doc":"Resumes auto-demanding on the specific pad.\n\nThe pad must have input direction and work in `:auto` flow control mode.\n\nThis action reverts the effects of `Membrane.Element.Action.pause_auto_demand()` action.\n\nWhen auto demanding is not paused, this action has no effect.","ref":"Membrane.Element.Action.html#t:resume_auto_demand/0"},{"type":"type","title":"Membrane.Element.Action.setup/0","doc":"Action that manages the end of the component setup.\n\nBy default, component setup ends with the end of `c:Membrane.Element.Base.handle_setup/2` callback.\nIf `{:setup, :incomplete}` is returned there, setup lasts until `{:setup, :complete}`\nis returned from antoher callback.\n\nUntils the setup lasts, the component won't enter `:playing` playback.","ref":"Membrane.Element.Action.html#t:setup/0"},{"type":"type","title":"Membrane.Element.Action.split/0","doc":"Allows to split callback execution into multiple applications of another callback\n(called from now sub-callback).\n\nExecutions are synchronous in the element process, and each of them passes\nsubsequent arguments from the args_list, along with the element state (passed\nas the last argument each time).\n\nReturn value of each execution of sub-callback can be any valid return value\nof the original callback (this also means sub-callback can return any action\nvalid for the original callback, unless explicitly stated). Returned actions\nare executed immediately (they are NOT accumulated and executed after all\nsub-callback executions are finished).\n\nUseful when a long action is to be undertaken, and partial results need to\nbe returned before entire process finishes.","ref":"Membrane.Element.Action.html#t:split/0"},{"type":"type","title":"Membrane.Element.Action.start_timer/0","doc":"Starts a timer that will invoke `c:Membrane.Element.Base.handle_tick/3` callback\nevery `interval` according to the given `clock`.\n\nThe timer's `id` is passed to the `c:Membrane.Element.Base.handle_tick/3`\ncallback and can be used for changing its interval via `t:timer_interval/0`\nor stopping it via `t:stop_timer/0`.\n\nIf `interval` is set to `:no_interval`, the timer won't issue any ticks until\nthe interval is set with `t:timer_interval/0` action.\n\nIf no `clock` is passed, parent's clock is chosen.\n\nTimers use `Process.send_after/3` under the hood.","ref":"Membrane.Element.Action.html#t:start_timer/0"},{"type":"type","title":"Membrane.Element.Action.stop_timer/0","doc":"Stops a timer started with `t:start_timer/0` action.\n\nThis action is atomic: stopping timer guarantees that no ticks will arrive from it.","ref":"Membrane.Element.Action.html#t:stop_timer/0"},{"type":"type","title":"Membrane.Element.Action.stream_actions/0","doc":"Actions that can be returned from callbacks when the element is in `playback: :playing` state.","ref":"Membrane.Element.Action.html#t:stream_actions/0"},{"type":"type","title":"Membrane.Element.Action.stream_format/0","doc":"Sends stream format through a pad.\n\nThe pad must have output direction. Sent stream format must fit constraints on the pad.\n\nAllowed only when playback is `playing`.","ref":"Membrane.Element.Action.html#t:stream_format/0"},{"type":"type","title":"Membrane.Element.Action.t/0","doc":"Type that defines a union of actions that may be returned from most of the callbacks.\n\nDepending on element type, callback, current playback and other\ncircumstances there may be different actions available.\nCheck the typespec and documentation of particular callbacks\nfor details.","ref":"Membrane.Element.Action.html#t:t/0"},{"type":"type","title":"Membrane.Element.Action.terminate/0","doc":"Terminates element with given reason.\n\nTermination reason follows the OTP semantics:\n- Use `:normal` for graceful termination. Allowed only when the parent already requested termination,\n  i.e. after `c:Membrane.Element.Base.handle_terminate_request/2` is called\n- If reason is neither `:normal`, `:shutdown` nor `{:shutdown, term}`, an error is logged","ref":"Membrane.Element.Action.html#t:terminate/0"},{"type":"type","title":"Membrane.Element.Action.timer_interval/0","doc":"Changes interval of a timer started with `t:start_timer/0`.\n\nPermitted only from `c:Membrane.Element.Base.handle_tick/3`, unless the interval\nwas previously set to `:no_interval`.\n\nIf the `interval` is `:no_interval`, the timer won't issue any ticks until\nanother `t:timer_interval/0` action. Otherwise, the timer will issue ticks every\nnew `interval`. The next tick after interval change is scheduled at\n`new_interval + previous_time`, where previous_time is the time of the latest\ntick or the time of returning `t:start_timer/0` action if no tick has been\nsent yet. Note that if `current_time - previous_time > new_interval`, a burst\nof `div(current_time - previous_time, new_interval)` ticks is issued immediately.","ref":"Membrane.Element.Action.html#t:timer_interval/0"},{"type":"behaviour","title":"Membrane.Element.Base","doc":"Module defining behaviour common to all elements.\n\nWhen used declares behaviour implementation, provides default callback definitions\nand imports macros.\n\n# Elements\n\nElements are units that produce, process or consume data. They can be linked\nwith `Membrane.Pipeline`, and thus form a pipeline able to perform complex data\nprocessing. Each element defines a set of pads, through which it can be linked\nwith other elements. During playback, pads can either send (output pads) or\nreceive (input pads) data. For more information on pads, see\n`Membrane.Pad`.\n\nNote: This module (`Membrane.Element.Base`) should not be `use`d directly.\n\nTo implement an element, one of the following base modules (`Membrane.Source`,\n`Membrane.Filter`, `Membrane.Endpoint` or `Membrane.Sink`)\nhas to be `use`d, depending on the element type:\n- source, producing buffers (contain only output pads),\n- filter, processing buffers (contain both input and output pads),\n- endpoint, producing and consuming buffers (contain both input and output pads),\n- sink, consuming buffers (contain only input pads).\nFor more information on each element type, check documentation for appropriate\nbase module.","ref":"Membrane.Element.Base.html"},{"type":"callback","title":"Membrane.Element.Base.__struct__/0","doc":"A callback for constructing struct. Will be defined by `def_options/1` if used.\n\nSee `defstruct/1` for a more in-depth description.","ref":"Membrane.Element.Base.html#c:__struct__/0"},{"type":"callback","title":"Membrane.Element.Base.__struct__/1","doc":"A callback for constructing struct with values. Will be defined by `def_options/1` if used.\n\nSee `defstruct/1` for a more in-depth description.","ref":"Membrane.Element.Base.html#c:__struct__/1"},{"type":"macro","title":"Membrane.Element.Base.__using__/1","doc":"Brings common stuff needed to implement an element. Used by\n`Membrane.Source.__using__/1`, `Membrane.Filter.__using__/1`,\n`Membrane.Endpoint.__using__/1` and `Membrane.Sink.__using__/1`.\n\nOptions:\n  - `:bring_pad?` - if true (default) requires and aliases `Membrane.Pad`","ref":"Membrane.Element.Base.html#__using__/1"},{"type":"macro","title":"Membrane.Element.Base.def_clock/1","doc":"Defines that element exports a clock to pipeline.\n\nExporting clock allows pipeline to choose it as the pipeline clock, enabling other\nelements to synchronize with it. Element's clock is accessible via `clock` field,\nwhile pipeline's one - via `parent_clock` field in callback contexts. Both of\nthem can be used for starting timers.","ref":"Membrane.Element.Base.html#def_clock/1"},{"type":"macro","title":"Membrane.Element.Base.def_options/1","doc":"Macro defining options that parametrize element.\n\nIt automatically generates appropriate struct and documentation.\n\nOptions are defined by a keyword list, where each key is an option name and\nis described by another keyword list with following fields:\n\n  * `spec:` typespec for value in struct\n  * `default:` default value for option. If not present, value for this option\n    will have to be provided each time options struct is created\n  * `inspector:` function converting fields' value to a string. Used when\n    creating documentation instead of `inspect/1`, eg. `inspector: &Membrane.Time.inspect/1`\n  * `description:` string describing an option. It will be used for generating the docs","ref":"Membrane.Element.Base.html#def_options/1"},{"type":"callback","title":"Membrane.Element.Base.handle_event/4","doc":"Callback that is called when event arrives.\n\nEvents may arrive from both input and output pads. In filters by default event is\nforwarded to all output and input pads, respectively.\nBy default, it ignores received event.","ref":"Membrane.Element.Base.html#c:handle_event/4"},{"type":"callback","title":"Membrane.Element.Base.handle_info/3","doc":"Callback invoked when element receives a message that is not recognized\nas an internal membrane message.\n\nUseful for receiving ticks from timer, data sent from NIFs or other stuff.\nBy default, it logs and ignores the received message.","ref":"Membrane.Element.Base.html#c:handle_info/3"},{"type":"callback","title":"Membrane.Element.Base.handle_init/2","doc":"Callback invoked on initialization of element.\n\nThis callback is synchronous: the parent waits until it finishes. Also, any failures\nthat happen in this callback crash the parent as well, regardless of crash groups.\nFor these reasons, it's important to do any long-lasting or complex work in `c:handle_setup/2`,\nwhile `handle_init` should be used for things like parsing options or initializing state.\nBy default, it converts the `opts` struct to a map and sets them as the element's state.","ref":"Membrane.Element.Base.html#c:handle_init/2"},{"type":"callback","title":"Membrane.Element.Base.handle_pad_added/3","doc":"Callback that is called when new pad has beed added to element. Executed\nONLY for dynamic pads.\n\nContext passed to this callback contains additional field `:pad_options`.\nBy default, it does nothing.","ref":"Membrane.Element.Base.html#c:handle_pad_added/3"},{"type":"callback","title":"Membrane.Element.Base.handle_pad_removed/3","doc":"Callback that is called when some pad of the element has beed removed. Executed\nONLY for dynamic pads.\n\nContext passed to this callback contains additional field `:pad_options`.\nBy default, it does nothing.","ref":"Membrane.Element.Base.html#c:handle_pad_removed/3"},{"type":"callback","title":"Membrane.Element.Base.handle_parent_notification/3","doc":"Callback invoked when a message from the parent is received.\nBy default, it ignores the received message.","ref":"Membrane.Element.Base.html#c:handle_parent_notification/3"},{"type":"callback","title":"Membrane.Element.Base.handle_playing/2","doc":"Callback invoked when bin switches the playback to `:playing`.\n\nFrom this point, element can send and receive buffers, events, stream formats and demands\nthrough its pads.\nBy default, it does nothing.","ref":"Membrane.Element.Base.html#c:handle_playing/2"},{"type":"callback","title":"Membrane.Element.Base.handle_setup/2","doc":"Callback invoked on element startup, right after `c:handle_init/2`.\n\nAny long-lasting or complex initialization should happen here.\nBy default, it does nothing.","ref":"Membrane.Element.Base.html#c:handle_setup/2"},{"type":"callback","title":"Membrane.Element.Base.handle_terminate_request/2","doc":"Callback invoked when element is removed by its parent.\n\nBy default, it returns `t:Membrane.Element.Action.terminate/0` with reason `:normal`.","ref":"Membrane.Element.Base.html#c:handle_terminate_request/2"},{"type":"callback","title":"Membrane.Element.Base.handle_tick/3","doc":"Callback invoked upon each timer tick. A timer can be started with `Membrane.Element.Action.start_timer`\naction.","ref":"Membrane.Element.Base.html#c:handle_tick/3"},{"type":"module","title":"Membrane.Element.CallbackContext","doc":"Describes context passed to the Membrane Element callbacks.","ref":"Membrane.Element.CallbackContext.html"},{"type":"type","title":"Membrane.Element.CallbackContext.t/0","doc":"Type describing context passed to the Membrane Element callbacks.\n\nField `:incoming_demand` is present only in\n`c:Membrane.Element.WithOutputPads.handle_demand/5`.\n\nField `:pad_options` is present only in `c:Membrane.Element.Base.handle_pad_added/3`\nand `c:Membrane.Element.Base.handle_pad_removed/3`.\n\nField `:start_of_stream_received?` is present only in\n`c:Membrane.Element.WithInputPads.handle_end_of_stream/3`.\n\nField `:old_stream_format` is present only in\n`c:Membrane.Element.WithInputPads.handle_stream_format/4`.","ref":"Membrane.Element.CallbackContext.html#t:t/0"},{"type":"module","title":"Membrane.Element.PadData","doc":"Struct describing current pad state.\n\nThe public fields are:\n  - `:availability` - see `t:Membrane.Pad.availability/0`\n  - `:stream_format` - the most recent `t:Membrane.StreamFormat.t/0` that have been sent (output) or received (input)\n    on the pad. May be `nil` if not yet set.\n  - `:direction` - see `t:Membrane.Pad.direction/0`\n  - `:end_of_stream?` - flag determining whether the stream processing via the pad has been finished\n  - `:flow_control` - see `t:Membrane.Pad.flow_control/0`.\n  - `:name` - see `t:Membrane.Pad.name/0`. Do not mistake with `:ref`\n  - `:options` - options passed in `Membrane.ParentSpec` when linking pad\n  - `:ref` - see `t:Membrane.Pad.ref/0`\n  - `:start_of_stream?` - flag determining whether the stream processing via the pad has been started\n  - `auto_demand_paused?` - flag determining if auto-demanding on the pad is paused or no\n\nOther fields in the struct ARE NOT PART OF THE PUBLIC API and should not be\naccessed or relied on.","ref":"Membrane.Element.PadData.html"},{"type":"type","title":"Membrane.Element.PadData.private_field/0","doc":"","ref":"Membrane.Element.PadData.html#t:private_field/0"},{"type":"type","title":"Membrane.Element.PadData.t/0","doc":"Struct describing current pad state.\n\nThe public fields are:\n  - `:availability` - see `t:Membrane.Pad.availability/0`\n  - `:stream_format` - the most recent `t:Membrane.StreamFormat.t/0` that have been sent (output) or received (input)\n    on the pad. May be `nil` if not yet set.\n  - `:direction` - see `t:Membrane.Pad.direction/0`\n  - `:end_of_stream?` - flag determining whether the stream processing via the pad has been finished\n  - `:flow_control` - see `t:Membrane.Pad.flow_control/0`.\n  - `:name` - see `t:Membrane.Pad.name/0`. Do not mistake with `:ref`\n  - `:options` - options passed in `Membrane.ParentSpec` when linking pad\n  - `:ref` - see `t:Membrane.Pad.ref/0`\n  - `:start_of_stream?` - flag determining whether the stream processing via the pad has been started\n  - `auto_demand_paused?` - flag determining if auto-demanding on the pad is paused or no\n\nOther fields in the struct ARE NOT PART OF THE PUBLIC API and should not be\naccessed or relied on.","ref":"Membrane.Element.PadData.html#t:t/0"},{"type":"behaviour","title":"Membrane.Element.WithInputPads","doc":"Module defining behaviour for sink, filter and endpoint elements.\n\nWhen used declares behaviour implementation, provides default callback definitions\nand imports macros.\n\nFor more information on implementing elements, see `Membrane.Element.Base`.","ref":"Membrane.Element.WithInputPads.html"},{"type":"macro","title":"Membrane.Element.WithInputPads.def_input_pad/2","doc":"Macro that defines input pad for the element.\n\nIt automatically generates documentation from the given definition\nand adds compile-time stream format specs validation.\n\nThe type `t:Membrane.Pad.element_spec/0` describes how the definition of pads should look.","ref":"Membrane.Element.WithInputPads.html#def_input_pad/2"},{"type":"callback","title":"Membrane.Element.WithInputPads.handle_buffer/4","doc":"Callback that is called when buffer should be processed by the Element.\n\nFor pads in pull mode it is called when buffer have been demanded (by returning\n`:demand` action from any callback).\n\nFor pads in push mode it is invoked when buffer arrive.","ref":"Membrane.Element.WithInputPads.html#c:handle_buffer/4"},{"type":"callback","title":"Membrane.Element.WithInputPads.handle_end_of_stream/3","doc":"Callback invoked when the previous element has finished processing via the pad,\nand it cannot be used anymore.","ref":"Membrane.Element.WithInputPads.html#c:handle_end_of_stream/3"},{"type":"callback","title":"Membrane.Element.WithInputPads.handle_start_of_stream/3","doc":"Callback invoked when element receives the first buffer from the specific pad, before `c:Elixir.Membrane.Element.WithInputPads.handle_buffer/4`.","ref":"Membrane.Element.WithInputPads.html#c:handle_start_of_stream/3"},{"type":"callback","title":"Membrane.Element.WithInputPads.handle_stream_format/4","doc":"Callback invoked when Element is receiving information about new stream format for\ngiven pad.\n\nIn filters stream format is forwarded through all output pads by default.\n\nContext passed to this callback contains additional field `:old_stream_format`.","ref":"Membrane.Element.WithInputPads.html#c:handle_stream_format/4"},{"type":"behaviour","title":"Membrane.Element.WithOutputPads","doc":"Module defining behaviour for source and filter elements.\n\nWhen used declares behaviour implementation, provides default callback definitions\nand imports macros.\n\nFor more information on implementing elements, see `Membrane.Element.Base`.","ref":"Membrane.Element.WithOutputPads.html"},{"type":"macro","title":"Membrane.Element.WithOutputPads.def_output_pad/2","doc":"Macro that defines output pad for the element.\n\nIt automatically generates documentation from the given definition\nand adds compile-time stream format specs validation.\n\nThe type `t:Membrane.Pad.element_spec/0` describes how the definition of pads should look.","ref":"Membrane.Element.WithOutputPads.html#def_output_pad/2"},{"type":"callback","title":"Membrane.Element.WithOutputPads.handle_demand/5","doc":"Callback called when buffers should be emitted by a source, filter or endpoint.\n\nIt is called only for output pads in the `:manual` flow control mode, as in their case demand\nis triggered by the input pad of the subsequent element.\n\nIn sources and endpoint, appropriate amount of data should be sent here.\n\nIn filters, this callback should usually return `:demand` action with\nsize sufficient for supplying incoming demand. This will result in calling\n`c:Membrane.WithInputPads.handle_buffer/4`, which is to supply\nthe demand.\n\nIf a source or an endpoint is unable to produce enough buffers, or a filter\nunderestimated returned demand, the `:redemand` action should be used (see\n`t:Membrane.Element.Action.redemand/0`).\n\nContext passed to this callback contains additional field `:incoming_demand`.","ref":"Membrane.Element.WithOutputPads.html#c:handle_demand/5"},{"type":"module","title":"Membrane.ChildrenSpec","doc":"A module with functionalities that allow to represent a topology of a pipeline/bin.\n\nThe children specification (commonly referred to as a \"children_spec\") is represented by the following type:\n`t:t/0`. It consists of two parts - a children's specification and the children's specification options.\n\nThe children's specification describes the desired topology and can be incorporated into a pipeline or a bin by returning\n`t:Membrane.Pipeline.Action.spec/0` or `t:Membrane.Bin.Action.spec/0`\naction, respectively. This commonly happens within `c:Membrane.Pipeline.handle_init/2`\nand `c:Membrane.Bin.handle_init/2`, but can be done in any other callback also.","ref":"Membrane.ChildrenSpec.html"},{"type":"module","title":"Children's specification - Membrane.ChildrenSpec","doc":"The children's specification allows specifying the children that need to be spawned in the action, as well as\nlinks between the children (both the children spawned in that action, and already existing children).\n\nThe children's processes are spawned with the use of `child/1`, `child/2`, `child/3` and `child/4` functions.\nThese functions can be used for spawning nodes of a link in an inline manner:\n```\n  spec = [child(:source, Source) |> child(:filter, %Filter{option: 1}) |> child(:sink, Sink)]\n```\nor just to spawn children processes, without linking the newly created children:\n```\n  spec = [child(:source, Source),\n    child(:filter, Filter),\n    child(:sink, Sink)\n  ]\n```\n\nProviding a child name is not necessary - you can spawn an anonymous child if you do not\nneed to refer to that child later on:\n```\n  spec = child(Source) |> child(Filter)\n```\nChildren created that way will have their automatically generated identifier consisting of a module\nname and a random unique reference number, so that you will be able to distinguish between anonymous\nchildren i.e. in log prints.\n\nIn case you need to refer to an already existing child (which could be spawned, i.e. in the previous `spec` action),\nuse `get_child/1` and `get_child/2` functions, as in the example below:\n```\n  spec = [get_child(:already_existing_source) |> child(:this_filter_will_be_spawned, Filter) |> get_child(:already_existing_sink)]\n```\n\nThe `child` functions allow specifying `:get_if_exists` option.\nIt might be helpful when you are not certain if the child with the given name exists, and, therefore, you are unable to\nchoose between `get_child` and `child` functions. After setting the `get_if_exists: true` option in `child/3` and `child/4` functions you can be sure\nthat in case a child with a given name already exists, you will simply refer to that child instead of respawning it.\n```\n  spec = [child(:sink, Sink),\n    child(:sink, Sink, get_if_exists: true) |> child(:source, Source)]\n```\nIn the example above you can see, that the `:sink` child is created in the first element of the `spec` list.\nIn the second element of that list, the `get_if_exists: true` option is used within `child/3`, which will have the same effect as if\n`get_child(:sink)` was used. At the same time, if the `:sink` child wasn't already spawned, it would be created in that link definition.\nPlease note that it makes sense to use `:get_if_exists` option only with named children.\n\n#","ref":"Membrane.ChildrenSpec.html#module-children-s-specification"},{"type":"module","title":"Links between pads - Membrane.ChildrenSpec","doc":"`via_in/2` and `via_out/2` functions allow\nspecifying pads' names and parameters. If pads are not specified, name `:input`\nis assumed for inputs and `:output` for outputs.\n\nSample definition:\n\n```\n[\n  get_child(:source_a)\n  |> get_child(:converter)\n  |> via_in(:input_a, target_queue_size: 20)\n  |> get_child(:mixer),\n  get_child(:source_b)\n  |> via_out(:custom_output)\n  |> via_in(:input_b, options: [mute: true])\n  |> get_child(:mixer)\n  |> via_in(:input, toilet_capacity: 500)\n  |> get_child(:sink)\n]\n```\n\nSee the docs for `via_in/3` and `via_out/3` for details on pad properties that can be set.\nLinks can also contain children's definitions, for example:\n\n```\n[\n  child(:first_element, %Element.With.Options.Struct{option_a: 42})\n  |> child(:some_element, Element.Without.Options)\n  |> get_child(:element_specified_before)\n]\n```\n\n#","ref":"Membrane.ChildrenSpec.html#module-links-between-pads"},{"type":"module","title":"Bins - Membrane.ChildrenSpec","doc":"For bin boundaries, there are special links allowed. The user should define links\nbetween the bin's input and the first child's input (input-input type) and the last\nchild's output and bin output (output-output type). In this case, `bin_input/1`\nand `bin_output/2` should be used.\n\nSample definition:\n\n```\n[\n  bin_input() |> get_child(:filter1) |> get_child(:filter2) |> bin_output(:custom_output)\n]\n```\n\n#","ref":"Membrane.ChildrenSpec.html#module-bins"},{"type":"module","title":"Dynamic pads - Membrane.ChildrenSpec","doc":"In most cases, dynamic pads can be linked the same way as static ones, although\nin the following situations, an exact pad reference must be passed instead of a name:\n\n- When that reference is needed later, for example, to handle a notification related\nto that particular pad instance\n\n```\npad = Pad.ref(:output, make_ref())\n[\n  get_child(:tee) |> via_out(pad) |> get_child(:sink)\n]\n```\n\n- When linking dynamic pads of a bin with its children, for example in\n`c:Membrane.Bin.handle_pad_added/3`\n\n```\n@impl true\ndef handle_pad_added(Pad.ref(:input, _) = pad, _ctx, state) do\n  spec = [bin_input(pad) |> get_child(:mixer)]\n  {{:ok, spec: spec}, state}\nend\n```","ref":"Membrane.ChildrenSpec.html#module-dynamic-pads"},{"type":"module","title":"Children's specification options - Membrane.ChildrenSpec","doc":"#","ref":"Membrane.ChildrenSpec.html#module-children-s-specification-options"},{"type":"module","title":"Stream sync - Membrane.ChildrenSpec","doc":"`:stream_sync` field can be used for specifying elements that should start playing\nat the same moment. An example can be audio and video player sinks. This option\naccepts either `:sinks` atom or a list of groups (lists) of elements. Passing `:sinks`\nresults in synchronizing all sinks in the pipeline, while passing a list of groups\nof elements synchronizes all elements in each group. It is worth mentioning\nthat to keep the stream synchronized all involved elements need to rely on\nthe same clock.\n\nBy default, no elements are synchronized.\n\nSample definitions:\n\n```\nchildren = ...\n  {children, stream_sync: [[:element1, :element2], [:element3, :element4]]}\n  {children, stream_sync: :sinks}\n```\n\n#","ref":"Membrane.ChildrenSpec.html#module-stream-sync"},{"type":"module","title":"Clock provider - Membrane.ChildrenSpec","doc":"A clock provider is an element that exports a clock that should be used as the pipeline\nclock. The pipeline clock is the default clock used by elements' timers.\nFor more information see `Membrane.Element.Base.def_clock/1`.\n\n#","ref":"Membrane.ChildrenSpec.html#module-clock-provider"},{"type":"module","title":"Children groups - Membrane.ChildrenSpec","doc":"Children groups allow aggregating the spawned children into easily identifiable groups.\nWith the use of them, it is possible to refer to all the children of the group with a single identifier.\nExample:\n```\n  spec1 = {links1, group: :first_children_group}\n  spec2 = {links2, group: :second_children_group}\n```\nThe children spawned within `links1` specification will be put inside `:first_children_group`, whereas the\nchildren spawned within `links2` specification will be put inside `second_children_group`.\n\nLater on, the children from a given group can be referred with their `group`, as in the example below:\n```\n  actions = [remove_children: :first_children_group]\n```\nWith the action defined above, all the children from the `:first_children_group` can be removed at once.\n\n#","ref":"Membrane.ChildrenSpec.html#module-children-groups"},{"type":"module","title":"Crash groups - Membrane.ChildrenSpec","doc":"A crash group is a logical entity that prevents the whole pipeline from crashing when one of\ncomponents crashes. A crash group is defined with the use of two children specification options:\n* `group` - which acts as a crash group identifier\n* `crash_group_mode` - its value specifies the behavior of children in the crash group. Currently, we support only\n`:temporary` mode which means that Membrane will not make any attempts to restart crashed child.\n\n##","ref":"Membrane.ChildrenSpec.html#module-crash-groups"},{"type":"module","title":"Adding children to a crash group - Membrane.ChildrenSpec","doc":"```\nspec = [\n  child(:some_element_1, %SomeElement{\n    # ...\n  },\n  child(:some_element_2, %SomeElement{\n    # ...\n  }\n]\n\nspec = {spec, group: group_id, crash_group_mode: :temporary}\n```\n\nIn the above snippet, we create new children - `:some_element_1` and `:some_element_2`, we add them\nto the crash group with id `group_id`. Crash of `:some_element_1` or `:some_element_2` propagates\nonly to the rest of the members of the crash group and the pipeline stays alive.\n\n##","ref":"Membrane.ChildrenSpec.html#module-adding-children-to-a-crash-group"},{"type":"module","title":"Handling crash of a crash group - Membrane.ChildrenSpec","doc":"When any of the members of the crash group goes down, the callback:\n[`handle_crash_group_down/3`](https://hexdocs.pm/membrane_core/Membrane.Pipeline.html#c:handle_crash_group_down/3)\nis called.\n\n```\n@impl true\ndef handle_crash_group_down(crash_group_id, ctx, state) do\n  # do some stuff in reaction to the crash of the group with id crash_group_id\nend\n```\n\n##","ref":"Membrane.ChildrenSpec.html#module-handling-crash-of-a-crash-group"},{"type":"module","title":"Limitations - Membrane.ChildrenSpec","doc":"At this moment crash groups are only useful for elements with dynamic pads.\nCrash groups work in pipelines and bins as well.\n\n#","ref":"Membrane.ChildrenSpec.html#module-limitations"},{"type":"module","title":"Log metadata - Membrane.ChildrenSpec","doc":"`:log_metadata` field can be used to set the `Membrane.Logger` metadata for all children in the given children specification.","ref":"Membrane.ChildrenSpec.html#module-log-metadata"},{"type":"module","title":"Nesting children's specifications - Membrane.ChildrenSpec","doc":"The children's specifications can be nested within themselves.\n\nConsider the following children's specifications:\n```\n{[\n  child(:a, A) |> child(:b, B),\n  {child(:c, C), group: :second, crash_group_mode: :temporary}\n], group: :first, crash_group_mode: :temporary, node: some_node}\n```\n\nChild `:c` will be spawned in the `:second` crash group, while children `:a` and `:b` will be spawned in the `:first` crash group.\nFurthermore, since the inner children specification does not define the `:node` option, it will be inherited from the outer children specification.\nThat means that child `:c` will be spawned on the `some_node` node, along with children `:a` and `:b`.","ref":"Membrane.ChildrenSpec.html#module-nesting-children-s-specifications"},{"type":"function","title":"Membrane.ChildrenSpec.bin_input/1","doc":"Begins a link with a bin's pad.\n\nSee the _Children's specification_ section of the moduledoc for more information.","ref":"Membrane.ChildrenSpec.html#bin_input/1"},{"type":"function","title":"Membrane.ChildrenSpec.bin_output/2","doc":"Ends a link with a bin's output.\n\nSee the _Children's specification_ section of the moduledoc for more information.","ref":"Membrane.ChildrenSpec.html#bin_output/2"},{"type":"function","title":"Membrane.ChildrenSpec.child/1","doc":"Used to spawn an anonymous child at the beggining of the link specification.\n\nSee the _Children's specification_ section of the moduledoc for more information.","ref":"Membrane.ChildrenSpec.html#child/1"},{"type":"function","title":"Membrane.ChildrenSpec.child/2","doc":"Used to spawn a named child at the beggining of the link\nspecification or to spawn an anynomous child.\n\nSee the _Children's specification_ section of the moduledoc for more information.","ref":"Membrane.ChildrenSpec.html#child/2"},{"type":"function","title":"Membrane.ChildrenSpec.child/3","doc":"Used to spawn a named child or an anonymous child in the middle\nof the link specification.\n\nSee the _Children's specification_ section of the moduledoc for more information.","ref":"Membrane.ChildrenSpec.html#child/3"},{"type":"function","title":"Membrane.ChildrenSpec.child/4","doc":"Used to spawn a named child in the middle of a link specification.\n\nSee the _Children's specification_ section of the moduledoc for more information.","ref":"Membrane.ChildrenSpec.html#child/4"},{"type":"function","title":"Membrane.ChildrenSpec.get_child/1","doc":"Used to refer to an existing child at a beginning of a link specification.\n\nSee the _Children's specification_ section of the moduledoc for more information.","ref":"Membrane.ChildrenSpec.html#get_child/1"},{"type":"function","title":"Membrane.ChildrenSpec.get_child/2","doc":"Used to refer to an existing child in a middle of a link specification.\n\nSee the _Children's specification_ section of the moduledoc for more information.","ref":"Membrane.ChildrenSpec.html#get_child/2"},{"type":"function","title":"Membrane.ChildrenSpec.via_in/3","doc":"Specifies input pad name and properties of the subsequent child.\n\nThe possible properties are:\n- `options` - If a pad defines options, they can be passed here as a keyword list. Pad options are documented\nin moduledoc of each element. See `Membrane.Element.WithInputPads.def_input_pad/2` and `Membrane.Bin.def_input_pad/2`\nfor information about defining pad options.\n\nAdditionally, the following properties can be used to adjust the flow control parameters. If set within a bin\non an input that connects to the bin input, they will be overridden if set when linking to the bin in its parent.\n\n- `toilet_capacity` - Used when a toilet is created, that is for pull input pads (with `:auto` or `:manual` flow control), that\nhave push output pads (with `:push` flow control) linked to them. When a push output produces more buffers than the pull input can consume, the buffers are accumulated\nin a queue called a toilet. If the toilet size grows above its capacity, it overflows by raising an error.\n- `target_queue_size` - The size of the queue of the input pad that Membrane will try to maintain. That allows for fulfilling\nthe demands of the element by taking data from the queue while the actual sending of demands is done asynchronously,\nsmoothing the processing. Used only for pads working in `:manual` flow control mode. See `t:Membrane.Pad.flow_control/0` for\nmore information.\n- `min_demand_factor` - A factor used to calculate `minimal demand` (`minimal_demand = target_queue_size * min_demand_factor`).\nMembrane won't send smaller demand than `minimal demand`, to reduce demands' overhead. However, the user will always receive\nas many buffers, as demanded, all excess buffers will be queued internally.\nUsed only for pads working in `:manual` flow control mode. See `t:Membrane.Pad.flow_control/0`\nfor more info. Defaults to `0.25` (the default may change in the future).\n- `auto_demand_size` - Size of automatically generated demands. Used only for pads working in `:auto` flow control mode.\n  See `t:Membrane.Pad.flow_control/0`  for more info.\n- `throttling_factor` - an integer specifying how frequently should a sender update the number of buffers in the `Toilet`. Defaults to 1,\n  meaning, that the sender will update the toilet with each buffer being sent. Setting that factor for elements,\n  which are running on the same node, does not have an impact of performance. However, once the sending element and the receiving element are put on different nodes,\n  the sender updates the toilet with interprocess messages and setting a bigger `throttling_factor` can reduce the number of messages\n  in the system.\n  At the same time, setting a greater `throttling_factor` can result in a toilet overflow being detected later.\n\nSee the _Children's specification_ section of the moduledoc for more information.","ref":"Membrane.ChildrenSpec.html#via_in/3"},{"type":"function","title":"Membrane.ChildrenSpec.via_out/3","doc":"Specifies output pad name and properties of the preceding child.\n\nThe possible properties are:\n- `options` - If a pad defines options, they can be passed here as a keyword list. Pad options are documented\nin moduledoc of each element. See `Membrane.Element.WithOutputPads.def_output_pad/2` and `Membrane.Bin.def_output_pad/2`\nfor information about defining pad options.\n\nSee the _Children's specification_ section of the moduledoc for more information.","ref":"Membrane.ChildrenSpec.html#via_out/3"},{"type":"opaque","title":"Membrane.ChildrenSpec.builder/0","doc":"","ref":"Membrane.ChildrenSpec.html#t:builder/0"},{"type":"type","title":"Membrane.ChildrenSpec.child_definition/0","doc":"","ref":"Membrane.ChildrenSpec.html#t:child_definition/0"},{"type":"type","title":"Membrane.ChildrenSpec.child_options/0","doc":"","ref":"Membrane.ChildrenSpec.html#t:child_options/0"},{"type":"type","title":"Membrane.ChildrenSpec.children_spec_options/0","doc":"","ref":"Membrane.ChildrenSpec.html#t:children_spec_options/0"},{"type":"type","title":"Membrane.ChildrenSpec.crash_group_mode/0","doc":"","ref":"Membrane.ChildrenSpec.html#t:crash_group_mode/0"},{"type":"type","title":"Membrane.ChildrenSpec.pad_options/0","doc":"","ref":"Membrane.ChildrenSpec.html#t:pad_options/0"},{"type":"type","title":"Membrane.ChildrenSpec.t/0","doc":"Used to describe the inner topology of the pipeline/bin.","ref":"Membrane.ChildrenSpec.html#t:t/0"},{"type":"module","title":"Membrane.Child","doc":"Module that keeps track of types used by both elements and bins","ref":"Membrane.Child.html"},{"type":"macro","title":"Membrane.Child.is_child_name?/1","doc":"","ref":"Membrane.Child.html#is_child_name?/1"},{"type":"type","title":"Membrane.Child.group/0","doc":"","ref":"Membrane.Child.html#t:group/0"},{"type":"type","title":"Membrane.Child.name/0","doc":"","ref":"Membrane.Child.html#t:name/0"},{"type":"type","title":"Membrane.Child.options/0","doc":"","ref":"Membrane.Child.html#t:options/0"},{"type":"module","title":"Membrane.ChildEntry","doc":"Struct describing child entry of a parent.\n\nThe public fields are:\n- `name` - child name\n- `module` - child module\n- `group` - child group name\n- `options` - options passed to the child\n- `component_type` - either `:element` or `:bin`\n- `playback` - either `:stopped` or `:playing`\n\nOther fields in the struct ARE NOT PART OF THE PUBLIC API and should not be\naccessed or relied on.","ref":"Membrane.ChildEntry.html"},{"type":"type","title":"Membrane.ChildEntry.t/0","doc":"Struct describing child entry of a parent.\n\nThe public fields are:\n- `name` - child name\n- `module` - child module\n- `group` - child group name\n- `options` - options passed to the child\n- `component_type` - either `:element` or `:bin`\n- `playback` - either `:stopped` or `:playing`\n\nOther fields in the struct ARE NOT PART OF THE PUBLIC API and should not be\naccessed or relied on.","ref":"Membrane.ChildEntry.html#t:t/0"},{"type":"module","title":"Membrane.Buffer","doc":"Structure representing a single chunk of data that flows between elements.\n\nFor now, it is just a wrapper around bitstring with optionally some metadata\nattached to it, but in future releases we plan to support different payload\ntypes.","ref":"Membrane.Buffer.html"},{"type":"function","title":"Membrane.Buffer.get_dts_or_pts/1","doc":"Returns `t:Membrane.Buffer.t/0` `:dts` if available or `:pts` if `:dts` is not set.\nIf none of them is set `nil` is returned.","ref":"Membrane.Buffer.html#get_dts_or_pts/1"},{"type":"type","title":"Membrane.Buffer.metadata/0","doc":"","ref":"Membrane.Buffer.html#t:metadata/0"},{"type":"type","title":"Membrane.Buffer.t/0","doc":"Structure representing a single chunk of data that flows between elements.\n\nFor now, it is just a wrapper around bitstring with optionally some metadata\nattached to it, but in future releases we plan to support different payload\ntypes.","ref":"Membrane.Buffer.html#t:t/0"},{"type":"module","title":"Membrane.ChildNotification","doc":"A child notification is a message sent from `Membrane.Element` or `Membrane.Bin` to a parent\nvia action `t:Membrane.Element.Action.notify_parent` or `t:Membrane.Bin.Action.notify_parent`\nreturned from any callback.\n\nA notification can be handled in parent with\n`c:Membrane.Parent.handle_child_notification/4` callback.","ref":"Membrane.ChildNotification.html"},{"type":"type","title":"Membrane.ChildNotification.t/0","doc":"A child notification is a message sent from `Membrane.Element` or `Membrane.Bin` to a parent\nvia action `t:Membrane.Element.Action.notify_parent` or `t:Membrane.Bin.Action.notify_parent`\nreturned from any callback.\n\nA notification can be handled in parent with\n`c:Membrane.Parent.handle_child_notification/4` callback.","ref":"Membrane.ChildNotification.html#t:t/0"},{"type":"module","title":"Membrane.Event","doc":"Represents a communication event, capable of flowing both downstream and upstream.\n\nEvents are dispatched using `t:Membrane.Element.Action.event/0` and are handled via the\n`c:Membrane.Element.Base.handle_event/4` callback. Each event must conform to the\n`Membrane.EventProtocol` to ensure the proper configuration of its behaviour.","ref":"Membrane.Event.html"},{"type":"function","title":"Membrane.Event.async?/1","doc":"","ref":"Membrane.Event.html#async?/1"},{"type":"function","title":"Membrane.Event.event?/1","doc":"Checks if the given argument is a Membrane event.\n\nReturns `true` if the `event` implements the `Membrane.EventProtocol`, otherwise `false`.","ref":"Membrane.Event.html#event?/1"},{"type":"function","title":"Membrane.Event.sticky?/1","doc":"","ref":"Membrane.Event.html#sticky?/1"},{"type":"type","title":"Membrane.Event.t/0","doc":"The Membrane event, based on the `Membrane.EventProtocol`.","ref":"Membrane.Event.html#t:t/0"},{"type":"protocol","title":"Membrane.EventProtocol","doc":"Protocol that allows to configure behaviour of `Membrane.Event`s.\n\nEach event has to implement or derive this protocol.","ref":"Membrane.EventProtocol.html"},{"type":"function","title":"Membrane.EventProtocol.async?/1","doc":"Determines whether event is synchronized with buffers (sync) or not (async).\nDefaults to false (sync).\n\nBuffers and sync events are always received in the same order they are\nsent. Async events are handled before any buffers enqueued that are waiting\nin Membrane internal queues to be processed.","ref":"Membrane.EventProtocol.html#async?/1"},{"type":"function","title":"Membrane.EventProtocol.sticky?/1","doc":"Specifies whether event is sent right away (not sticky), or it is 'pushed' by\nthe next sent buffer (sticky). Defaults to false (not sticky).\n\nReturning a sticky event from a callback stores it in a queue. When the next\nbuffer is to be sent, all events from the queue are sent before it. An example\ncan be the `Membrane.Event.StartOfStream` event.","ref":"Membrane.EventProtocol.html#sticky?/1"},{"type":"type","title":"Membrane.EventProtocol.t/0","doc":"A type describing all the types that implement the `Membrane.EventProtocol` protocol.","ref":"Membrane.EventProtocol.html#t:t/0"},{"type":"module","title":"Membrane.KeyframeRequestEvent","doc":"Generic event for requesting a key frame.\n\nThe key frame is meant as a part of stream such that\nthe stream can be decoded from the beginning of each key\nframe without knowledge of the stream content before that\npoint.","ref":"Membrane.KeyframeRequestEvent.html"},{"type":"type","title":"Membrane.KeyframeRequestEvent.t/0","doc":"Generic event for requesting a key frame.\n\nThe key frame is meant as a part of stream such that\nthe stream can be decoded from the beginning of each key\nframe without knowledge of the stream content before that\npoint.","ref":"Membrane.KeyframeRequestEvent.html#t:t/0"},{"type":"module","title":"Membrane.Pad","doc":"Pads are units defined by elements and bins, allowing them to be linked with their\nsiblings. This module consists of pads typespecs and utils.\n\nEach pad is described by its name, direction, availability, mode and possible stream format.\nFor pads to be linkable, these properties have to be compatible. For more\ninformation on each of them, check appropriate type in this module.\n\nEach link can only consist of exactly two pads.","ref":"Membrane.Pad.html"},{"type":"function","title":"Membrane.Pad.availability_mode/1","doc":"Returns pad availability mode for given availability.","ref":"Membrane.Pad.html#availability_mode/1"},{"type":"macro","title":"Membrane.Pad.is_availability/1","doc":"","ref":"Membrane.Pad.html#is_availability/1"},{"type":"macro","title":"Membrane.Pad.is_availability_dynamic/1","doc":"","ref":"Membrane.Pad.html#is_availability_dynamic/1"},{"type":"macro","title":"Membrane.Pad.is_availability_static/1","doc":"","ref":"Membrane.Pad.html#is_availability_static/1"},{"type":"macro","title":"Membrane.Pad.is_dynamic_pad_ref/1","doc":"","ref":"Membrane.Pad.html#is_dynamic_pad_ref/1"},{"type":"macro","title":"Membrane.Pad.is_pad_name/1","doc":"","ref":"Membrane.Pad.html#is_pad_name/1"},{"type":"macro","title":"Membrane.Pad.is_pad_ref/1","doc":"","ref":"Membrane.Pad.html#is_pad_ref/1"},{"type":"macro","title":"Membrane.Pad.is_static_pad_ref/1","doc":"","ref":"Membrane.Pad.html#is_static_pad_ref/1"},{"type":"function","title":"Membrane.Pad.name_by_ref/1","doc":"Returns the name for the given pad reference","ref":"Membrane.Pad.html#name_by_ref/1"},{"type":"function","title":"Membrane.Pad.opposite_direction/1","doc":"","ref":"Membrane.Pad.html#opposite_direction/1"},{"type":"macro","title":"Membrane.Pad.ref/1","doc":"Creates a static pad reference.","ref":"Membrane.Pad.html#ref/1"},{"type":"macro","title":"Membrane.Pad.ref/2","doc":"Creates a dynamic pad reference.","ref":"Membrane.Pad.html#ref/2"},{"type":"type","title":"Membrane.Pad.accepted_format/0","doc":"Describes pattern, that should be matched by stream format send by element on specific\npad. Will not be evaluated during runtime, but used for matching struct passed in\n`:stream_format` action.\nCan be a module name, pattern describing struct, or call to `any_of` function, which\narguments are such patterns or modules names.\nIf a module name is passed to the `:accepted_format` option or is passed to `any_of`,\nit will be converted to the match on a struct defined in that module, eg.\n`accepted_format: My.Format` will have this same effect, as `accepted_format: %My.Format{}`\nand `accepted_format: any_of(My.Format, %My.Another.Format{field: value} when value in\n[:some, :enumeration])` will have this same effect, as `accepted_format: any_of(%My.Format{},\n%My.Another.Format{field: value} when value in [:some, :enumeration])`","ref":"Membrane.Pad.html#t:accepted_format/0"},{"type":"type","title":"Membrane.Pad.availability/0","doc":"Values used when defining pad availability:\n\n- `:always` - a static pad, which can remain unlinked in `stopped` state only.\n- `:on_request` - a dynamic pad, instance of which is created every time it is\nlinked to another pad. Thus linking the pad with _k_ other pads, creates _k_\ninstances of the pad, and links each with another pad.","ref":"Membrane.Pad.html#t:availability/0"},{"type":"type","title":"Membrane.Pad.availability_mode/0","doc":"Type describing availability mode of a created pad:\n\n- `:static` - there always exist exactly one instance of such pad.\n- `:dynamic` - multiple instances of such pad may be created and removed (which\nentails executing `handle_pad_added` and `handle_pad_removed` callbacks,\nrespectively).","ref":"Membrane.Pad.html#t:availability_mode/0"},{"type":"type","title":"Membrane.Pad.bin_spec/0","doc":"Describes how a pad should be declared inside a bin.\n\nDemand unit is derived from the first element inside the bin linked to the\ngiven input.","ref":"Membrane.Pad.html#t:bin_spec/0"},{"type":"type","title":"Membrane.Pad.description/0","doc":"Type describing a pad. Contains data parsed from `t:spec/0`","ref":"Membrane.Pad.html#t:description/0"},{"type":"type","title":"Membrane.Pad.direction/0","doc":"Defines possible pad directions:\n\n- `:output` - data can only be sent through such pad,\n- `:input` - data can only be received through such pad.\n\nOne cannot link two pads with the same direction.","ref":"Membrane.Pad.html#t:direction/0"},{"type":"type","title":"Membrane.Pad.dynamic_id/0","doc":"Possible id of dynamic pad","ref":"Membrane.Pad.html#t:dynamic_id/0"},{"type":"type","title":"Membrane.Pad.element_spec/0","doc":"Describes how a pad should be declared inside an element.","ref":"Membrane.Pad.html#t:element_spec/0"},{"type":"type","title":"Membrane.Pad.flow_control/0","doc":"Describes how an element sends and receives data.\n\nThe available values for that field are:\n- `:manual` - meaning that the pad works in a pull mode and the demand is manually handled and requested.\nAn element with output `:manual` pad can send data through such a pad only if it has already received demand\non that pad. And element with input `:manual` pad receives data through such a pad only if it has been\npreviously demanded, so that no undemanded data can arrive For more info, see `Membrane.Element.Action.demand`,\n`Membrane.Element.Action.redemand` and `c:Membrane.Element.WithOutputPads.handle_demand/5`.\n- `:auto` - meaning that the pad works in a pull mode and the demand is managed automatically:\nthe core ensures that there's demand on each input pad (that has `flow_control` set to `:auto`)\nwhenever there's demand on all output pads (that have `flow_control` set to `:auto`).\nCurrently works for `Membrane.Filter`s, `Membrane.Endpoint`s and `Membrane.Sink`s.\n- `:push` - meaning that the pad works in a push mode. An element with a `:push` output pad can send data\nthrough that pad whenever it wants. An element with a `:push` input pad has to deal with data whenever it\ncomes through such a pad - note, that it needs to be done fast enough so that not to let data accumulate,\nwhat may lead to overflow of element process erlang queue, which is highly unwanted.\n\nLinking pads with different flow control is possible, but only in case of an output pad\nworking in a push mode, and an input pad in pull mode (that is - with `:manual` or `:auto` flow control).\nIn such case, however, an error will be raised whenever too many buffers accumulate on the input pad,\nwaiting to be processed.","ref":"Membrane.Pad.html#t:flow_control/0"},{"type":"type","title":"Membrane.Pad.name/0","doc":"Defines the name of pad or group of dynamic pads","ref":"Membrane.Pad.html#t:name/0"},{"type":"type","title":"Membrane.Pad.ref/0","doc":"Defines the term by which the pad instance is identified.","ref":"Membrane.Pad.html#t:ref/0"},{"type":"type","title":"Membrane.Pad.spec/0","doc":"Describes how a pad should be declared in element or bin.","ref":"Membrane.Pad.html#t:spec/0"},{"type":"module","title":"Membrane.ParentNotification","doc":"A parent notification is a message sent from `Membrane.Parent` or `Membrane.Bin` to a child\nvia action `t:Membrane.Pipeline.Action.notify_parent` or `t:Membrane.Bin.Action.notify_child`\nreturned from any callback.\n\nA notification can be handled in child with `c:Membrane.Element.Base.handle_parent_notification/3` or\n`c:Membrane.Bin.handle_parent_notification/3` callback.","ref":"Membrane.ParentNotification.html"},{"type":"type","title":"Membrane.ParentNotification.t/0","doc":"A parent notification is a message sent from `Membrane.Parent` or `Membrane.Bin` to a child\nvia action `t:Membrane.Pipeline.Action.notify_parent` or `t:Membrane.Bin.Action.notify_child`\nreturned from any callback.\n\nA notification can be handled in child with `c:Membrane.Element.Base.handle_parent_notification/3` or\n`c:Membrane.Bin.handle_parent_notification/3` callback.","ref":"Membrane.ParentNotification.html#t:t/0"},{"type":"protocol","title":"Membrane.Payload","doc":"This protocol describes actions common to all payload types.\n\nThe most basic payload type is simply a binary for which `Elixir.Membrane.Payload`\nis implemented by the Membrane Core.","ref":"Membrane.Payload.html"},{"type":"function","title":"Membrane.Payload.concat/2","doc":"Concatenates the contents of two payloads.","ref":"Membrane.Payload.html#concat/2"},{"type":"function","title":"Membrane.Payload.drop/2","doc":"Drops first `n` bytes of payload.","ref":"Membrane.Payload.html#drop/2"},{"type":"function","title":"Membrane.Payload.module/1","doc":"Returns a module responsible for this type of payload\nand implementing `Membrane.Payload.Behaviour`","ref":"Membrane.Payload.html#module/1"},{"type":"function","title":"Membrane.Payload.size/1","doc":"Returns total size of payload in bytes","ref":"Membrane.Payload.html#size/1"},{"type":"function","title":"Membrane.Payload.split_at/2","doc":"Splits the payload at given position (1st part has the size equal to `at_pos` argument)\n\n`at_pos` has to be greater than 0 and smaller than the size of payload, otherwise\nan error is raised. This guarantees returned payloads are never empty.","ref":"Membrane.Payload.html#split_at/2"},{"type":"function","title":"Membrane.Payload.to_binary/1","doc":"Converts payload into binary","ref":"Membrane.Payload.html#to_binary/1"},{"type":"type","title":"Membrane.Payload.t/0","doc":"A type describing all the types that implement `Membrane.Payload` protocol.","ref":"Membrane.Payload.html#t:t/0"},{"type":"module","title":"Membrane.RemoteStream","doc":"Format describing an unparsed data stream. It should be used whenever outputting\nor accepting an unknown stream (not to be confused with _any_ stream, which\ncan have well-specified format either), or a stream whose format can't/shouldn't\nbe created at that stage.\n\nParameters:\n- `:content_format` - format that is supposed to be carried in the stream,\n`nil` if unknown (default)\n- `:type` - either `:bytestream` (continuous stream) or `:packetized` (each buffer\ncontains exactly one specified unit of data)","ref":"Membrane.RemoteStream.html"},{"type":"type","title":"Membrane.RemoteStream.t/0","doc":"Format describing an unparsed data stream. It should be used whenever outputting\nor accepting an unknown stream (not to be confused with _any_ stream, which\ncan have well-specified format either), or a stream whose format can't/shouldn't\nbe created at that stage.\n\nParameters:\n- `:content_format` - format that is supposed to be carried in the stream,\n`nil` if unknown (default)\n- `:type` - either `:bytestream` (continuous stream) or `:packetized` (each buffer\ncontains exactly one specified unit of data)","ref":"Membrane.RemoteStream.html#t:t/0"},{"type":"module","title":"Membrane.StreamFormat","doc":"Defines the capabilities of a pad within the Membrane framework.\n\nEach pad in a multimedia pipeline has specific capabilities, determining the type and format\nof data it can handle. For example, a pad's capabilities might include handling raw audio\nwith a specific sample rate or managing encoded audio in a specified format.\n\nTo successfully link two pads together, their capabilities must be compatible.","ref":"Membrane.StreamFormat.html"},{"type":"type","title":"Membrane.StreamFormat.t/0","doc":"Represents a pad's capabilities. For more information, see: `Membrane.StreamFormat`.","ref":"Membrane.StreamFormat.html#t:t/0"},{"type":"behaviour","title":"Membrane.Buffer.Metric","doc":"Specifies API for metrics that analyze data in terms of a given unit","ref":"Membrane.Buffer.Metric.html"},{"type":"callback","title":"Membrane.Buffer.Metric.buffer_size_approximation/0","doc":"","ref":"Membrane.Buffer.Metric.html#c:buffer_size_approximation/0"},{"type":"callback","title":"Membrane.Buffer.Metric.buffers_size/1","doc":"","ref":"Membrane.Buffer.Metric.html#c:buffers_size/1"},{"type":"function","title":"Membrane.Buffer.Metric.from_unit/1","doc":"","ref":"Membrane.Buffer.Metric.html#from_unit/1"},{"type":"callback","title":"Membrane.Buffer.Metric.split_buffers/2","doc":"","ref":"Membrane.Buffer.Metric.html#c:split_buffers/2"},{"type":"type","title":"Membrane.Buffer.Metric.unit/0","doc":"","ref":"Membrane.Buffer.Metric.html#t:unit/0"},{"type":"module","title":"Membrane.Buffer.Metric.ByteSize","doc":"Implementation of `Membrane.Buffer.Metric` for the `:bytes` unit","ref":"Membrane.Buffer.Metric.ByteSize.html"},{"type":"module","title":"Membrane.Buffer.Metric.Count","doc":"Implementation of `Membrane.Buffer.Metric` for the `:buffers` unit","ref":"Membrane.Buffer.Metric.Count.html"},{"type":"module","title":"Membrane.Event.Discontinuity","doc":"Generic discontinuity event.\n\nThis event means that flow of buffers in the stream was interrupted, but stream\nitself is not done.\n\nFrequent reasons for this are soundcards drops while capturing sound, network\ndata loss etc.\n\nIf duration of the discontinuity is known, it can be passed as an argument.","ref":"Membrane.Event.Discontinuity.html"},{"type":"type","title":"Membrane.Event.Discontinuity.duration/0","doc":"","ref":"Membrane.Event.Discontinuity.html#t:duration/0"},{"type":"type","title":"Membrane.Event.Discontinuity.t/0","doc":"Generic discontinuity event.\n\nThis event means that flow of buffers in the stream was interrupted, but stream\nitself is not done.\n\nFrequent reasons for this are soundcards drops while capturing sound, network\ndata loss etc.\n\nIf duration of the discontinuity is known, it can be passed as an argument.","ref":"Membrane.Event.Discontinuity.html#t:t/0"},{"type":"module","title":"Membrane.Event.Underrun","doc":"Generic underrun event.\n\nThis event means that certain element is willing to consume more buffers,\nbut there are none available.\n\nIt makes sense to use this event as an upstream event to notify previous\nelements in the pipeline that they should generate more buffers.","ref":"Membrane.Event.Underrun.html"},{"type":"type","title":"Membrane.Event.Underrun.t/0","doc":"Generic underrun event.\n\nThis event means that certain element is willing to consume more buffers,\nbut there are none available.\n\nIt makes sense to use this event as an upstream event to notify previous\nelements in the pipeline that they should generate more buffers.","ref":"Membrane.Event.Underrun.html#t:t/0"},{"type":"module","title":"Membrane.EventProtocol.DefaultImpl","doc":"Default implementation of `Membrane.EventProtocol`.\n\nIf `use`d in `defimpl`, not implemented callbacks fallback to default ones.","ref":"Membrane.EventProtocol.DefaultImpl.html"},{"type":"behaviour","title":"Membrane.Payload.Behaviour","doc":"Behaviour that should be implemented by every module that has\n`Membrane.Payload` protocol implementation.","ref":"Membrane.Payload.Behaviour.html"},{"type":"callback","title":"Membrane.Payload.Behaviour.empty/0","doc":"Creates an empty payload","ref":"Membrane.Payload.Behaviour.html#c:empty/0"},{"type":"callback","title":"Membrane.Payload.Behaviour.new/1","doc":"Creates a new payload initialized with the given binary","ref":"Membrane.Payload.Behaviour.html#c:new/1"},{"type":"module","title":"Membrane.Payload.Binary","doc":"`Membrane.Payload.Behaviour` implementation for binary payload.\nComplements `Membrane.Payload` protocol implementation.","ref":"Membrane.Payload.Binary.html"},{"type":"module","title":"Membrane.Logger","doc":"Wrapper around the Elixir logger. Adds Membrane prefixes and handles verbose logging.","ref":"Membrane.Logger.html"},{"type":"module","title":"Prefixes - Membrane.Logger","doc":"By default, this wrapper prepends each log with a prefix containing the context\nof the log, such as element name. This can be turned off via configuration:\n\n    use Mix.Config\n    config :membrane_core, :logger, prefix: false\n\nRegardless of the config, the prefix is passed to `Logger` metadata under `:mb_prefix` key.\nPrefixes are passed via process dictionary, so they have process-wide scope,\nbut it can be extended with `get_prefix/0` and `set_prefix/1`.","ref":"Membrane.Logger.html#module-prefixes"},{"type":"module","title":"Verbose logging - Membrane.Logger","doc":"For verbose debug logs that should be silenced by default, use `debug_verbose/2`\nmacro. Verbose logs are purged in the compile time, unless turned on via configuration:\n\n    use Mix.Config\n    config :membrane_core, :logger, verbose: true\n\nVerbose debugs should be used for logs that are USUALLY USEFUL for debugging,\nbut printed so often that they make the output illegible. For example, it may\nbe a good idea to debug_verbose from within `c:Membrane.WithInputPads.handle_buffer/4`\nor `c:Membrane.Element.WithOutputPads.handle_demand/5` callbacks.","ref":"Membrane.Logger.html#module-verbose-logging"},{"type":"function","title":"Membrane.Logger.bare_log/3","doc":"Wrapper around `Logger.bare_log/3` that adds Membrane prefix.\n\nFor details, see the 'prefixes' section of the moduledoc.","ref":"Membrane.Logger.html#bare_log/3"},{"type":"macro","title":"Membrane.Logger.debug/2","doc":"Wrapper around `Logger.debug/2` that adds Membrane prefix.\n\nFor details, see the ['prefixes'](#module-prefixes) section of the moduledoc.","ref":"Membrane.Logger.html#debug/2"},{"type":"macro","title":"Membrane.Logger.debug_verbose/2","doc":"Macro for verbose debug logs, that are silenced by default.\n\nFor details, see the ['verbose logging'](#module-verbose-logging) section of the moduledoc.","ref":"Membrane.Logger.html#debug_verbose/2"},{"type":"macro","title":"Membrane.Logger.error/2","doc":"Wrapper around `Logger.error/2` that adds Membrane prefix.\n\nFor details, see the ['prefixes'](#module-prefixes) section of the moduledoc.","ref":"Membrane.Logger.html#error/2"},{"type":"function","title":"Membrane.Logger.get_config/0","doc":"Returns the Membrane logger config.","ref":"Membrane.Logger.html#get_config/0"},{"type":"function","title":"Membrane.Logger.get_config/2","doc":"Returns value at given key in the Membrane logger config.","ref":"Membrane.Logger.html#get_config/2"},{"type":"function","title":"Membrane.Logger.get_prefix/0","doc":"Returns the logger prefix.\n\nReturns an empty string if no prefix is set.","ref":"Membrane.Logger.html#get_prefix/0"},{"type":"macro","title":"Membrane.Logger.info/2","doc":"Wrapper around `Logger.info/2` that adds Membrane prefix.\n\nFor details, see the ['prefixes'](#module-prefixes) section of the moduledoc.","ref":"Membrane.Logger.html#info/2"},{"type":"macro","title":"Membrane.Logger.log/3","doc":"Wrapper around `Logger.log/3` that adds Membrane prefix.\n\nFor details, see the ['prefixes'](#module-prefixes) section of the moduledoc.","ref":"Membrane.Logger.html#log/3"},{"type":"function","title":"Membrane.Logger.set_prefix/1","doc":"Sets the logger prefix. Avoid using in Membrane-managed processes.\n\nThis function is intended to enable setting prefix obtained in a Membrane-managed\nprocess via `get_prefix/1`. If some custom data needs to be prepended to logs,\nplease use `Logger.metadata/1`.\n\nPrefixes in Membrane-managed processes are set automatically and using this\nfunction there would overwrite them, which is usually unintended.","ref":"Membrane.Logger.html#set_prefix/1"},{"type":"macro","title":"Membrane.Logger.warn/2","doc":"Wrapper around `Logger.warn/2` that adds Membrane prefix.\n\nFor details, see the ['prefixes'](#module-prefixes) section of the moduledoc.","ref":"Membrane.Logger.html#warn/2"},{"type":"macro","title":"Membrane.Logger.warning/2","doc":"Wrapper around `Logger.warning/2` that adds Membrane prefix.\n\nFor details, see the ['prefixes'](#module-prefixes) section of the moduledoc.","ref":"Membrane.Logger.html#warning/2"},{"type":"module","title":"Membrane.Testing.Assertions","doc":"This module contains a set of assertion functions and macros.\n\nThese assertions will work ONLY in conjunction with\n`Membrane.Testing.Pipeline` and ONLY when pid of tested pipeline is provided\nas an argument to these assertions.","ref":"Membrane.Testing.Assertions.html"},{"type":"macro","title":"Membrane.Testing.Assertions.assert_child_pad_removed/4","doc":"Asserts that `Membrane.Testing.Pipeline` child with name `child` removed or is going to\nremove it's pad with ref `pad` within the `timeout` period specified in milliseconds.","ref":"Membrane.Testing.Assertions.html#assert_child_pad_removed/4"},{"type":"macro","title":"Membrane.Testing.Assertions.assert_end_of_stream/4","doc":"Asserts that `Membrane.Testing.Pipeline` received or is going to receive end_of_stream\nnotification about from the element with a name `element_name` within the `timeout` period\nspecified in milliseconds.\n\n    assert_end_of_stream(pipeline, :an_element)","ref":"Membrane.Testing.Assertions.html#assert_end_of_stream/4"},{"type":"macro","title":"Membrane.Testing.Assertions.assert_pipeline_crash_group_down/3","doc":"Asserts that a crash group within pipeline will be down within the `timeout` period specified in\nmilliseconds.\n\nUsage example:\n\n    assert_pipeline_crash_group_down(pipeline, :group_1)","ref":"Membrane.Testing.Assertions.html#assert_pipeline_crash_group_down/3"},{"type":"macro","title":"Membrane.Testing.Assertions.assert_pipeline_notified/4","doc":"Asserts that pipeline received or will receive a notification from the element\nwith name `element_name` within the `timeout` period specified in milliseconds.\n\nThe `notification_pattern` must be a match pattern.\n\n    assert_pipeline_notified(pipeline, :element_name, :hi)","ref":"Membrane.Testing.Assertions.html#assert_pipeline_notified/4"},{"type":"macro","title":"Membrane.Testing.Assertions.assert_pipeline_receive/3","doc":"Asserts that pipeline received or will receive a message matching\n`message_pattern` from another process within the `timeout` period specified\nin milliseconds.\n\nThe `message_pattern` must be a match pattern.\n\n    assert_pipeline_receive(pid, :tick)\n\nSuch call would flunk if the message `:tick` has not been handled by\n`c:Membrane.Parent.handle_info/3` within the `timeout`.","ref":"Membrane.Testing.Assertions.html#assert_pipeline_receive/3"},{"type":"macro","title":"Membrane.Testing.Assertions.assert_pipeline_setup/2","doc":"","ref":"Membrane.Testing.Assertions.html#assert_pipeline_setup/2"},{"type":"macro","title":"Membrane.Testing.Assertions.assert_resource_guard_cleanup/3","doc":"Asserts that `Membrane.Testing.MockResourceGuard` was requested to cleanup a given tag.","ref":"Membrane.Testing.Assertions.html#assert_resource_guard_cleanup/3"},{"type":"macro","title":"Membrane.Testing.Assertions.assert_resource_guard_register/4","doc":"Asserts that a cleanup function was registered in `Membrane.Testing.MockResourceGuard`.","ref":"Membrane.Testing.Assertions.html#assert_resource_guard_register/4"},{"type":"macro","title":"Membrane.Testing.Assertions.assert_resource_guard_unregister/3","doc":"Asserts that a tag was unregistered in `Membrane.Testing.MockResourceGuard`.","ref":"Membrane.Testing.Assertions.html#assert_resource_guard_unregister/3"},{"type":"macro","title":"Membrane.Testing.Assertions.assert_sink_buffer/4","doc":"Asserts that `Membrane.Testing.Sink` with name `sink_name` received or will\nreceive a buffer matching `pattern` within the `timeout` period specified in\nmilliseconds.\n\nWhen the `Membrane.Testing.Sink` is a part of `Membrane.Testing.Pipeline` you\ncan assert whether it received a buffer matching provided pattern.\n    import Membrane.ChildrenSpec\n    spec = [\n        ...\n        |> child(:the_sink, %Membrane.Testing.Sink{}) |>\n        ...\n    ]\n    {:ok, pid} = Membrane.Testing.Pipeline.start_link(\n      spec: spec,\n    )\n\nYou can match for exact value:\n\n    assert_sink_buffer(pid, :the_sink ,%Membrane.Buffer{payload: ^specific_payload})\n\nYou can also use pattern to extract data from the buffer:\n\n    assert_sink_buffer(pid, :sink, %Buffer{payload:  >    >})\n    do_something(data)","ref":"Membrane.Testing.Assertions.html#assert_sink_buffer/4"},{"type":"macro","title":"Membrane.Testing.Assertions.assert_sink_event/4","doc":"Asserts that `Membrane.Testing.Sink` with name `sink_name` received or will\nreceive an event within the `timeout` period specified in milliseconds.\n\nWhen a `Membrane.Testing.Sink` is part of `Membrane.Testing.Pipeline` you can\nassert whether it received an event matching a provided pattern.\n\n    assert_sink_event(pid, :the_sink, %Discontinuity{})","ref":"Membrane.Testing.Assertions.html#assert_sink_event/4"},{"type":"macro","title":"Membrane.Testing.Assertions.assert_sink_playing/3","doc":"Asserts that `Membrane.Testing.Sink` with name `sink_name` entered the playing\nplayback.","ref":"Membrane.Testing.Assertions.html#assert_sink_playing/3"},{"type":"macro","title":"Membrane.Testing.Assertions.assert_sink_stream_format/4","doc":"Asserts that `Membrane.Testing.Sink` with name `sink_name` received or will\nreceive stream format matching `pattern` within the `timeout` period specified in\nmilliseconds.\n\nWhen the `Membrane.Testing.Sink` is a part of `Membrane.Testing.Pipeline` you\ncan assert whether it received stream format matching provided pattern.\n    import Membrane.ChildrenSpec\n    children = [\n        ...,\n        child(:the_sink, %Membrane.Testing.Sink{})\n    ]\n    {:ok, pid} = Membrane.Testing.Pipeline.start_link(\n      spec: children,\n    )\n\nYou can match for exact value:\n\n    assert_sink_stream_format(pid, :the_sink , %StreamFormat{prop: ^value})\n\nYou can also use pattern to extract data from the stream_format:\n\n    assert_sink_stream_format(pid, :the_sink , %StreamFormat{prop: value})\n    do_something(value)","ref":"Membrane.Testing.Assertions.html#assert_sink_stream_format/4"},{"type":"macro","title":"Membrane.Testing.Assertions.assert_start_of_stream/4","doc":"Asserts that `Membrane.Testing.Pipeline` received or is going to receive start_of_stream\nnotification from the element with a name `element_name` within the `timeout` period\nspecified in milliseconds.\n\n    assert_start_of_stream(pipeline, :an_element)","ref":"Membrane.Testing.Assertions.html#assert_start_of_stream/4"},{"type":"macro","title":"Membrane.Testing.Assertions.refute_pipeline_crash_group_down/3","doc":"Refutes that a crash group within pipeline won't be down within the `timeout` period specified in\nmilliseconds.\n\nUsage example:\n\n    refute_pipeline_crash_group_down(pipeline, :group_1)","ref":"Membrane.Testing.Assertions.html#refute_pipeline_crash_group_down/3"},{"type":"macro","title":"Membrane.Testing.Assertions.refute_pipeline_notified/4","doc":"Refutes that pipeline received or will receive a notification from the element\nwith name `element_name` within the `timeout` period specified in milliseconds.\n\nThe `notification_pattern` must be a match pattern.\n\n    refute_pipeline_notified(pipeline, :element_name, :hi)","ref":"Membrane.Testing.Assertions.html#refute_pipeline_notified/4"},{"type":"macro","title":"Membrane.Testing.Assertions.refute_pipeline_receive/3","doc":"Asserts that pipeline has not received and will not receive a message from\nanother process matching `message_pattern` within the `timeout` period\nspecified in milliseconds.\n\nThe `message_pattern` must be a match pattern.\n\n\n    refute_pipeline_receive(pid, :tick)\n\n\nSuch call would flunk if the message `:tick` has been handled by\n`c:Membrane.Parent.handle_info/3`.","ref":"Membrane.Testing.Assertions.html#refute_pipeline_receive/3"},{"type":"macro","title":"Membrane.Testing.Assertions.refute_sink_buffer/4","doc":"Asserts that `Membrane.Testing.Sink` with name `sink_name` has not received\nand will not receive a buffer matching `buffer_pattern` within the `timeout`\nperiod specified in milliseconds.\n\nSimilarly as in the `assert_sink_buffer/4` `the_sink` needs to be part of a\n`Membrane.Testing.Pipeline`.\n\n    refute_sink_buffer(pipeline, :the_sink, %Buffer{payload:  >})\n\nSuch expression will flunk if `the_sink` received or will receive a buffer\nwith payload ` >`.","ref":"Membrane.Testing.Assertions.html#refute_sink_buffer/4"},{"type":"macro","title":"Membrane.Testing.Assertions.refute_sink_event/4","doc":"Asserts that `Membrane.Testing.Sink` has not received and will not receive\nevent matching provided pattern within the `timeout` period specified in\nmilliseconds.\n\n    refute_sink_event(pid, :the_sink, %Discontinuity{})","ref":"Membrane.Testing.Assertions.html#refute_sink_event/4"},{"type":"macro","title":"Membrane.Testing.Assertions.refute_sink_playing/3","doc":"Asserts that `Membrane.Testing.Sink` with name `sink_name` didn't enter the playing\nplayback.","ref":"Membrane.Testing.Assertions.html#refute_sink_playing/3"},{"type":"macro","title":"Membrane.Testing.Assertions.refute_sink_stream_format/4","doc":"Asserts that `Membrane.Testing.Sink` with name `sink_name` has not received\nand will not receive stream format matching `stream_format_pattern` within the `timeout`\nperiod specified in milliseconds.\n\nSimilarly as in the `assert_sink_stream_format/4` `the_sink` needs to be part of a\n`Membrane.Testing.Pipeline`.\n\n    refute_sink_stream_format(pipeline, :the_sink, %StreamFormat{prop: ^val})\n\nSuch expression will flunk if `the_sink` received or will receive stream_format with\nproperty equal to value of `val` variable.","ref":"Membrane.Testing.Assertions.html#refute_sink_stream_format/4"},{"type":"module","title":"Membrane.Testing.DynamicSource","doc":"Testing Element for supplying data based on generator function or payloads passed\nthrough options. It is very similar to `Membrane.Testing.Source` but is has dynamic\npad instead of static.","ref":"Membrane.Testing.DynamicSource.html"},{"type":"module","title":"Example usage - Membrane.Testing.DynamicSource","doc":"As mentioned earlier you can use this element in one of two ways, providing\neither a generator function or an `Enumerable.t`.\n\nIf you provide an `Enumerable.t` with payloads, then each of those payloads will\nbe wrapped in a `Membrane.Buffer` and sent through `:output` pad.\n```\n%Source{output: [0xA1, 0xB2, 0xC3, 0xD4]}\n```\n\nIn order to specify `Membrane.Testing.Source` with generator function you need\nto provide initial state and function that matches `t:generator/0` type. This\nfunction should take state and demand size as its arguments and return\na tuple consisting of actions that element will return during the\n`c:Membrane.Element.WithOutputPads.handle_demand/5`\ncallback and new state.\n\n```\ngenerator_function = fn state, pad, size ->\n  #generate some buffers\n  {actions, state + 1}\nend\n\n%Source{output: {1, generator_function}}\n```","ref":"Membrane.Testing.DynamicSource.html#module-example-usage"},{"type":"module","title":"Element options - Membrane.Testing.DynamicSource","doc":"Passed via struct `t:Membrane.Testing.DynamicSource.t/0`\n\n- `output`  \n\n  ```\n  {initial_state :: any(), generator()} | Enum.t()\n  ```\n  \n  Default value: `{0, &Membrane.Testing.DynamicSource.default_buf_gen/3}`  \n  If `output` is an enumerable with `t:Membrane.Payload.t/0` then\n  buffer containing those payloads will be sent through the\n  `:output` pad and followed by `t:Membrane.Element.Action.end_of_stream/0`.\n  \n  If `output` is a `{initial_state, function}` tuple then the\n  the function will be invoked each time `handle_demand` is called.\n  It is an action generator that takes two arguments.\n  The first argument is the state that is initially set to\n  `initial_state`. The second one defines the pad on which the demand\n  has been requested. The third one defines the size of the demand.\n  Such function should return `{actions, next_state}` where\n  `actions` is a list of actions that will be returned from\n  `handle_demand/4` and `next_state` is the value that will be\n  used for the next call.\n\n\n- `stream_format`  \n\n  ```\n  struct()\n  ```\n  \n  Default value: `%Membrane.RemoteStream{content_format: nil, type: :bytestream}`  \n  Stream format to be sent before the `output`.","ref":"Membrane.Testing.DynamicSource.html#module-element-options"},{"type":"module","title":"Pads - Membrane.Testing.DynamicSource","doc":"### `:output`\n\nAccepted formats:\n```\n_any\n```\n\nDirection: | `:output`\nAvailability: | `:on_request`\nFlow control: | `:manual`\nDemand unit: | `nil`","ref":"Membrane.Testing.DynamicSource.html#module-pads"},{"type":"function","title":"Membrane.Testing.DynamicSource.default_buf_gen/3","doc":"","ref":"Membrane.Testing.DynamicSource.html#default_buf_gen/3"},{"type":"function","title":"Membrane.Testing.DynamicSource.options/0","doc":"Returns description of options available for this module","ref":"Membrane.Testing.DynamicSource.html#options/0"},{"type":"type","title":"Membrane.Testing.DynamicSource.generator/0","doc":"","ref":"Membrane.Testing.DynamicSource.html#t:generator/0"},{"type":"type","title":"Membrane.Testing.DynamicSource.t/0","doc":"Struct containing options for `Membrane.Testing.DynamicSource`","ref":"Membrane.Testing.DynamicSource.html#t:t/0"},{"type":"module","title":"Membrane.Testing.Endpoint","doc":"","ref":"Membrane.Testing.Endpoint.html"},{"type":"function","title":"Membrane.Testing.Endpoint.default_buf_gen/2","doc":"","ref":"Membrane.Testing.Endpoint.html#default_buf_gen/2"},{"type":"function","title":"Membrane.Testing.Endpoint.options/0","doc":"Returns description of options available for this module","ref":"Membrane.Testing.Endpoint.html#options/0"},{"type":"function","title":"Membrane.Testing.Endpoint.output_from_buffers/1","doc":"Creates output with generator function from list of buffers.","ref":"Membrane.Testing.Endpoint.html#output_from_buffers/1"},{"type":"type","title":"Membrane.Testing.Endpoint.generator/0","doc":"","ref":"Membrane.Testing.Endpoint.html#t:generator/0"},{"type":"type","title":"Membrane.Testing.Endpoint.t/0","doc":"Struct containing options for `Membrane.Testing.Endpoint`","ref":"Membrane.Testing.Endpoint.html#t:t/0"},{"type":"module","title":"Membrane.Testing.Event","doc":"Empty event that can be used in tests","ref":"Membrane.Testing.Event.html"},{"type":"module","title":"Membrane.Testing.MockResourceGuard","doc":"Mock for `Membrane.ResourceGuard`.\n\nInforms the test process about registered cleanup functions and tags.\nWorks with `Membrane.Testing.Assertions`, for example\n`Membrane.Testing.Assertions.assert_resource_guard_register/4`:\n\n    iex> guard = Membrane.Testing.MockResourceGuard.start_link_supervised!()\n    ...> Membrane.ResourceGuard.register(guard, fn -> :abc end, tag: :some_tag)\n    ...> import Membrane.Testing.Assertions\n    ...> assert_resource_guard_register(guard, function, :some_tag)\n    ...> function.()\n    :abc","ref":"Membrane.Testing.MockResourceGuard.html"},{"type":"function","title":"Membrane.Testing.MockResourceGuard.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Membrane.Testing.MockResourceGuard.html#child_spec/1"},{"type":"function","title":"Membrane.Testing.MockResourceGuard.start_link/1","doc":"","ref":"Membrane.Testing.MockResourceGuard.html#start_link/1"},{"type":"function","title":"Membrane.Testing.MockResourceGuard.start_link_supervised!/1","doc":"","ref":"Membrane.Testing.MockResourceGuard.html#start_link_supervised!/1"},{"type":"type","title":"Membrane.Testing.MockResourceGuard.options/0","doc":"","ref":"Membrane.Testing.MockResourceGuard.html#t:options/0"},{"type":"module","title":"Membrane.Testing.Pipeline","doc":"This Pipeline was created to reduce testing boilerplate and ease communication\nwith its children. It also provides a utility for informing testing process about\nplayback changes and received notifications.\n\nWhen you want a build Pipeline to test your children you need three things:\n - Pipeline Module\n - List of children\n - Links between those children\n\nTo start a testing pipeline you need to build\na keyword list representing the options used to determine the pipeline's behaviour and then\npass that options list to the `Membrane.Testing.Pipeline.start_link_supervised!/1`.\nThe testing pipeline can be started in one of two modes - either with its `:default` behaviour, or by\ninjecting a custom module behaviour. The usage of a `:default` pipeline implementation is presented below:\n\n```\nlinks = [\n    child(:el1, MembraneElement1) |>\n    child(:el2, MembraneElement2)\n    ...\n]\noptions =  [\n  module: :default # :default is the default value for this parameter, so you do not need to pass it here\n  spec: links\n]\npipeline = Membrane.Testing.Pipeline.start_link_supervised!(options)\n```\n\nYou can also pass your custom pipeline's module as a `:module` option of\nthe options list. Every callback of the module\nwill be executed before the callbacks of Testing.Pipeline.\nPassed module has to return a proper spec. There should be no children\nnor links specified in options passed to test pipeline as that would\nresult in a failure.\n\n```\noptions = [\n  module: Your.Module\n]\npipeline = Membrane.Testing.Pipeline.start_link_supervised!(options)\n```\n\nSee `t:Membrane.Testing.Pipeline.options/0` for available options.","ref":"Membrane.Testing.Pipeline.html"},{"type":"module","title":"Assertions - Membrane.Testing.Pipeline","doc":"This pipeline is designed to work with `Membrane.Testing.Assertions`. Check\nthem out or see example below for more details.","ref":"Membrane.Testing.Pipeline.html#module-assertions"},{"type":"module","title":"Messaging children - Membrane.Testing.Pipeline","doc":"You can send messages to children using their names specified in the children\nlist. Please check `notify_child/3` for more details.","ref":"Membrane.Testing.Pipeline.html#module-messaging-children"},{"type":"module","title":"Example usage - Membrane.Testing.Pipeline","doc":"Firstly, we can start the pipeline providing its options as a keyword list:\n  import Membrane.ChildrenSpec\n  children = [\n      child(source, %Membrane.Testing.Source{} ) |>\n      child(:tested_element, TestedElement) |>\n      child(sink, %Membrane.Testing.Sink{})\n  ]\n  {:ok, pipeline} = Membrane.Testing.Pipeline.start_link(spec: links)\n\nWe can now wait till the end of the stream reaches the sink element (don't forget\nto import `Membrane.Testing.Assertions`):\n\n    assert_end_of_stream(pipeline, :sink)\n\nWe can also assert that the `Membrane.Testing.Sink` processed a specific\nbuffer:\n\n    assert_sink_buffer(pipeline, :sink, %Membrane.Buffer{payload: 1})","ref":"Membrane.Testing.Pipeline.html#module-example-usage"},{"type":"function","title":"Membrane.Testing.Pipeline.child_spec/1","doc":"Returns child specification for spawning under a supervisor","ref":"Membrane.Testing.Pipeline.html#child_spec/1"},{"type":"function","title":"Membrane.Testing.Pipeline.execute_actions/2","doc":"Executes specified actions in the pipeline.\n\nThe actions are returned from the `handle_info` callback.","ref":"Membrane.Testing.Pipeline.html#execute_actions/2"},{"type":"function","title":"Membrane.Testing.Pipeline.get_child_pid/2","doc":"Returns the pid of the children process.\n\nAccepts pipeline pid as a first argument and a child reference or a list\nof child references representing a path as a second argument.\n\nIf second argument is a child reference, function gets pid of this child\nfrom pipeline.\n\nIf second argument is a path of child references, function gets pid of\nlast a component pointed by this path.\n\nReturns\n * `{:ok, child_pid}`, if a child was succesfully found\n * `{:error, reason}`, if, for example, pipeline is not alive or children path is invalid","ref":"Membrane.Testing.Pipeline.html#get_child_pid/2"},{"type":"function","title":"Membrane.Testing.Pipeline.get_child_pid!/2","doc":"Returns the pid of the children process.\n\nWorks as get_child_pid/2, but raises an error instead of returning\n`{:error, reason}` tuple.","ref":"Membrane.Testing.Pipeline.html#get_child_pid!/2"},{"type":"function","title":"Membrane.Testing.Pipeline.message_child/3","doc":"Deprecated since `v1.1.0-rc0`, use `notify_child/3` instead.\n\nSends message to a child by Element name.","ref":"Membrane.Testing.Pipeline.html#message_child/3"},{"type":"function","title":"Example - Membrane.Testing.Pipeline.message_child/3","doc":"Knowing that `pipeline` has child named `sink`, message can be sent as follows:\n\n    message_child(pipeline, :sink, {:message, \"to handle\"})","ref":"Membrane.Testing.Pipeline.html#message_child/3-example"},{"type":"function","title":"Membrane.Testing.Pipeline.notify_child/3","doc":"Sends notification to a child by Element name.","ref":"Membrane.Testing.Pipeline.html#notify_child/3"},{"type":"function","title":"Example - Membrane.Testing.Pipeline.notify_child/3","doc":"Knowing that `pipeline` has child named `sink`, notification can be sent as follows:\n\n    notify_child(pipeline, :sink, {:notification, \"to handle\"})","ref":"Membrane.Testing.Pipeline.html#notify_child/3-example"},{"type":"function","title":"Membrane.Testing.Pipeline.start/1","doc":"","ref":"Membrane.Testing.Pipeline.html#start/1"},{"type":"function","title":"Membrane.Testing.Pipeline.start_link/1","doc":"","ref":"Membrane.Testing.Pipeline.html#start_link/1"},{"type":"function","title":"Membrane.Testing.Pipeline.start_link_supervised/1","doc":"Starts the pipeline under the ExUnit test supervisor and links it to the current process.\n\nCan be used only in tests.","ref":"Membrane.Testing.Pipeline.html#start_link_supervised/1"},{"type":"function","title":"Membrane.Testing.Pipeline.start_link_supervised!/1","doc":"","ref":"Membrane.Testing.Pipeline.html#start_link_supervised!/1"},{"type":"function","title":"Membrane.Testing.Pipeline.start_supervised/1","doc":"Starts the pipeline under the ExUnit test supervisor.\n\nCan be used only in tests.","ref":"Membrane.Testing.Pipeline.html#start_supervised/1"},{"type":"function","title":"Membrane.Testing.Pipeline.start_supervised!/1","doc":"","ref":"Membrane.Testing.Pipeline.html#start_supervised!/1"},{"type":"function","title":"Membrane.Testing.Pipeline.terminate/2","doc":"","ref":"Membrane.Testing.Pipeline.html#terminate/2"},{"type":"type","title":"Membrane.Testing.Pipeline.options/0","doc":"","ref":"Membrane.Testing.Pipeline.html#t:options/0"},{"type":"module","title":"Membrane.Testing.Sink","doc":"Sink Element that notifies the pipeline about buffers and events it receives.\n\nBy default `Sink` will demand buffers automatically, but you can override that\nbehaviour by using `autodemand` option. If set to false no automatic demands\nshall be made. Demands can be then triggered by sending `{:make_demand, size}`\nmessage.\n\nThis element can be used in conjunction with `Membrane.Testing.Pipeline` to\nenable asserting on buffers and events it receives.\n\n    alias Membrane.Testing\n    links = [\n        ... |>\n        child(:sink, %Testing.Sink{}) |>\n        ...\n    ]\n    {:ok, pid} = Testing.Pipeline.start_link(\n      spec: links\n    )\n\nAsserting that `Membrane.Testing.Sink` element processed a buffer that matches\na specific pattern can be achieved using\n`Membrane.Testing.Assertions.assert_sink_buffer/3`.\n\n    assert_sink_buffer(pid, :sink ,%Membrane.Buffer{payload: 255})","ref":"Membrane.Testing.Sink.html"},{"type":"module","title":"Element options - Membrane.Testing.Sink","doc":"Passed via struct `t:Membrane.Testing.Sink.t/0`\n\n- `autodemand`  \n\n  ```\n  boolean()\n  ```\n  \n  Default value: `true`  \n  If true element will automatically make demands.\n  If it is set to false demand has to be triggered manually by sending `:make_demand` message.","ref":"Membrane.Testing.Sink.html#module-element-options"},{"type":"module","title":"Pads - Membrane.Testing.Sink","doc":"### `:input`\n\nAccepted formats:\n```\n_any\n```\n\nDirection: | `:input`\nAvailability: | `:always`\nFlow control: | `:manual`\nDemand unit: | `:buffers`","ref":"Membrane.Testing.Sink.html#module-pads"},{"type":"function","title":"Membrane.Testing.Sink.options/0","doc":"Returns description of options available for this module","ref":"Membrane.Testing.Sink.html#options/0"},{"type":"type","title":"Membrane.Testing.Sink.t/0","doc":"Struct containing options for `Membrane.Testing.Sink`","ref":"Membrane.Testing.Sink.html#t:t/0"},{"type":"module","title":"Membrane.Testing.Source","doc":"Testing Element for supplying data based on generator function or payloads passed\nthrough options.","ref":"Membrane.Testing.Source.html"},{"type":"module","title":"Example usage - Membrane.Testing.Source","doc":"As mentioned earlier you can use this element in one of two ways, providing\neither a generator function or an `Enumerable.t`.\n\nIf you provide an `Enumerable.t` with payloads, then each of those payloads will\nbe wrapped in a `Membrane.Buffer` and sent through `:output` pad.\n```\n%Source{output: [0xA1, 0xB2, 0xC3, 0xD4]}\n```\n\nIn order to specify `Membrane.Testing.Source` with generator function you need\nto provide initial state and function that matches `t:generator/0` type. This\nfunction should take state and demand size as its arguments and return\na tuple consisting of actions that element will return during the\n`c:Membrane.Element.WithOutputPads.handle_demand/5`\ncallback and new state.\n```\ngenerator_function = fn state, size ->\n  #generate some buffers\n  {actions, state + 1}\nend\n%Source{output: {1, generator_function}}\n```","ref":"Membrane.Testing.Source.html#module-example-usage"},{"type":"module","title":"Element options - Membrane.Testing.Source","doc":"Passed via struct `t:Membrane.Testing.Source.t/0`\n\n- `output`  \n\n  ```\n  Enum.t() | {initial_state :: any(), generator}\n  ```\n  \n  Default value: `{0, &Membrane.Testing.Source.default_buf_gen/2}`  \n  If `output` is an Enumerable, then each element of it will be sent\n  through the `:output` pad, followed by `t:Membrane.Element.Action.end_of_stream/0`.\n  Each element of the Enumerable must be either `t:Membrane.Buffer.t/0`\n  or `t:Membrane.Payload.t/0`. In the latter case, it will be automatically wrapped into\n  `t:Membrane.Buffer.t/0` before sending.\n  \n  Otherwise, if `output` is a `{initial_state, function}` tuple then the\n  the function will be invoked each time `handle_demand` is called.\n  It is an action generator that takes two arguments.\n  The first argument is the state that is initially set to\n  `initial_state`. The second one defines the size of the demand.\n  Such function should return `{actions, next_state}` where\n  `actions` is a list of actions that will be returned from\n  `handle_demand/4` and `next_state` is the value that will be\n  used for the next call.\n\n\n- `stream_format`  \n\n  ```\n  struct()\n  ```\n  \n  Default value: `%Membrane.RemoteStream{content_format: nil, type: :bytestream}`  \n  StreamFormat to be sent before the `output`.","ref":"Membrane.Testing.Source.html#module-element-options"},{"type":"module","title":"Pads - Membrane.Testing.Source","doc":"### `:output`\n\nAccepted formats:\n```\n_any\n```\n\nDirection: | `:output`\nAvailability: | `:always`\nFlow control: | `:manual`\nDemand unit: | `nil`","ref":"Membrane.Testing.Source.html#module-pads"},{"type":"function","title":"Membrane.Testing.Source.default_buf_gen/2","doc":"","ref":"Membrane.Testing.Source.html#default_buf_gen/2"},{"type":"function","title":"Membrane.Testing.Source.options/0","doc":"Returns description of options available for this module","ref":"Membrane.Testing.Source.html#options/0"},{"type":"function","title":"Membrane.Testing.Source.output_from_buffers/1","doc":"Creates output with generator function from list of buffers.","ref":"Membrane.Testing.Source.html#output_from_buffers/1"},{"type":"type","title":"Membrane.Testing.Source.generator/0","doc":"","ref":"Membrane.Testing.Source.html#t:generator/0"},{"type":"type","title":"Membrane.Testing.Source.t/0","doc":"Struct containing options for `Membrane.Testing.Source`","ref":"Membrane.Testing.Source.html#t:t/0"},{"type":"module","title":"Membrane.Clock","doc":"Clock is a Membrane utility that allows elements to measure time according to\na particular clock, which can be e.g. a soundcard hardware clock.\n\nInternally, Clock is a GenServer process that can receive _updates_ (see `t:update_message/0`),\nwhich are messages containing amount of time until the next update.\nFor example, a sink playing audio to the sound card can send an update before\neach write to the sound card buffer (for practical reasons that can be done every\n100 or 1000 writes). Although it might be more intuitive to send updates with\nthe time passed, in practice the described approach turns out to be more convenient,\nas it simplifies the first update.\n\nBasing on updates, Clock calculates the `t:ratio/0` of its time to the reference\ntime. The reference time can be configured with `:time_provider` option. The ratio\nis broadcasted (see `t:ratio_message/0`) to _subscribers_ (see `subscribe/2`)\n- processes willing to synchronize to the custom clock. Subscribers can adjust\ntheir timers according to received ratio - timers started with\n`t:Membrane.Element.Action.start_timer/0` action in elements do it automatically.\nInitial ratio is equal to 1, which means that if no updates are received,\nClock is synchronized to the reference time.","ref":"Membrane.Clock.html"},{"type":"module","title":"Proxy mode - Membrane.Clock","doc":"Clock can work in _proxy_ mode, which means it cannot receive updates, but\nit receives ratio from another clock instead, and forwards it to subscribers.\nProxy mode is enabled with `proxy_for: pid` or `proxy: true` (no initial proxy)\noption, and the proxy is set/changed using `proxy_for/2`.","ref":"Membrane.Clock.html#module-proxy-mode"},{"type":"function","title":"Membrane.Clock.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Membrane.Clock.html#child_spec/1"},{"type":"function","title":"Membrane.Clock.proxy_for/2","doc":"Sets a new proxy clock to `clock_to_proxy_for`.","ref":"Membrane.Clock.html#proxy_for/2"},{"type":"function","title":"Membrane.Clock.start/2","doc":"","ref":"Membrane.Clock.html#start/2"},{"type":"function","title":"Membrane.Clock.start_link/2","doc":"","ref":"Membrane.Clock.html#start_link/2"},{"type":"function","title":"Membrane.Clock.subscribe/2","doc":"Subscribes `pid` for receiving `t:ratio_message/0` messages from the clock.\n\nThis function can be called multiple times from the same process. To unsubscribe,\n`unsubscribe/2` should be called the same amount of times. The subscribed pid\nalways receives one message, regardless of how many times it called `subscribe/2`.","ref":"Membrane.Clock.html#subscribe/2"},{"type":"function","title":"Membrane.Clock.unsubscribe/2","doc":"Unsubscribes `pid` from receiving `t:ratio_message/0` messages from the clock.\n\nFor unsubscription to take effect, `unsubscribe/2` should be called the same\namount of times as `subscribe/2`.","ref":"Membrane.Clock.html#unsubscribe/2"},{"type":"type","title":"Membrane.Clock.option/0","doc":"Options accepted by `start_link/2` and `start/2` functions.\n\nThey are the following:\n  - `time_provider` - function providing the reference time in milliseconds\n  - `proxy` - determines whether the Clock should work in proxy mode\n  - `proxy_for` - enables the proxy mode and sets proxied Clock to pid\n\nCheck the moduledoc for more details.","ref":"Membrane.Clock.html#t:option/0"},{"type":"type","title":"Membrane.Clock.ratio/0","doc":"Ratio of the Clock time to the reference time.","ref":"Membrane.Clock.html#t:ratio/0"},{"type":"type","title":"Membrane.Clock.ratio_message/0","doc":"Ratio message sent by the Clock to all its subscribers. It contains the ratio\nof the custom clock time to the reference time.","ref":"Membrane.Clock.html#t:ratio_message/0"},{"type":"type","title":"Membrane.Clock.t/0","doc":"Clock is a Membrane utility that allows elements to measure time according to\na particular clock, which can be e.g. a soundcard hardware clock.\n\nInternally, Clock is a GenServer process that can receive _updates_ (see `t:update_message/0`),\nwhich are messages containing amount of time until the next update.\nFor example, a sink playing audio to the sound card can send an update before\neach write to the sound card buffer (for practical reasons that can be done every\n100 or 1000 writes). Although it might be more intuitive to send updates with\nthe time passed, in practice the described approach turns out to be more convenient,\nas it simplifies the first update.\n\nBasing on updates, Clock calculates the `t:ratio/0` of its time to the reference\ntime. The reference time can be configured with `:time_provider` option. The ratio\nis broadcasted (see `t:ratio_message/0`) to _subscribers_ (see `subscribe/2`)\n- processes willing to synchronize to the custom clock. Subscribers can adjust\ntheir timers according to received ratio - timers started with\n`t:Membrane.Element.Action.start_timer/0` action in elements do it automatically.\nInitial ratio is equal to 1, which means that if no updates are received,\nClock is synchronized to the reference time.","ref":"Membrane.Clock.html#t:t/0"},{"type":"type","title":"Proxy mode - Membrane.Clock.t/0","doc":"Clock can work in _proxy_ mode, which means it cannot receive updates, but\nit receives ratio from another clock instead, and forwards it to subscribers.\nProxy mode is enabled with `proxy_for: pid` or `proxy: true` (no initial proxy)\noption, and the proxy is set/changed using `proxy_for/2`.","ref":"Membrane.Clock.html#t:t/0-proxy-mode"},{"type":"type","title":"Membrane.Clock.update_message/0","doc":"Update message received by the Clock. It should contain the time till the next\nupdate.","ref":"Membrane.Clock.html#t:update_message/0"},{"type":"module","title":"Membrane.ComponentPath","doc":"A list consisting of following pipeline/bin/element names down the assembled pipeline.\n\nIt traces element's path inside a pipeline.\nInformation is being stored in a process dictionary and can be set/appended to.","ref":"Membrane.ComponentPath.html"},{"type":"function","title":"Membrane.ComponentPath.format/1","doc":"Returns formatted string of given path's names.","ref":"Membrane.ComponentPath.html#format/1"},{"type":"function","title":"Membrane.ComponentPath.get/0","doc":"Returns currently stored path.\n\nIf path has not been set, empty list is returned.","ref":"Membrane.ComponentPath.html#get/0"},{"type":"function","title":"Membrane.ComponentPath.get_formatted/0","doc":"Works the same way as `format/1` but uses currently stored path.","ref":"Membrane.ComponentPath.html#get_formatted/0"},{"type":"function","title":"Membrane.ComponentPath.set/1","doc":"Sets current path.\n\nIf path had already existed then replaces it.","ref":"Membrane.ComponentPath.html#set/1"},{"type":"type","title":"Membrane.ComponentPath.path/0","doc":"A list consisting of following pipeline/bin/element names down the assembled pipeline.\n\nIt traces element's path inside a pipeline.\nInformation is being stored in a process dictionary and can be set/appended to.","ref":"Membrane.ComponentPath.html#t:path/0"},{"type":"module","title":"Membrane.Playback","doc":"Playback defines whether media is processed within a pipeline (`:playing`) or not (`:stopped`).\n\nComponent playback will not enter `:playing` state until its parent playback is playing and\nall components from the `spec`, that started this component have ended their setups and linked\ntheir pads.\n\nBy default, component setup ends with the end of `handle_setup/2` callback.\nIf `{:setup, :incomplete}` is returned there, setup lasts until `{:setup, :complete}`\nis returned from antoher callback.\n\nUntils the setup lasts, the component won't enter `:playing` playback.","ref":"Membrane.Playback.html"},{"type":"type","title":"Membrane.Playback.t/0","doc":"Playback defines whether media is processed within a pipeline (`:playing`) or not (`:stopped`).\n\nComponent playback will not enter `:playing` state until its parent playback is playing and\nall components from the `spec`, that started this component have ended their setups and linked\ntheir pads.\n\nBy default, component setup ends with the end of `handle_setup/2` callback.\nIf `{:setup, :incomplete}` is returned there, setup lasts until `{:setup, :complete}`\nis returned from antoher callback.\n\nUntils the setup lasts, the component won't enter `:playing` playback.","ref":"Membrane.Playback.html#t:t/0"},{"type":"module","title":"Membrane.ResourceGuard","doc":"Utility for handling resources that must be cleaned up after use.\n\nThis utility uses a separate process that allows registering functions\nthat are called when the owner process (passed to `start_link/1`) dies for\nany reason. Each Membrane component spawns its resource guard on startup\nand provides it via callback context.\n\n#","ref":"Membrane.ResourceGuard.html"},{"type":"module","title":"Example - Membrane.ResourceGuard","doc":"def handle_setup(ctx, state) do\n      resource = MyResource.create()\n\n      Membrane.ResourceGuard.register(ctx.resource_guard, fn ->\n        MyResource.cleanup(resource)\n      end)\n\n      {:ok, %{state | my_resource: resource}}\n    end","ref":"Membrane.ResourceGuard.html#module-example"},{"type":"function","title":"Membrane.ResourceGuard.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Membrane.ResourceGuard.html#child_spec/1"},{"type":"function","title":"Membrane.ResourceGuard.cleanup/1","doc":"Executes all cleanup functions registered in the resource gurard.","ref":"Membrane.ResourceGuard.html#cleanup/1"},{"type":"function","title":"Membrane.ResourceGuard.cleanup/2","doc":"Executes cleanup functions registered with the specifc tag.\n\nIf many cleanup functions are registered with the same tag, all of them are executed.","ref":"Membrane.ResourceGuard.html#cleanup/2"},{"type":"function","title":"Membrane.ResourceGuard.register/3","doc":"Registers a resource cleanup function in the resource guard.\n\nRegistered functions are called in the order reverse to the registration order.\nFunction returns a tag of the registered cleanup function. Tag can be passed\nunder a `:tag` key in `opts`. Many functions can be registered with the same tag.\nIf there is no `:tag` key in `opts`, tag will be result of `make_ref()`.","ref":"Membrane.ResourceGuard.html#register/3"},{"type":"function","title":"Membrane.ResourceGuard.start_link/1","doc":"","ref":"Membrane.ResourceGuard.html#start_link/1"},{"type":"function","title":"Membrane.ResourceGuard.unregister/2","doc":"Unregisters a resource cleanup function from the resource guard.\n\nAll cleanup functions with tag `tag` are deleted.","ref":"Membrane.ResourceGuard.html#unregister/2"},{"type":"type","title":"Membrane.ResourceGuard.t/0","doc":"Utility for handling resources that must be cleaned up after use","ref":"Membrane.ResourceGuard.html#t:t/0"},{"type":"module","title":"Membrane.Sync","doc":"Sync allows to synchronize multiple processes, so that they could perform their\njobs at the same time.\n\nThe main purpose for Sync is to synchronize multiple streams within a pipeline.\nThe flow of usage goes as follows:\n- A Sync process is started.\n- Processes register themselves (or are registered) in the Sync, using\n`register/2`. Registered processes are not being synchronized till the Sync\nbecomes active (see the next step). Each registered process is monitored and\nautomatically unregistered upon exit. Sync can be setup to exit when all the\nregistered processes exit by passing the `empty_exit?` option to `start_link/2`.\n- When all processes that need to be registered are registered, the Sync can\nbe activated with `activate/1` function. This disables registration and enables\nsynchronization.\n- Once a process needs to sync, it invokes `sync/2`, which results in blocking\nuntil all the registered processes invoke `sync/2`. This works only when the Sync\nis active - otherwise calling `sync/2` returns immediately.\n- Once all the ready processes invoke `sync/2`, the calls return, and they become\nregistered again.\n- When synchronization needs to be turned off, the Sync should be deactivated\nwith `deactivate/2`. This disables synchronization and enables registration again.\nAll the calls to `sync/2` return immediately.\n\nIf a process designed to work with Sync should not be synced, `no_sync/0` should\nbe used. Then all calls to `sync/2` return immediately.","ref":"Membrane.Sync.html"},{"type":"function","title":"Membrane.Sync.activate/1","doc":"","ref":"Membrane.Sync.html#activate/1"},{"type":"function","title":"Membrane.Sync.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Membrane.Sync.html#child_spec/1"},{"type":"function","title":"Membrane.Sync.deactivate/1","doc":"","ref":"Membrane.Sync.html#deactivate/1"},{"type":"function","title":"Membrane.Sync.no_sync/0","doc":"Returns a Sync that always returns immediately when calling `sync/2` on it.","ref":"Membrane.Sync.html#no_sync/0"},{"type":"function","title":"Membrane.Sync.register/2","doc":"","ref":"Membrane.Sync.html#register/2"},{"type":"function","title":"Membrane.Sync.start_link/2","doc":"Starts a Sync process linked to the current process.","ref":"Membrane.Sync.html#start_link/2"},{"type":"function","title":"Options - Membrane.Sync.start_link/2","doc":"- :empty_exit? - if true, Sync automatically exits when all the registered\n  processes exit; defaults to false","ref":"Membrane.Sync.html#start_link/2-options"},{"type":"function","title":"Membrane.Sync.sync/2","doc":"","ref":"Membrane.Sync.html#sync/2"},{"type":"type","title":"Membrane.Sync.status/0","doc":"","ref":"Membrane.Sync.html#t:status/0"},{"type":"type","title":"Membrane.Sync.t/0","doc":"A type describing the the synchronization mechanism, used to synchronize multiple\nprocesses, so that they could perform their jobs at the same time.","ref":"Membrane.Sync.html#t:t/0"},{"type":"module","title":"Membrane.Telemetry","doc":"Defines basic telemetry event types used by Membrane's Core.\n\nMembrane uses [Telemetry Package](https://hex.pm/packages/telemetry) for instrumentation and does not store or save any measurements by itself.\n\nIt is user's responsibility to use some sort of metrics reporter\nthat will be attached to `:telemetry` package to consume and process generated measurements.","ref":"Membrane.Telemetry.html"},{"type":"module","title":"Instrumentation - Membrane.Telemetry","doc":"The following events are published by Membrane's Core with following measurement types and metadata:\n\n  * `[:membrane, :metric, :value]` - used to report metrics, such as input buffer's size inside functions, incoming events and received stream format\n      * Measurement: `t:metric_event_value/0`\n      * Metadata: `%{}`\n\n  * `[:membrane, :link, :new]` - to report new link connection being initialized in pipeline.\n      * Measurement: `t:link_event_value/0`\n      * Metadata: `%{}`\n\n  * `[:membrane, :pipeline | :bin | :element, :init]` - to report pipeline/element/bin initialization\n      * Measurement: `t:init_or_terminate_event_value/0`\n      * Metadata: `%{log_metadata: keyword()}`, includes Logger's metadata of created component\n\n  * `[:membrane, :pipeline | :bin | :element, :terminate]` - to report pipeline/element/bin termination\n      * Measurement: `t:init_or_terminate_event_value/0`\n      * Metadata: `%{}`","ref":"Membrane.Telemetry.html#module-instrumentation"},{"type":"module","title":"Enabling certain metrics/events - Membrane.Telemetry","doc":"A lot of events can happen literally hundreds times per second such as registering that a buffer has been sent/processed.\n\nThis behaviour can come with a great performance penalties but may be helpful for certain discoveries. To avoid any runtime overhead\nwhen the reporting is not necessary all metrics/events are hidden behind a compile-time feature flags.\nTo enable a particular measurement one must recompile membrane core with the following snippet put inside\nuser's application `config.exs` file:\n\n    config :membrane_core,\n      telemetry_flags: [\n        :flag_name,\n        ...,\n        {:metrics, [list of metrics]}\n        ...\n      ]\n\nAvailable flags are (those are of a very low overhead):\n* `:links` - enables new links notifications\n* `:inits_and_terminates` - enables notifications of `init` and `terminate` events for elements/bins/pipelines\n\n\nAdditionally one can control which metric values should get measured by passing an option of format :\n`{:metrics, [list of metrics]}`\n\nAvailable metrics are:\n* `:buffer` - number of buffers being sent from a particular element\n* `:bitrate` - total number of bits carried by the buffers mentioned above\n* `:queue_len` - number of messages in element's message queue during GenServer's `handle_info` invocation\n* `:stream_format` - indicates that given element has received new stream format, value always equals '1'\n* `:event` - indicates that given element has received a new event, value always equals '1'\n* `:store` - reports the current size of a input buffer when storing a new buffer\n* `:take_and_demand` - reports the current size of a input buffer when taking buffers and making a new demand","ref":"Membrane.Telemetry.html#module-enabling-certain-metrics-events"},{"type":"type","title":"Membrane.Telemetry.event_name/0","doc":"","ref":"Membrane.Telemetry.html#t:event_name/0"},{"type":"type","title":"Membrane.Telemetry.init_or_terminate_event_value/0","doc":"* path - element's path","ref":"Membrane.Telemetry.html#t:init_or_terminate_event_value/0"},{"type":"type","title":"Membrane.Telemetry.link_event_value/0","doc":"* parent_path - process path of link's parent\n* from - from element name\n* to - to element name\n* pad_from - from's pad name\n* pad_to - to's pad name","ref":"Membrane.Telemetry.html#t:link_event_value/0"},{"type":"type","title":"Membrane.Telemetry.metric_event_value/0","doc":"* component_path - element's or bin's path\n* metric - metric's name\n* value - metric's value","ref":"Membrane.Telemetry.html#t:metric_event_value/0"},{"type":"module","title":"Membrane.Time","doc":"Module containing functions needed to perform handling of time.\n\nMembrane always internally uses nanosecond as a time unit. This is how all time\nunits should represented in the code unless there's a good reason to act\ndifferently.\n\nPlease note that Erlang VM may internally use different units and that may\ndiffer from platform to platform. Still, unless you need to perform calculations\nthat do not touch hardware clock, you should use Membrane units for consistency.","ref":"Membrane.Time.html"},{"type":"function","title":"Membrane.Time.as_days/2","doc":"Returns time in days.\n\nIf the `mode` argument is set to `:exact` (default), the result is\nrepresented as a rational number. Otherwise, if the `mode` is\n`:round`, the result is rounded to the nearest integer.","ref":"Membrane.Time.html#as_days/2"},{"type":"function","title":"Membrane.Time.as_hours/2","doc":"Returns time in hours.\n\nIf the `mode` argument is set to `:exact` (default), the result is\nrepresented as a rational number. Otherwise, if the `mode` is\n`:round`, the result is rounded to the nearest integer.","ref":"Membrane.Time.html#as_hours/2"},{"type":"function","title":"Membrane.Time.as_microseconds/2","doc":"Returns time in microseconds.\n\nIf the `mode` argument is set to `:exact` (default), the result is\nrepresented as a rational number. Otherwise, if the `mode` is\n`:round`, the result is rounded to the nearest integer.","ref":"Membrane.Time.html#as_microseconds/2"},{"type":"function","title":"Membrane.Time.as_milliseconds/2","doc":"Returns time in milliseconds.\n\nIf the `mode` argument is set to `:exact` (default), the result is\nrepresented as a rational number. Otherwise, if the `mode` is\n`:round`, the result is rounded to the nearest integer.","ref":"Membrane.Time.html#as_milliseconds/2"},{"type":"function","title":"Membrane.Time.as_minutes/2","doc":"Returns time in minutes.\n\nIf the `mode` argument is set to `:exact` (default), the result is\nrepresented as a rational number. Otherwise, if the `mode` is\n`:round`, the result is rounded to the nearest integer.","ref":"Membrane.Time.html#as_minutes/2"},{"type":"function","title":"Membrane.Time.as_nanoseconds/2","doc":"Returns time in nanoseconds.\n\nIf the `mode` argument is set to `:exact` (default), the result is\nrepresented as a rational number. Otherwise, if the `mode` is\n`:round`, the result is rounded to the nearest integer.","ref":"Membrane.Time.html#as_nanoseconds/2"},{"type":"function","title":"Membrane.Time.as_seconds/2","doc":"Returns time in seconds.\n\nIf the `mode` argument is set to `:exact` (default), the result is\nrepresented as a rational number. Otherwise, if the `mode` is\n`:round`, the result is rounded to the nearest integer.","ref":"Membrane.Time.html#as_seconds/2"},{"type":"function","title":"Membrane.Time.day/0","doc":"Returns one day in `Membrane.Time` units.","ref":"Membrane.Time.html#day/0"},{"type":"function","title":"Membrane.Time.days/1","doc":"Returns given amount of days in `Membrane.Time` units.","ref":"Membrane.Time.html#days/1"},{"type":"function","title":"Membrane.Time.divide_by_timebase/2","doc":"Divides timestamp by a timebase. The result is rounded to the nearest integer.","ref":"Membrane.Time.html#divide_by_timebase/2"},{"type":"function","title":"Examples: - Membrane.Time.divide_by_timebase/2","doc":"iex> timestamp = 10 |> Membrane.Time.seconds()\n    iex> timebase = Ratio.new(Membrane.Time.second(), 30)\n    iex> Membrane.Time.divide_by_timebase(timestamp, timebase)\n    300","ref":"Membrane.Time.html#divide_by_timebase/2-examples"},{"type":"function","title":"Membrane.Time.from_datetime/1","doc":"Converts `DateTime` to `Membrane.Time` units.","ref":"Membrane.Time.html#from_datetime/1"},{"type":"function","title":"Membrane.Time.from_iso8601!/1","doc":"Converts iso8601 string to `Membrane.Time` units.\nIf `value` is invalid, throws match error.","ref":"Membrane.Time.html#from_iso8601!/1"},{"type":"function","title":"Membrane.Time.from_ntp_timestamp/1","doc":"Converts NTP timestamp (time since 0h on 1st Jan 1900) into Unix timestamp\n(time since 1st Jan 1970) represented in `Membrane.Time` units.\n\nNTP timestamp uses fixed point representation with the integer part in the first 32 bits\nand the fractional part in the last 32 bits.","ref":"Membrane.Time.html#from_ntp_timestamp/1"},{"type":"function","title":"Membrane.Time.hour/0","doc":"Returns one hour in `Membrane.Time` units.","ref":"Membrane.Time.html#hour/0"},{"type":"function","title":"Membrane.Time.hours/1","doc":"Returns given amount of hours in `Membrane.Time` units.","ref":"Membrane.Time.html#hours/1"},{"type":"function","title":"Membrane.Time.inspect/1","doc":"Returns string representation of result of `to_code/1`.","ref":"Membrane.Time.html#inspect/1"},{"type":"macro","title":"Membrane.Time.is_time/1","doc":"Checks whether a value is `Membrane.Time.t`.","ref":"Membrane.Time.html#is_time/1"},{"type":"function","title":"Membrane.Time.microsecond/0","doc":"Returns one microsecond in `Membrane.Time` units.","ref":"Membrane.Time.html#microsecond/0"},{"type":"function","title":"Membrane.Time.microseconds/1","doc":"Returns given amount of microseconds in `Membrane.Time` units.","ref":"Membrane.Time.html#microseconds/1"},{"type":"function","title":"Membrane.Time.millisecond/0","doc":"Returns one millisecond in `Membrane.Time` units.","ref":"Membrane.Time.html#millisecond/0"},{"type":"function","title":"Membrane.Time.milliseconds/1","doc":"Returns given amount of milliseconds in `Membrane.Time` units.","ref":"Membrane.Time.html#milliseconds/1"},{"type":"function","title":"Membrane.Time.minute/0","doc":"Returns one minute in `Membrane.Time` units.","ref":"Membrane.Time.html#minute/0"},{"type":"function","title":"Membrane.Time.minutes/1","doc":"Returns given amount of minutes in `Membrane.Time` units.","ref":"Membrane.Time.html#minutes/1"},{"type":"function","title":"Membrane.Time.monotonic_time/0","doc":"Returns current monotonic time based on `System.monotonic_time/0`\nin `Membrane.Time` units.","ref":"Membrane.Time.html#monotonic_time/0"},{"type":"function","title":"Membrane.Time.nanosecond/0","doc":"Returns one nanosecond in `Membrane.Time` units.","ref":"Membrane.Time.html#nanosecond/0"},{"type":"function","title":"Membrane.Time.nanoseconds/1","doc":"Returns given amount of nanoseconds in `Membrane.Time` units.","ref":"Membrane.Time.html#nanoseconds/1"},{"type":"function","title":"Membrane.Time.native_unit/0","doc":"Returns one VM native unit in `Membrane.Time` units.","ref":"Membrane.Time.html#native_unit/0"},{"type":"function","title":"Membrane.Time.native_units/1","doc":"Returns given amount of VM native units in `Membrane.Time` units.","ref":"Membrane.Time.html#native_units/1"},{"type":"function","title":"Membrane.Time.os_time/0","doc":"Returns current POSIX time of operating system based on `System.os_time/0`\nin `Membrane.Time` units.\n\nThis time is not monotonic.","ref":"Membrane.Time.html#os_time/0"},{"type":"function","title":"Membrane.Time.pretty_duration/1","doc":"Returns duration as a string with unit. Chosen unit is the biggest possible\nthat doesn't involve precission loss.","ref":"Membrane.Time.html#pretty_duration/1"},{"type":"function","title":"Examples - Membrane.Time.pretty_duration/1","doc":"iex> import Membrane.Time\n    iex> 10 |> milliseconds() |> pretty_duration()\n    \"10 ms\"\n    iex> 60_000_000 |> microseconds() |> pretty_duration()\n    \"1 min\"\n    iex> 2 |> nanoseconds() |> pretty_duration()\n    \"2 ns\"","ref":"Membrane.Time.html#pretty_duration/1-examples"},{"type":"function","title":"Membrane.Time.pretty_now/0","doc":"Returns current time in pretty format (currently iso8601), as string\nUses `os_time/0` under the hood.","ref":"Membrane.Time.html#pretty_now/0"},{"type":"function","title":"Membrane.Time.second/0","doc":"Returns one second in `Membrane.Time` units.","ref":"Membrane.Time.html#second/0"},{"type":"function","title":"Membrane.Time.seconds/1","doc":"Returns given amount of seconds in `Membrane.Time` units.","ref":"Membrane.Time.html#seconds/1"},{"type":"function","title":"Membrane.Time.to_code/1","doc":"Returns quoted code producing given amount time. Chosen unit is the biggest possible\nthat doesn't involve precission loss.","ref":"Membrane.Time.html#to_code/1"},{"type":"function","title":"Examples - Membrane.Time.to_code/1","doc":"iex> import Membrane.Time\n    iex> 10 |> milliseconds() |> to_code() |> Macro.to_string()\n    quote do 10 |> Membrane.Time.milliseconds() end |> Macro.to_string()\n    iex> 60_000_000 |> microseconds() |> to_code() |> Macro.to_string()\n    quote do Membrane.Time.minute() end |> Macro.to_string()\n    iex> 2 |> nanoseconds() |> to_code() |> Macro.to_string()\n    quote do 2 |> Membrane.Time.nanoseconds() end |> Macro.to_string()","ref":"Membrane.Time.html#to_code/1-examples"},{"type":"function","title":"Membrane.Time.to_datetime/1","doc":"Returns time as a `DateTime` struct. TimeZone is set to UTC.","ref":"Membrane.Time.html#to_datetime/1"},{"type":"function","title":"Membrane.Time.to_iso8601/1","doc":"Returns time as a iso8601 string.","ref":"Membrane.Time.html#to_iso8601/1"},{"type":"function","title":"Membrane.Time.to_native_units/1","doc":"Returns time in VM native units. Rounded using Kernel.round/1.","ref":"Membrane.Time.html#to_native_units/1"},{"type":"function","title":"Membrane.Time.to_ntp_timestamp/1","doc":"Converts the timestamp into NTP timestamp. May introduce small rounding errors.","ref":"Membrane.Time.html#to_ntp_timestamp/1"},{"type":"function","title":"Membrane.Time.vm_time/0","doc":"Returns current Erlang VM system time based on `System.system_time/0`\nin `Membrane.Time` units.\n\nIt is the VM view of the `os_time/0`. They may not match in case of time warps.\nIt is not monotonic.","ref":"Membrane.Time.html#vm_time/0"},{"type":"type","title":"Membrane.Time.non_neg/0","doc":"","ref":"Membrane.Time.html#t:non_neg/0"},{"type":"type","title":"Membrane.Time.t/0","doc":"The time represented in Membrane's time units.","ref":"Membrane.Time.html#t:t/0"},{"type":"module","title":"Membrane.UtilitySupervisor","doc":"A supervisor responsible for managing utility processes under the pipeline's\nsupervision tree.\n\nThe supervisor is spawned with each component and can be accessed from callback contexts.\n\n`Membrane.UtilitySupervisor` does not restart processes. Rather, it ensures that these utility processes\nterminate gracefully when the component that initiated them terminates.\n\nIf a process needs to be able to restart, spawn a dedicated supervisor  under this supervisor.","ref":"Membrane.UtilitySupervisor.html"},{"type":"module","title":"Example - Membrane.UtilitySupervisor","doc":"def handle_setup(ctx, state) do\n      Membrane.UtilitySupervisor.start_link_child(\n        ctx.utility_supervisor,\n        {MySupervisor, children: [SomeWorker, OtherWorker], restart: :one_for_one})\n    end","ref":"Membrane.UtilitySupervisor.html#module-example"},{"type":"function","title":"Membrane.UtilitySupervisor.start_child/2","doc":"Starts a process under the utility supervisor.\n\nSemantics of the `child_spec` argument are the same as in `Supervisor.child_spec/2`.","ref":"Membrane.UtilitySupervisor.html#start_child/2"},{"type":"function","title":"Membrane.UtilitySupervisor.start_link_child/2","doc":"Starts a process under the utility supervisor and links it to the current process.\n\nSemantics of the `child_spec` argument are the same as in `Supervisor.child_spec/2`.","ref":"Membrane.UtilitySupervisor.html#start_link_child/2"},{"type":"type","title":"Membrane.UtilitySupervisor.t/0","doc":"The pid of the `Membrane.UtilitySupervisor` process.","ref":"Membrane.UtilitySupervisor.html#t:t/0"},{"type":"exception","title":"Membrane.ActionError","doc":"","ref":"Membrane.ActionError.html"},{"type":"exception","title":"Membrane.BinError","doc":"","ref":"Membrane.BinError.html"},{"type":"exception","title":"Membrane.CallbackError","doc":"","ref":"Membrane.CallbackError.html"},{"type":"exception","title":"Membrane.ElementError","doc":"","ref":"Membrane.ElementError.html"},{"type":"exception","title":"Membrane.LinkError","doc":"","ref":"Membrane.LinkError.html"},{"type":"exception","title":"Membrane.PadDirectionError","doc":"","ref":"Membrane.PadDirectionError.html"},{"type":"exception","title":"Membrane.PadError","doc":"","ref":"Membrane.PadError.html"},{"type":"exception","title":"Membrane.ParentError","doc":"","ref":"Membrane.ParentError.html"},{"type":"exception","title":"Membrane.PipelineError","doc":"","ref":"Membrane.PipelineError.html"},{"type":"exception","title":"Membrane.SetupError","doc":"","ref":"Membrane.SetupError.html"},{"type":"exception","title":"Membrane.StreamFormatError","doc":"","ref":"Membrane.StreamFormatError.html"},{"type":"exception","title":"Membrane.TimerError","doc":"","ref":"Membrane.TimerError.html"},{"type":"exception","title":"Membrane.UnknownChildError","doc":"","ref":"Membrane.UnknownChildError.html"},{"type":"exception","title":"Membrane.UnknownPadError","doc":"","ref":"Membrane.UnknownPadError.html"},{"type":"extras","title":"Membrane Framework","doc":"# Membrane Framework\n\n[![Hex.pm](https://img.shields.io/hexpm/v/membrane_core.svg)](https://hex.pm/packages/membrane_core)\n[![API Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_core/)\n[![CircleCI](https://circleci.com/gh/membraneframework/membrane_core.svg?style=svg)](https://circleci.com/gh/membraneframework/membrane_core)\n\nMembrane is a versatile multimedia streaming & processing framework. You can use it to build a media server of your need, that can:\n- stream via WebRTC, RTSP, RTMP, HLS, HTTP and other protocols,\n- transcode, mix and apply custom processing of video & audio,\n- accept and generate / record to MP4, MKV, FLV and other containers,\n- handle dynamically connecting and disconnecting streams,\n- seamlessly scale and recover from errors,\n- do whatever you imagine if you implement it yourself :D Membrane makes it easy to plug in your code at almost any point of processing.\n\nThe abbreviations above don't ring any bells? Visit [membrane.stream/learn](https://membrane.stream/learn) and let Membrane introduce you to the multimedia world!\n\nWant a generic media server, instead of building a custom one? Try [Jellyfish](https://github.com/jellyfish-dev/jellyfish) - it's built on top of Membrane and provides many of its features via simple, WebSocket API. We'll soon [provide it as a SAAS](https://membrane.stream/cloud) too.\n\nIf you have questions or need consulting, we're for you at our [Discord](https://discord.gg/nwnfVSY), [forum](https://elixirforum.com/c/elixir-framework-forums/membrane-forum/), [GitHub discussions](https://github.com/orgs/membraneframework/discussions), [X (Twitter)](https://twitter.com/ElixirMembrane) and via [e-mail](mailto:info@membraneframework.org).\n\nYou can also [follow Membrane on X (Twitter)](https://twitter.com/ElixirMembrane) or [join our Discord](https://discord.gg/nwnfVSY) to be up to date and get involved in the community.\n\nMembrane is maintained by [Software Mansion](swmansion.com).","ref":"readme.html"},{"type":"extras","title":"Quick start - Membrane Framework","doc":"```elixir\nMix.install([\n  :membrane_hackney_plugin,\n  :membrane_mp3_mad_plugin,\n  :membrane_portaudio_plugin,\n])\n\ndefmodule MyPipeline do\n  use Membrane.Pipeline\n\n  @impl true\n  def handle_init(_ctx, mp3_url) do\n    spec =\n      child(%Membrane.Hackney.Source{\n        location: mp3_url, hackney_opts: [follow_redirect: true]\n      })\n      |> child(Membrane.MP3.MAD.Decoder)\n      |> child(Membrane.PortAudio.Sink)\n\n    {[spec: spec], %{}}\n  end\nend\n\nmp3_url = \"https://raw.githubusercontent.com/membraneframework/membrane_demo/master/simple_pipeline/sample.mp3\"\n\nMembrane.Pipeline.start_link(MyPipeline, mp3_url)\n```\n\nThis is an [Elixir](elixir-lang.org) snippet, that streams an mp3 via HTTP and plays it on your speaker. Here's how to run it:\n- Option 1: Click the button below:\n\n  [![Run in Livebook](https://livebook.dev/badge/v1/blue.svg)](https://livebook.dev/run?url=https%3A%2F%2Fgithub.com%2Fmembraneframework%2Fmembrane_core%2Fblob%2Fmaster%2Fexample.livemd)\n\n  It will install [Livebook](livebook.dev), an interactive notebook similar to Jupyter, and it'll open the snippet in there for you. Then just click the 'run' button in there.\n\n- Option 2: If you don't want to use Livebook, you can [install Elixir](https://elixir-lang.org/install.html), type `iex` to run the interactive shell and paste the snippet there.\n\nAfter that, you should hear music playing on your speaker :tada:\n\nTo learn step-by-step what exactly happens here, follow [this tutorial](https://membrane.stream/learn/get_started_with_membrane).","ref":"readme.html#quick-start"},{"type":"extras","title":"Learning - Membrane Framework","doc":"The best place to learn Membrane is the [membrane.stream/learn](https://membrane.stream/learn) website and the [membrane_demo](https://github.com/membraneframework/membrane_demo) repository. Try them out, then hack something exciting!","ref":"readme.html#learning"},{"type":"extras","title":"Structure of the framework - Membrane Framework","doc":"The most basic media processing entities of Membrane are `Element`s. An element might be able, for example, to mux incoming audio and video streams into MP4, or play raw audio using your sound card. You can create elements yourself, or choose from the ones provided by the framework.\n\nElements can be organized into a pipeline - a sequence of linked elements that perform a specific task. For example, a pipeline might receive an incoming RTSP stream from a webcam and convert it to an HLS stream, or act as a selective forwarding unit (SFU) to implement your own videoconferencing room. The [Quick start](#quick-start) section above shows how to create a simple pipeline.\n\n#","ref":"readme.html#structure-of-the-framework"},{"type":"extras","title":"Membrane packages - Membrane Framework","doc":"To embrace modularity, Membrane is delivered to you in multiple packages, including plugins, formats, core and standalone libraries. The complete list of all the Membrane packages maintained by the Membrane team is available [here](#Membrane-packages).\n\n**Plugins**\n\nPlugins provide elements that you can use in your pipeline. Each plugin lives in a `membrane_X_plugin` repository, where X can be a protocol, codec, container or functionality, for example [mebrane_opus_plugin](https://github.com/membraneframework/membrane_opus_plugin). Plugins wrapping a tool or library are named `membrane_X_LIBRARYNAME_plugin` or just `membrane_LIBRARYNAME_plugin`, like [membrane_mp3_mad_plugin](https://github.com/membraneframework/membrane_mp3_mad_plugin). Plugins are published on [hex.pm](https://hex.pm), for example [hex.pm/packages/membrane_opus_plugin](https://hex.pm/pakcages/membrane_opus_plugin) and docs are at [hexdocs](https://hexdocs.pm), like [hexdocs.pm/membrane_opus_plugin](https://hexdocs.pm/membrane_opus_plugin). Some plugins require native libraries installed in your OS. Those requirements, along with usage examples are outlined in each plugin's readme.\n\n**Formats**\n\nApart from plugins, Membrane has stream formats, which live in `membrane_X_format` repositories, where X is usually a codec or container, for example [mebrane_opus_format](https://github.com/membraneframework/mebrane_opus_format). Stream formats are published the same way as packages and are used by elements to define what kind of stream can be sent or received. They also provide utility functions to deal with a given codec/container.\n\n**Core**\n\nThe API for creating pipelines (and custom elements too) is provided by [membrane_core](https://github.com/membraneframework/membrane_core). To install it, add the following line to your `deps` in `mix.exs` and run `mix deps.get`\n\n```elixir\n{:membrane_core, \"~> 1.1\"}\n```\n\n**Standalone libraries**\n\nLast but not least, Membrane provides tools and libraries that can be used standalone and don't depend on the membrane_core, for example [video_compositor](https://github.com/membraneframework/video_compositor), [ex_sdp](https://github.com/membraneframework/ex_sdp) or [unifex](https://github.com/membraneframework/unifex).","ref":"readme.html#membrane-packages"},{"type":"extras","title":"Goals - Membrane Framework","doc":"The main goals of Membrane are:\n- To make work with multimedia a more pleasant experience than it is now.\n- To provide a welcoming ecosystem for learning multimedia development.\n- To power resilient, maintainable and scalable systems.","ref":"readme.html#goals"},{"type":"extras","title":"Elixir language - Membrane Framework","doc":"We chose Elixir for Membrane because it's a modern, high-level, easy-to-learn language, that lets us rapidly develop media solutions. Elixir's main selling points are built-in parallelism and fault-tolerance features, so we can build scalable systems that are self-healing. It also runs on the battle-tested BEAM VM, that's been widely used and actively developed since the '80s. When we need the performance of a low-level language, we delegate to Rust or C.\n\nIf you don't know Elixir, try [this tutorial](https://elixir-lang.org/getting-started) - it shouldn't take long and you'll know more than enough to get started with Membrane.","ref":"readme.html#elixir-language"},{"type":"extras","title":"Contributing - Membrane Framework","doc":"We welcome everyone to contribute to Membrane. Here are some ways to contribute:\n- Spread the word about Membrane! Even though multimedia are present everywhere today, media dev is still quite niche. Let it be no longer!\n- Create learning materials. We try our best but can cover only a limited number of Membrane use cases.\n- Improve docs. We know it's not the most exciting part, but if you had a hard time understanding the docs, you're the best person to fix them ;)\n- Contribute code - plugins, features and bug fixes. It's best to contact us before, so we can provide our help & assistance, and agree on important matters. For details see the [contribution guide](CONTRIBUTING.md).","ref":"readme.html#contributing"},{"type":"extras","title":"Support and questions - Membrane Framework","doc":"If you have any questions regarding Membrane Framework or need consulting, feel free to contact us via [Discord](https://discord.gg/nwnfVSY), [forum](https://elixirforum.com/c/elixir-framework-forums/membrane-forum/), [GitHub discussions](https://github.com/orgs/membraneframework/discussions), [X (Twitter)](https://twitter.com/ElixirMembrane) or [e-mail](mailto:info@membraneframework.org).","ref":"readme.html#support-and-questions"},{"type":"extras","title":"All packages - Membrane Framework","doc":"#","ref":"readme.html#all-packages"},{"type":"extras","title":"General - Membrane Framework","doc":"| Package | Description | Links |\n| --- | --- | --- |\n| [membrane_sdk](https://github.com/membraneframework/membrane_sdk) | Full power of Membrane in a single package | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_sdk.svg)](https://hex.pm/api/packages/membrane_sdk) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_sdk/) |\n| [membrane_core](https://github.com/membraneframework/membrane_core) | The core of the Membrane Framework, advanced multimedia processing framework | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_core.svg)](https://hex.pm/api/packages/membrane_core) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_core/) |\n| [membrane_rtc_engine](https://github.com/jellyfish-dev/membrane_rtc_engine) | [Maintainer: [jellyfish-dev](https://github.com/jellyfish-dev)] Customizable Real-time Communication Engine/SFU library focused on WebRTC. | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_rtc_engine.svg)](https://hex.pm/api/packages/membrane_rtc_engine) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_rtc_engine/) |\n| [kino_membrane](https://github.com/membraneframework/kino_membrane) | Utilities for introspecting Membrane pipelines in Livebook | [![Hex.pm](https://img.shields.io/hexpm/v/kino_membrane.svg)](https://hex.pm/api/packages/kino_membrane) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/kino_membrane/) |\n| [docker_membrane](https://github.com/membraneframework-labs/docker_membrane) | [Labs] A docker image based on Ubuntu, with Erlang, Elixir and libraries necessary to test and run the Membrane Framework. |   |\n| [membrane_demo](https://github.com/membraneframework/membrane_demo) | Examples of using the Membrane Framework |   |\n| [membrane_tutorials](https://github.com/membraneframework/membrane_tutorials) | Repository which contains text and assets used in Membrane Framework tutorials. |   |\n\n#","ref":"readme.html#general"},{"type":"extras","title":"Plugins - Membrane Framework","doc":"##","ref":"readme.html#plugins"},{"type":"extras","title":"General purpose - Membrane Framework","doc":"| Package | Description | Links |\n| --- | --- | --- |\n| [membrane_file_plugin](https://github.com/membraneframework/membrane_file_plugin) | Membrane plugin for reading and writing to files | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_file_plugin.svg)](https://hex.pm/api/packages/membrane_file_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_file_plugin/) |\n| [membrane_hackney_plugin](https://github.com/membraneframework/membrane_hackney_plugin) | HTTP sink and source based on Hackney | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_hackney_plugin.svg)](https://hex.pm/api/packages/membrane_hackney_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_hackney_plugin/) |\n| [membrane_scissors_plugin](https://github.com/membraneframework/membrane_scissors_plugin) | Element for cutting off parts of the stream | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_scissors_plugin.svg)](https://hex.pm/api/packages/membrane_scissors_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_scissors_plugin/) |\n| [membrane_tee_plugin](https://github.com/membraneframework/membrane_tee_plugin) | Membrane plugin for splitting data from a single input to multiple outputs | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_tee_plugin.svg)](https://hex.pm/api/packages/membrane_tee_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_tee_plugin/) |\n| [membrane_funnel_plugin](https://github.com/membraneframework/membrane_funnel_plugin) | Membrane plugin for merging multiple input streams into a single output | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_funnel_plugin.svg)](https://hex.pm/api/packages/membrane_funnel_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_funnel_plugin/) |\n| [membrane_realtimer_plugin](https://github.com/membraneframework/membrane_realtimer_plugin) | Membrane element limiting playback speed to realtime, according to buffers' timestamps | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_realtimer_plugin.svg)](https://hex.pm/api/packages/membrane_realtimer_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_realtimer_plugin/) |\n| [membrane_stream_plugin](https://github.com/membraneframework/membrane_stream_plugin) | Plugin for recording the entire stream sent through Membrane pads into a binary format and replaying it | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_stream_plugin.svg)](https://hex.pm/api/packages/membrane_stream_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_stream_plugin/) |\n| [membrane_fake_plugin](https://github.com/membraneframework/membrane_fake_plugin) | Fake Membrane sinks that drop incoming data | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_fake_plugin.svg)](https://hex.pm/api/packages/membrane_fake_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_fake_plugin/) |\n| [membrane_pcap_plugin](https://github.com/membraneframework-labs/membrane_pcap_plugin) | [Labs] Membrane PCAP source, capable of reading captured packets in pcap format |   |\n| [membrane_live_framerate_converter_plugin](https://github.com/kim-company/membrane_live_framerate_converter_plugin) | [Maintainer: [kim-company](https://github.com/kim-company)] Membrane.Filter that drops or duplicates frames to match a target framerate. Designed for realtime applications |   |\n| [membrane_template_plugin](https://github.com/membraneframework/membrane_template_plugin) | Template for Membrane Elements |   |\n\n##","ref":"readme.html#general-purpose"},{"type":"extras","title":"Streaming protocols - Membrane Framework","doc":"| Package | Description | Links |\n| --- | --- | --- |\n| [membrane_webrtc_plugin](https://github.com/jellyfish-dev/membrane_webrtc_plugin) | [Maintainer: [jellyfish-dev](https://github.com/jellyfish-dev)] Membrane plugin for sending and receiving media with WebRTC | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_webrtc_plugin.svg)](https://hex.pm/api/packages/membrane_webrtc_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_webrtc_plugin/) |\n| [membrane_rtmp_plugin](https://github.com/membraneframework/membrane_rtmp_plugin) | RTMP server & client | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_rtmp_plugin.svg)](https://hex.pm/api/packages/membrane_rtmp_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_rtmp_plugin/) |\n| [membrane_http_adaptive_stream_plugin](https://github.com/membraneframework/membrane_http_adaptive_stream_plugin) | Plugin generating manifests for HLS (DASH support planned) | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_http_adaptive_stream_plugin.svg)](https://hex.pm/api/packages/membrane_http_adaptive_stream_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_http_adaptive_stream_plugin/) |\n| [membrane_ice_plugin](https://github.com/jellyfish-dev/membrane_ice_plugin) | [Maintainer: [jellyfish-dev](https://github.com/jellyfish-dev)] Membrane plugin for ICE protocol | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_ice_plugin.svg)](https://hex.pm/api/packages/membrane_ice_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_ice_plugin/) |\n| [membrane_udp_plugin](https://github.com/membraneframework/membrane_udp_plugin) | Membrane plugin for sending and receiving UDP streams | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_udp_plugin.svg)](https://hex.pm/api/packages/membrane_udp_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_udp_plugin/) |\n| [membrane_rtp_plugin](https://github.com/membraneframework/membrane_rtp_plugin) | Membrane bins and elements for sending and receiving RTP/SRTP and RTCP/SRTCP streams | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_rtp_plugin.svg)](https://hex.pm/api/packages/membrane_rtp_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_rtp_plugin/) |\n| [membrane_rtp_h264_plugin](https://github.com/membraneframework/membrane_rtp_h264_plugin) | Membrane RTP payloader and depayloader for H264 | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_rtp_h264_plugin.svg)](https://hex.pm/api/packages/membrane_rtp_h264_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_rtp_h264_plugin/) |\n| [membrane_rtp_vp8_plugin](https://github.com/membraneframework/membrane_rtp_vp8_plugin) | Membrane elements for payloading and depayloading VP8 into RTP | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_rtp_vp8_plugin.svg)](https://hex.pm/api/packages/membrane_rtp_vp8_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_rtp_vp8_plugin/) |\n| [membrane_rtp_vp9_plugin](https://github.com/membraneframework-labs/membrane_rtp_vp9_plugin) | [Labs] Membrane elements for payloading and depayloading VP9 into RTP |   |\n| [membrane_rtp_mpegaudio_plugin](https://github.com/membraneframework/membrane_rtp_mpegaudio_plugin) | Membrane RTP MPEG Audio depayloader | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_rtp_mpegaudio_plugin.svg)](https://hex.pm/api/packages/membrane_rtp_mpegaudio_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_rtp_mpegaudio_plugin/) |\n| [membrane_rtp_opus_plugin](https://github.com/membraneframework/membrane_rtp_opus_plugin) | Membrane RTP payloader and depayloader for OPUS audio | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_rtp_opus_plugin.svg)](https://hex.pm/api/packages/membrane_rtp_opus_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_rtp_opus_plugin/) |\n| [membrane_rtp_g711_plugin](https://github.com/jellyfish-dev/membrane_rtp_g711_plugin) | [Maintainer: [jellyfish-dev](https://github.com/jellyfish-dev)] Membrane RTP payloader and depayloader for G711 audio | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_rtp_g711_plugin.svg)](https://hex.pm/api/packages/membrane_rtp_g711_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_rtp_g711_plugin/) |\n| [membrane_quic_plugin](https://github.com/mickel8/membrane_quic_plugin) | [Maintainer: [mickel8](https://github.com/mickel8)]  |   |\n| [membrane_mpeg_ts_plugin](https://github.com/kim-company/membrane_mpeg_ts_plugin) | [Maintainer: [kim-company](https://github.com/kim-company)] Membrane.Filter that demuxes MPEG-TS streams |   |\n| [membrane_hls_plugin](https://github.com/kim-company/membrane_hls_plugin) | [Maintainer: [kim-company](https://github.com/kim-company)] Plugin providing a `Membrane.HLS.Source` element for HTTP Live Streaming (HLS). |   |\n\n##","ref":"readme.html#streaming-protocols"},{"type":"extras","title":"Containers - Membrane Framework","doc":"| Package | Description | Links |\n| --- | --- | --- |\n| [membrane_mp4_plugin](https://github.com/membraneframework/membrane_mp4_plugin) | Utilities for MP4 container parsing and serialization and elements for muxing the stream to CMAF | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_mp4_plugin.svg)](https://hex.pm/api/packages/membrane_mp4_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_mp4_plugin/) |\n| [membrane_matroska_plugin](https://github.com/membraneframework/membrane_matroska_plugin) | Matroska muxer and demuxer | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_matroska_plugin.svg)](https://hex.pm/api/packages/membrane_matroska_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_matroska_plugin/) |\n| [membrane_flv_plugin](https://github.com/membraneframework/membrane_flv_plugin) | Muxer and demuxer elements for FLV format | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_flv_plugin.svg)](https://hex.pm/api/packages/membrane_flv_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_flv_plugin/) |\n| [membrane_ivf_plugin](https://github.com/membraneframework/membrane_ivf_plugin) | Plugin for converting video stream into IVF format | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_ivf_plugin.svg)](https://hex.pm/api/packages/membrane_ivf_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_ivf_plugin/) |\n| [membrane_ogg_plugin](https://github.com/membraneframework/membrane_ogg_plugin) | Plugin for depayloading an Ogg file into an Opus stream | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_ogg_plugin.svg)](https://hex.pm/api/packages/membrane_ogg_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_ogg_plugin/) |\n\n##","ref":"readme.html#containers"},{"type":"extras","title":"Audio codecs - Membrane Framework","doc":"| Package | Description | Links |\n| --- | --- | --- |\n| [membrane_aac_plugin](https://github.com/membraneframework/membrane_aac_plugin) | AAC parser and complementary elements for AAC codec | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_aac_plugin.svg)](https://hex.pm/api/packages/membrane_aac_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_aac_plugin/) |\n| [membrane_aac_fdk_plugin](https://github.com/membraneframework/membrane_aac_fdk_plugin) | Membrane AAC decoder and encoder based on FDK library | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_aac_fdk_plugin.svg)](https://hex.pm/api/packages/membrane_aac_fdk_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_aac_fdk_plugin/) |\n| [membrane_flac_plugin](https://github.com/membraneframework/membrane_flac_plugin) | Parser for files in FLAC bitstream format | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_flac_plugin.svg)](https://hex.pm/api/packages/membrane_flac_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_flac_plugin/) |\n| [membrane_mp3_lame_plugin](https://github.com/membraneframework/membrane_mp3_lame_plugin) | Membrane MP3 encoder based on Lame | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_mp3_lame_plugin.svg)](https://hex.pm/api/packages/membrane_mp3_lame_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_mp3_lame_plugin/) |\n| [membrane_mp3_mad_plugin](https://github.com/membraneframework/membrane_mp3_mad_plugin) | Membrane MP3 decoder based on MAD. | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_mp3_mad_plugin.svg)](https://hex.pm/api/packages/membrane_mp3_mad_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_mp3_mad_plugin/) |\n| [membrane_opus_plugin](https://github.com/membraneframework/membrane_opus_plugin) | Membrane Opus encoder and decoder | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_opus_plugin.svg)](https://hex.pm/api/packages/membrane_opus_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_opus_plugin/) |\n| [membrane_wav_plugin](https://github.com/membraneframework/membrane_wav_plugin) | Plugin providing elements handling audio in WAV file format. | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_wav_plugin.svg)](https://hex.pm/api/packages/membrane_wav_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_wav_plugin/) |\n| [membrane_g711_plugin](https://github.com/jellyfish-dev/membrane_g711_plugin) | [Maintainer: [jellyfish-dev](https://github.com/jellyfish-dev)] Membrane G.711 decoder, encoder and parser | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_g711_plugin.svg)](https://hex.pm/api/packages/membrane_g711_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_g711_plugin/) |\n\n##","ref":"readme.html#audio-codecs"},{"type":"extras","title":"Video codecs - Membrane Framework","doc":"| Package | Description | Links |\n| --- | --- | --- |\n| [membrane_h26x_plugin](https://github.com/membraneframework/membrane_h26x_plugin) | Membrane h264 and h265 parsers | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_h26x_plugin.svg)](https://hex.pm/api/packages/membrane_h26x_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_h26x_plugin/) |\n| [membrane_h264_ffmpeg_plugin](https://github.com/membraneframework/membrane_h264_ffmpeg_plugin) | Membrane H264 decoder and encoder based on FFmpeg and x264 | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_h264_ffmpeg_plugin.svg)](https://hex.pm/api/packages/membrane_h264_ffmpeg_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_h264_ffmpeg_plugin/) |\n| [membrane_abr_transcoder_plugin](https://github.com/membraneframework/membrane_abr_transcoder_plugin) |  | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_abr_transcoder_plugin.svg)](https://hex.pm/api/packages/membrane_abr_transcoder_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_abr_transcoder_plugin/) |\n| [membrane_h265_ffmpeg_plugin](https://github.com/gBillal/membrane_h265_ffmpeg_plugin) | [Maintainer: [gBillal](https://github.com/gBillal)] Membrane H265 decoder and encoder based on FFmpeg and x265 | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_h265_ffmpeg_plugin.svg)](https://hex.pm/api/packages/membrane_h265_ffmpeg_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_h265_ffmpeg_plugin/) |\n| [elixir-turbojpeg](https://github.com/BinaryNoggin/elixir-turbojpeg) | [Maintainer: [BinaryNoggin](https://github.com/BinaryNoggin)] libjpeg-turbo bindings for Elixir |   |\n| [membrane_subtitle_mixer_plugin](https://github.com/kim-company/membrane_subtitle_mixer_plugin) | [Maintainer: [kim-company](https://github.com/kim-company)] Membrane.Filter that uses CEA708 to merge subtitles directly in H264 packets. |   |\n\n##","ref":"readme.html#video-codecs"},{"type":"extras","title":"Raw audio & video - Membrane Framework","doc":"| Package | Description | Links |\n| --- | --- | --- |\n| [membrane_generator_plugin](https://github.com/membraneframework/membrane_generator_plugin) | Video and audio samples generator | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_generator_plugin.svg)](https://hex.pm/api/packages/membrane_generator_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_generator_plugin/) |\n\n**Raw audio**\n| Package | Description | Links |\n| --- | --- | --- |\n| [membrane_raw_audio_parser_plugin](https://github.com/membraneframework/membrane_raw_audio_parser_plugin) | Membrane element for parsing raw audio | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_raw_audio_parser_plugin.svg)](https://hex.pm/api/packages/membrane_raw_audio_parser_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_raw_audio_parser_plugin/) |\n| [membrane_portaudio_plugin](https://github.com/membraneframework/membrane_portaudio_plugin) | Raw audio retriever and player based on PortAudio | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_portaudio_plugin.svg)](https://hex.pm/api/packages/membrane_portaudio_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_portaudio_plugin/) |\n| [membrane_audio_mix_plugin](https://github.com/membraneframework/membrane_audio_mix_plugin) | Plugin providing an element mixing raw audio frames. | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_audio_mix_plugin.svg)](https://hex.pm/api/packages/membrane_audio_mix_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_audio_mix_plugin/) |\n| [membrane_audio_filler_plugin](https://github.com/membraneframework/membrane_audio_filler_plugin) | Element for filling missing buffers in audio stream | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_audio_filler_plugin.svg)](https://hex.pm/api/packages/membrane_audio_filler_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_audio_filler_plugin/) |\n| [membrane_ffmpeg_swresample_plugin](https://github.com/membraneframework/membrane_ffmpeg_swresample_plugin) | Plugin performing audio conversion, resampling and channel mixing, using SWResample module of FFmpeg library | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_ffmpeg_swresample_plugin.svg)](https://hex.pm/api/packages/membrane_ffmpeg_swresample_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_ffmpeg_swresample_plugin/) |\n| [membrane_audiometer_plugin](https://github.com/membraneframework/membrane_audiometer_plugin) | Elements for measuring the level of the audio stream | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_audiometer_plugin.svg)](https://hex.pm/api/packages/membrane_audiometer_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_audiometer_plugin/) |\n\n**Raw video**\n| Package | Description | Links |\n| --- | --- | --- |\n| [membrane_raw_video_parser_plugin](https://github.com/membraneframework/membrane_raw_video_parser_plugin) | Membrane plugin for parsing raw video streams | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_raw_video_parser_plugin.svg)](https://hex.pm/api/packages/membrane_raw_video_parser_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_raw_video_parser_plugin/) |\n| [membrane_video_merger_plugin](https://github.com/membraneframework/membrane_video_merger_plugin) | Membrane raw video cutter, merger and cut & merge bin | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_video_merger_plugin.svg)](https://hex.pm/api/packages/membrane_video_merger_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_video_merger_plugin/) |\n| [membrane_video_compositor_plugin](https://github.com/membraneframework/membrane_video_compositor_plugin) | Membrane plugin for video and audio mixing/composing | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_video_compositor_plugin.svg)](https://hex.pm/api/packages/membrane_video_compositor_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_video_compositor_plugin/) |\n| [membrane_camera_capture_plugin](https://github.com/membraneframework/membrane_camera_capture_plugin) | A set of elements allowing for capturing local media such as camera or microphone | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_camera_capture_plugin.svg)](https://hex.pm/api/packages/membrane_camera_capture_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_camera_capture_plugin/) |\n| [membrane_rpicam_plugin](https://github.com/membraneframework/membrane_rpicam_plugin) | Membrane rpicam plugin | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_rpicam_plugin.svg)](https://hex.pm/api/packages/membrane_rpicam_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_rpicam_plugin/) |\n| [membrane_framerate_converter_plugin](https://github.com/membraneframework/membrane_framerate_converter_plugin) | Element for converting frame rate of raw video stream | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_framerate_converter_plugin.svg)](https://hex.pm/api/packages/membrane_framerate_converter_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_framerate_converter_plugin/) |\n| [membrane_sdl_plugin](https://github.com/membraneframework/membrane_sdl_plugin) | Membrane video player based on SDL | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_sdl_plugin.svg)](https://hex.pm/api/packages/membrane_sdl_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_sdl_plugin/) |\n| [membrane_overlay_plugin](https://github.com/membraneframework/membrane_overlay_plugin) | Filter for applying overlay image or text on top of video | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_overlay_plugin.svg)](https://hex.pm/api/packages/membrane_overlay_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_overlay_plugin/) |\n| [membrane_ffmpeg_swscale_plugin](https://github.com/membraneframework/membrane_ffmpeg_swscale_plugin) | Plugin providing an element scaling raw video frames, using SWScale module of FFmpeg library. | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_ffmpeg_swscale_plugin.svg)](https://hex.pm/api/packages/membrane_ffmpeg_swscale_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_ffmpeg_swscale_plugin/) |\n| [membrane_ffmpeg_video_filter_plugin](https://github.com/membraneframework/membrane_ffmpeg_video_filter_plugin) | FFmpeg-based video filters | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_ffmpeg_video_filter_plugin.svg)](https://hex.pm/api/packages/membrane_ffmpeg_video_filter_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_ffmpeg_video_filter_plugin/) |\n| [membrane_video_mixer_plugin](https://github.com/kim-company/membrane_video_mixer_plugin) | [Maintainer: [kim-company](https://github.com/kim-company)] Membrane.Filter that mixes a variable number of input videos into one output using ffmpeg filters | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_video_mixer_plugin.svg)](https://hex.pm/api/packages/membrane_video_mixer_plugin)  |\n\n##","ref":"readme.html#raw-audio-video"},{"type":"extras","title":"External APIs - Membrane Framework","doc":"| Package | Description | Links |\n| --- | --- | --- |\n| [membrane_aws_plugin](https://github.com/jellyfish-dev/membrane_aws_plugin) | [Maintainer: [jellyfish-dev](https://github.com/jellyfish-dev)]  |   |\n| [membrane_agora_plugin](https://github.com/membraneframework/membrane_agora_plugin) | Membrane Sink for Agora Server Gateway | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_agora_plugin.svg)](https://hex.pm/api/packages/membrane_agora_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_agora_plugin/) |\n| [membrane_element_gcloud_speech_to_text](https://github.com/membraneframework/membrane_element_gcloud_speech_to_text) | Membrane plugin providing speech recognition via Google Cloud Speech-to-Text API | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_element_gcloud_speech_to_text.svg)](https://hex.pm/api/packages/membrane_element_gcloud_speech_to_text) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_element_gcloud_speech_to_text/) |\n| [membrane_element_ibm_speech_to_text](https://github.com/membraneframework/membrane_element_ibm_speech_to_text) | Membrane plugin providing speech recognition via IBM Cloud Speech-to-Text service | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_element_ibm_speech_to_text.svg)](https://hex.pm/api/packages/membrane_element_ibm_speech_to_text) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_element_ibm_speech_to_text/) |\n| [membrane_s3_plugin](https://github.com/YuzuTen/membrane_s3_plugin) | [Maintainer: [YuzuTen](https://github.com/YuzuTen)] Membrane framework plugin to support S3 sources/destinations | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_s3_plugin.svg)](https://hex.pm/api/packages/membrane_s3_plugin) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_s3_plugin/) |\n| [membrane_transcription](https://github.com/lawik/membrane_transcription) | [Maintainer: [lawik](https://github.com/lawik)] Prototype transcription for Membrane |   |\n\n#","ref":"readme.html#external-apis"},{"type":"extras","title":"Formats - Membrane Framework","doc":"| Package | Description | Links |\n| --- | --- | --- |\n| [membrane_rtp_format](https://github.com/membraneframework/membrane_rtp_format) | Real-time Transport Protocol format for Membrane Framework | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_rtp_format.svg)](https://hex.pm/api/packages/membrane_rtp_format) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_rtp_format/) |\n| [membrane_cmaf_format](https://github.com/membraneframework/membrane_cmaf_format) | Membrane description for Common Media Application Format | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_cmaf_format.svg)](https://hex.pm/api/packages/membrane_cmaf_format) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_cmaf_format/) |\n| [membrane_matroska_format](https://github.com/membraneframework/membrane_matroska_format) | Matroska Membrane format | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_matroska_format.svg)](https://hex.pm/api/packages/membrane_matroska_format) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_matroska_format/) |\n| [membrane_mp4_format](https://github.com/membraneframework/membrane_mp4_format) | MPEG-4 container Membrane format | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_mp4_format.svg)](https://hex.pm/api/packages/membrane_mp4_format) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_mp4_format/) |\n| [membrane_raw_audio_format](https://github.com/membraneframework/membrane_raw_audio_format) | Raw audio format definition for the Membrane Multimedia Framework | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_raw_audio_format.svg)](https://hex.pm/api/packages/membrane_raw_audio_format) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_raw_audio_format/) |\n| [membrane_raw_video_format](https://github.com/membraneframework/membrane_raw_video_format) | Membrane Multimedia Framework: Raw video format definition | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_raw_video_format.svg)](https://hex.pm/api/packages/membrane_raw_video_format) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_raw_video_format/) |\n| [membrane_aac_format](https://github.com/membraneframework/membrane_aac_format) | Advanced Audio Codec Membrane format | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_aac_format.svg)](https://hex.pm/api/packages/membrane_aac_format) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_aac_format/) |\n| [membrane_opus_format](https://github.com/membraneframework/membrane_opus_format) | Opus audio format definition for Membrane Framework | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_opus_format.svg)](https://hex.pm/api/packages/membrane_opus_format) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_opus_format/) |\n| [membrane_flac_format](https://github.com/membraneframework/membrane_flac_format) | FLAC audio format description for Membrane Framework | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_flac_format.svg)](https://hex.pm/api/packages/membrane_flac_format) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_flac_format/) |\n| [membrane_mpegaudio_format](https://github.com/membraneframework/membrane_mpegaudio_format) | MPEG audio format definition for Membrane Framework | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_mpegaudio_format.svg)](https://hex.pm/api/packages/membrane_mpegaudio_format) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_mpegaudio_format/) |\n| [membrane_h264_format](https://github.com/membraneframework/membrane_h264_format) | Membrane Multimedia Framework: H264 video format definition | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_h264_format.svg)](https://hex.pm/api/packages/membrane_h264_format) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_h264_format/) |\n| [membrane_vp8_format](https://github.com/membraneframework/membrane_vp8_format) | VP8 Membrane format | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_vp8_format.svg)](https://hex.pm/api/packages/membrane_vp8_format) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_vp8_format/) |\n| [membrane_vp9_format](https://github.com/membraneframework/membrane_vp9_format) | VP9 Membrane format | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_vp9_format.svg)](https://hex.pm/api/packages/membrane_vp9_format) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_vp9_format/) |\n| [membrane_g711_format](https://github.com/jellyfish-dev/membrane_g711_format) | [Maintainer: [jellyfish-dev](https://github.com/jellyfish-dev)] Membrane Multimedia Framework: G711 audio format definition | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_g711_format.svg)](https://hex.pm/api/packages/membrane_g711_format) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_g711_format/) |\n\n#","ref":"readme.html#formats"},{"type":"extras","title":"Standalone media libs - Membrane Framework","doc":"| Package | Description | Links |\n| --- | --- | --- |\n| [video_compositor](https://github.com/membraneframework/video_compositor) | Media server for low-latency live video and audio mixing/composing |   |\n| [ex_sdp](https://github.com/membraneframework/ex_sdp) | Parser and serializer for Session Description Protocol | [![Hex.pm](https://img.shields.io/hexpm/v/ex_sdp.svg)](https://hex.pm/api/packages/ex_sdp) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/ex_sdp/) |\n| [ex_libnice](https://github.com/membraneframework/ex_libnice) | Libnice-based Interactive Connectivity Establishment (ICE) protocol support for Elixir | [![Hex.pm](https://img.shields.io/hexpm/v/ex_libnice.svg)](https://hex.pm/api/packages/ex_libnice) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/ex_libnice/) |\n| [ex_libsrtp](https://github.com/membraneframework/ex_libsrtp) | Elixir bindings for libsrtp | [![Hex.pm](https://img.shields.io/hexpm/v/ex_libsrtp.svg)](https://hex.pm/api/packages/ex_libsrtp) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/ex_libsrtp/) |\n| [ex_m3u8](https://github.com/membraneframework/ex_m3u8) | Elixir package for serializing and deserializing M3U8 manifests. | [![Hex.pm](https://img.shields.io/hexpm/v/ex_m3u8.svg)](https://hex.pm/api/packages/ex_m3u8) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/ex_m3u8/) |\n| [membrane_rtsp](https://github.com/membraneframework/membrane_rtsp) | RTSP client for Elixir | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_rtsp.svg)](https://hex.pm/api/packages/membrane_rtsp) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_rtsp/) |\n| [membrane_ffmpeg_generator](https://github.com/membraneframework-labs/membrane_ffmpeg_generator) | [Labs] FFmpeg video and audio generator for tests, benchmarks and demos. | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_ffmpeg_generator.svg)](https://hex.pm/api/packages/membrane_ffmpeg_generator) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_ffmpeg_generator/) |\n\n#","ref":"readme.html#standalone-media-libs"},{"type":"extras","title":"Utils - Membrane Framework","doc":"| Package | Description | Links |\n| --- | --- | --- |\n| [unifex](https://github.com/membraneframework/unifex) | Tool for generating interfaces between native C code and Elixir | [![Hex.pm](https://img.shields.io/hexpm/v/unifex.svg)](https://hex.pm/api/packages/unifex) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/unifex/) |\n| [bundlex](https://github.com/membraneframework/bundlex) | Multiplatform app bundler tool for Elixir | [![Hex.pm](https://img.shields.io/hexpm/v/bundlex.svg)](https://hex.pm/api/packages/bundlex) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/bundlex/) |\n| [beamchmark](https://github.com/membraneframework/beamchmark) | Elixir tool for benchmarking EVM performance | [![Hex.pm](https://img.shields.io/hexpm/v/beamchmark.svg)](https://hex.pm/api/packages/beamchmark) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/beamchmark/) |\n| [bunch](https://github.com/membraneframework/bunch) | A bunch of helper functions, intended to make life easier | [![Hex.pm](https://img.shields.io/hexpm/v/bunch.svg)](https://hex.pm/api/packages/bunch) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/bunch/) |\n| [bunch_native](https://github.com/membraneframework/bunch_native) | Native part of the Bunch package | [![Hex.pm](https://img.shields.io/hexpm/v/bunch_native.svg)](https://hex.pm/api/packages/bunch_native) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/bunch_native/) |\n| [shmex](https://github.com/membraneframework/shmex) | Elixir bindings for shared memory | [![Hex.pm](https://img.shields.io/hexpm/v/shmex.svg)](https://hex.pm/api/packages/shmex) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/shmex/) |\n| [membrane_timestamp_queue](https://github.com/membraneframework/membrane_timestamp_queue) |  |   |\n| [membrane_common_c](https://github.com/membraneframework/membrane_common_c) | Membrane Multimedia Framework: Common C Routines | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_common_c.svg)](https://hex.pm/api/packages/membrane_common_c) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_common_c/) |\n| [membrane_telemetry_metrics](https://github.com/membraneframework/membrane_telemetry_metrics) | Membrane tool for generating metrics | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_telemetry_metrics.svg)](https://hex.pm/api/packages/membrane_telemetry_metrics) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_telemetry_metrics/) |\n| [membrane_opentelemetry](https://github.com/membraneframework-labs/membrane_opentelemetry) | [Labs] Utilities for using OpenTelemetry with Membrane | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_opentelemetry.svg)](https://hex.pm/api/packages/membrane_opentelemetry) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_opentelemetry/) |\n| [membrane_precompiled_dependency_provider](https://github.com/membraneframework/membrane_precompiled_dependency_provider) |  | [![Hex.pm](https://img.shields.io/hexpm/v/membrane_precompiled_dependency_provider.svg)](https://hex.pm/api/packages/membrane_precompiled_dependency_provider) [![Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_precompiled_dependency_provider/) |","ref":"readme.html#utils"},{"type":"extras","title":"Copyright and License - Membrane Framework","doc":"Copyright 2018, [Software Mansion](https://swmansion.com/?utm_source=git&utm_medium=readme&utm_campaign=membrane)\n\n[![Software Mansion](https://logo.swmansion.com/logo?color=white&variant=desktop&width=200&tag=membrane-github)](https://swmansion.com/?utm_source=git&utm_medium=readme&utm_campaign=membrane)\n\nLicensed under the [Apache License, Version 2.0](LICENSE)","ref":"readme.html#copyright-and-license"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"1.1.0 - Changelog","doc":"* Add new callbacks `handle_child_setup_completed/3` and `handle_child_playing/3` in Bins and Pipelines. [#801](https://github.com/membraneframework/membrane_core/pull/801)\n * Deprecate `handle_spec_started/3` callback in Bins and Pipelines. [#708](https://github.com/membraneframework/membrane_core/pull/708)\n * Handle buffers from input pads having `flow_control: :auto` only if demand on all output pads having `flow_control: :auto` is positive. [#693](https://github.com/membraneframework/membrane_core/pull/693)\n * Set `:ratio` dependency version to `\"~> 3.0 or ~> 4.0\"`. [#780](https://github.com/membraneframework/membrane_core/pull/780)\n * Deprecate `Membrane.Testing.Pipeline.message_child/3`. Introduce `Membrane.Testing.Pipeline.notify_child/3` instead. [#779](https://github.com/membraneframework/membrane_core/pull/779)","ref":"changelog.html#1-1-0"},{"type":"extras","title":"1.0.1 - Changelog","doc":"* Specify the order in which state fields will be printed in the error logs. [#614](https://github.com/membraneframework/membrane_core/pull/614)\n * Fix clock selection [#626](https://github.com/membraneframework/membrane_core/pull/626)\n * Log messages in the default handle_info implementation [#680](https://github.com/membraneframework/membrane_core/pull/680)\n * Fix typespecs in Membrane.UtilitySupervisor [#681](https://github.com/membraneframework/membrane_core/pull/681)\n * Improve callback return types and group actions types [#702](https://github.com/membraneframework/membrane_core/pull/702) \n * Add `crash_reason` to `handle_crash_group_down/3` callback context in bins and pipelines. [#720](https://github.com/membraneframework/membrane_core/pull/720)","ref":"changelog.html#1-0-1"},{"type":"extras","title":"1.0.0 - Changelog","doc":"* Introduce `:remove_link` action in pipelines and bins.\n * Add children groups - a mechanism that allows refering to multiple children with a single identifier. \n * Rename `remove_child` action into `remove_children` and allow for removing a children group with a single action.\n * Add an ability to spawn anonymous children.\n * Replace `Membrane.Time.round_to_ ` with `Membrane.Time.as_ /2` with second argument equal `:round`. Rename `Membrane.Time.round_to_timebase` to `Membrane.Time.divide_by_timebase/2`. [#494](https://github.com/membraneframework/membrane_core/pull/494)\n * Remove `:playback` action. Introduce `:setup` action. [#496](https://github.com/membraneframework/membrane_core/pull/496)\n * Add `Membrane.Testing.Pipeline.get_child_pid/2`. [#497](https://github.com/membraneframework/membrane_core/pull/497)\n * Make callback contexts to be maps. [#504](https://github.com/membraneframework/membrane_core/pull/504)\n * All Membrane Elements can be compatible till now on - pads working in `:pull` mode, handling different `demand_units`, can be now linked.\n * Output pads working in `:pull` mode should have their `demand_unit` specified. If case it's not available, it's assumed that the pad handles demands in both `:bytes` and `:buffers` units.\n * Rename callbacks `handle_process/4` and `handle_write/4` to `handle_buffer/4` in [#506](https://github.com/membraneframework/membrane_core/pull/506)\n * The flow control of the pad is now set with a single `:flow_control` option instead of `:mode` and `:demand_mode` options.\n * Remove _t suffix from types [#509](https://github.com/membraneframework/membrane_core/pull/509)\n * Implement automatic demands in Membrane Sinks and Endpoints. [#512](https://github.com/membraneframework/membrane_core/pull/512)\n * Add `handle_child_pad_removed/4` callback in Bins and Pipelines. [#513](https://github.com/membraneframework/membrane_core/pull/513)\n * Introduce support for crash groups in Bins. [#521](https://github.com/membraneframework/membrane_core/pull/521)\n * Remove `assert_pipeline_play/2` from `Membrane.Testing.Assertions`. [#528](https://github.com/membraneframework/membrane_core/pull/528)\n * Make sure enumerable with all elements being `Membrane.Buffer.t()`, passed as `:output` parameter for `Membrane.Testing.Source` won't get rewrapped in `Membrane.Buffer.t()` struct.\n * Implement `Membrane.Debug.Filter` and `Membrane.Debug.Sink`. [#552](https://github.com/membraneframework/membrane_core/pull/552)\n * Add `:pause_auto_demand` and `:resume_auto_demand` actions. [#586](https://github.com/membraneframework/membrane_core/pull/586)\n * Send `:end_of_stream`, even if it is not preceded by `:start_of_stream`. [#557](https://github.com/membraneframework/membrane_core/pull/577)\n * Fix process leak in starting clocks. [#594](https://github.com/membraneframework/membrane_core/pull/594)\n * Add child exit reason to the supervisor exit reason. [#595](https://github.com/membraneframework/membrane_core/pull/595)\n * Remove default implementation of `start_/2`, `start_link/2` and `terminate/2` in modules using `Membrane.Pipeline`. [#598](https://github.com/membraneframework/membrane_core/pull/598) \n * Remove callback _Membrane.Element.WithInputPads.handle_buffers_batch/4_. [#601](https://github.com/membraneframework/membrane_core/pull/601)\n * Sort component state fields in the error logs in the order from the most to the least important. [#614](https://github.com/membraneframework/membrane_core/pull/614)","ref":"changelog.html#1-0-0"},{"type":"extras","title":"0.11.0 - Changelog","doc":"* Separate element_name and pad arguments in handle_element_{start, end}_of_stream signature [#219](https://github.com/membraneframework/membrane_core/issues/219)\n * Refine communication between parent and its children [#270](https://github.com/membraneframework/membrane_core/issues/270)\n * Add `handle_call/3` callback in the pipeline, as well as a `:reply` and `:reply_to` actions. Rename `handle_other/3` callback into `handle_info/3` [#334](https://github.com/membraneframework/membrane_core/issues/334)\n * Add `Membrane.FilterAggregator` that allows to run multiple filters sequentially within one process. [#355](https://github.com/membraneframework/membrane_core/pull/355)\n * Log info about element's playback state change as debug, not as debug_verbose. [#430](https://github.com/membraneframework/membrane_core/pull/430)\n * Rename `Membrane.Time.to_ /1` into `Membrane.Time.round_to_ /1` to indicate that the result will be rounded. Make `Membrane.Time. /1` accept `%Ratio{}` as an argument. Add `Membrane.Time.round_to_timebase` function.\n * New `spec` action syntax - the structure of pipeline is now defined with the use of `Membrane.ChildrenSpec`\n * Rename `:caps` to `:stream_format`.\n * Use Elixir patterns as `:accepted_format` in pad definition.\n * Delete `:ok` from tuples returned from callbacks.\n * Remove `:type` from specs passed to `def_options/1` macro in bins and elements.\n * Add `Membrane.Testing.MockResourceGuard`.","ref":"changelog.html#0-11-0"},{"type":"extras","title":"0.10.0 - Changelog","doc":"* Remove all deprecated stuff [#399](https://github.com/membraneframework/membrane_core/pull/399)\n * Make `Membrane.Pipeline.{prepare, play, stop}` deprecated and add `:playback` action instead\n * Make `Membrane.Pipeline.stop_and_terminate` deprecated and add `Membrane.Pipeline.terminate/2` instead\n * Add `Membrane.RemoteControlled.Pipeline` - a basic implementation of a `Membrane.Pipeline` that can be spawned and controlled by an external process [#366](https://github.com/membraneframework/membrane_core/pull/366)\n * Disallow sending buffers without sending caps first [#341](https://github.com/membraneframework/membrane_core/issues/341)\n * Refine the `Membrane.Testing.Pipeline` API - deprecate the `Membrane.Testing.Pipeline.Options` usage, use keyword list as options in `Membrane.Testing.Pipeline.start/1` and `Membrane.Testing.Pipeline.start_link/1`","ref":"changelog.html#0-10-0"},{"type":"extras","title":"0.9.0 - Changelog","doc":"* Automatic demands [#313](https://github.com/membraneframework/membrane_core/pull/313)\n * Stop forwarding notifications by default in bins [#358](https://github.com/membraneframework/membrane_core/pull/358)\n * More fine-grained control over emitted metrics [#365](https://github.com/membraneframework/membrane_core/pull/365)\n\n #","ref":"changelog.html#0-9-0"},{"type":"extras","title":"PRs not influencing public API: - Changelog","doc":"* Added log metadata when reporting init in telemetry [#376](https://github.com/membraneframework/membrane_core/pull/376)\n * Fix generation of pad documentation inside an element [#377](https://github.com/membraneframework/membrane_core/pull/377)\n * Leaving static pads unlinked and transiting to a playback state other than `:stopped` will result\n in runtime error (previously only a warning was printed out). [#389](https://github.com/membraneframework/membrane_core/pull/389)\n * It is possible now to assert on crash group down when using Testing.Pipeline. [#391](https://github.com/membraneframework/membrane_core/pull/391)","ref":"changelog.html#prs-not-influencing-public-api"},{"type":"extras","title":"0.8.2 - Changelog","doc":"* Fixed PadAdded spec [#359](https://github.com/membraneframework/membrane_core/pull/359)\n#","ref":"changelog.html#0-8-2"},{"type":"extras","title":"PRs not influencing public API: - Changelog","doc":"* Prevent internal testing notifications from reaching pipeline module [#350](https://github.com/membraneframework/membrane_core/pull/350)\n * Fix unknown node error on distribution changes [#352](https://github.com/membraneframework/membrane_core/pull/352)\n * Add new type of element, `Membrane.Endpoint` [#382](https://github.com/membraneframework/membrane_core/pull/382)","ref":"changelog.html#prs-not-influencing-public-api"},{"type":"extras","title":"0.8.1 - Changelog","doc":"* allow telemetry in version 1.0 only [#347](https://github.com/membraneframework/membrane_core/pull/347)\n#","ref":"changelog.html#0-8-1"},{"type":"extras","title":"PRs not influencing public API: - Changelog","doc":"* fix action enforcing CHANGELOG update with PR [#345](https://github.com/membraneframework/membrane_core/pull/345)\n * Release membrane_core v0.8.1 [#348](https://github.com/membraneframework/membrane_core/pull/348)","ref":"changelog.html#prs-not-influencing-public-api"},{"type":"extras","title":"0.8.0 - Changelog","doc":"#","ref":"changelog.html#0-8-0"},{"type":"extras","title":"Release notes: - Changelog","doc":"* PTS and DTS timestamps were added to `Membrane.Buffer` structure explicitly. Timestamps should no longer live in `Membrane.Buffer.metadata` field [#335](https://github.com/membraneframework/membrane_core/pull/335).\n\n#","ref":"changelog.html#release-notes"},{"type":"extras","title":"PRs not influencing public API: - Changelog","doc":"* add CHANGELOG update verification [#340](https://github.com/membraneframework/membrane_core/pull/340)\n  * action enforcing changelog fix [#342](https://github.com/membraneframework/membrane_core/pull/342)\n  * bump version to 0.8.0 [#344](https://github.com/membraneframework/membrane_core/pull/344)","ref":"changelog.html#prs-not-influencing-public-api"},{"type":"extras","title":"Contributing","doc":"# Contributing\n\nAny contributions to Membrane Framework are welcome. If you would like contribute, but you're not sure how to start or have some questions, don't hesitate to contact us via channels described in [README](README.md#support-and-questions)\n\nBefore submitting code please read our [Coding style guide](https://membrane.stream/guide/v0.9/contributing_guide.html)\n\nWhen contributing to existing repo:\n\n- fork it\n- apply change on some branch\n- create a PR from fork to our repo\n- await feedback from someone from the team\n- after passing the review, the PR will be merged\n\nIf you wish to create a new plugin you can give us a shout and if it's something we want as a part of our ecosystem we'll create a repo for you, guide you on your work and maintain the plugin in future.\n\nHowever, if you feel confident enough to maintain it on your own, you can, of course, create your own repo and hex package - we only ask you to follow the naming convetions of the framework package and modules. In that case, don't forget to let us know about your work so we could include it on the list of available plugins.","ref":"contributing.html"},{"type":"extras","title":"Upgrading to v0.11","doc":"# Upgrading to v0.11\n\nImprovements in v0.11 required some breaking changes, so here comes the guide that will help you adjust your code to the new API. See the [changelog](https://github.com/membraneframework/membrane_core/releases/tag/v0.11.0) for details.","ref":"v0-11.html"},{"type":"extras","title":"Deps upgrade - Upgrading to v0.11","doc":"Update `membrane_core` to `v0.11`\n```elixir \ndefp deps do\n   [\n     {:membrane_core, \"~> 0.11.0\"},\n     ...\n   ]\nend\n```","ref":"v0-11.html#deps-upgrade"},{"type":"extras","title":"Use the new way of interaction with `Membrane.Pipeline` - Upgrading to v0.11","doc":"Use the new return type of `Membrane.Pipeline.start/3` and `Membrane.Pipeline.start_link/3`\n\n```diff\n- {:ok, pid} = Membrane.Pipeline.start_link(...)\n- send(pid, :message)\n+ {:ok, _pipeline_supervisor, pipeline} = Membrane.Pipeline.start_link(...)\n+ send(pipeline, :message)\n```\n\nNow, when you get `EXIT` signal from `pipeline`, it means only that the pipeline process is dead.\nWhen you get `EXIT` signal from `pipeline_supervisor`, it means that the pipeline,\nall its children, descendants and utilities are dead.\n\nIn tests, it's best to use `Membrane.Testing.Pipeline.start_link_supervised!/1`. See the _Testing_ section for details.","ref":"v0-11.html#use-the-new-way-of-interaction-with-membrane-pipeline"},{"type":"extras","title":"Update format of tuples returned from callbacks - Upgrading to v0.11","doc":"Update type of tuples returned from callbacks to `{actions, state}`\nIf callback returns:\n\n * `{:ok, state}`, change it to `{[], state}`\n\n```diff\n- {:ok, state}\n+ {[], state}\n```\n\n * `{{:ok, actions}, state}`, change it to `{actions, state}`\n\n```diff\n- {{:ok, event: {pad, event}}, state}\n+ {[event: {pad, event}], state}\n```\n\n * `{:error, reason}` or `{{:error, reason}, state}`, raise an error, instead of returning value from callback\n\n```diff\n- {:error, :something_went_wrong}\n+ raise \"Something went wrong!\"\n```","ref":"v0-11.html#update-format-of-tuples-returned-from-callbacks"},{"type":"extras","title":"Update callbacks - Upgrading to v0.11","doc":"Add `context` argument to `handle_init`\n\n```diff\n- def handle_init(options) do\n+ def handle_init(_context, options) do\n```\n\nInstead of `handle_stopped_to_prepared/2`, implement `handle_setup/2`\n\n```diff\n- def handle_stopped_to_prepared(ctx, state) do\n+ def handle_setup(ctx, state) do\n```\n\nIn general, `handle_init` should cover simple tasks like parsing options, while `handle_setup` is there for complex and long-lasting operations. See the docs for details.\n\n___\n\nRename `handle_prepared_to_playing/2` to `handle_playing/2`\n\n```diff\n- def handle_prepared_to_playing(ctx, state) do\n+ def handle_playing(ctx, state) do\n```\n\nRemove `handle_playing_to_prepared/2`, `handle_prepared_to_stopped/2` and `handle_shutdown/2`. Instead, use `Membrane.ResourceGuard`, `Membrane.UtilitySupervisor` or `handle_terminate_request/2`:\n\n```diff\n- def handle_stopped_to_prepared(_ctx, state) do\n-  resource = create_resource()\n-  {:ok, %{state | resource: resource}}\n- end\n- ...\n- def handle_prepared_to_stopped(_ctx, state) do\n-  cleanup(state.resource)\n- end\n+ def handle_setup(ctx, state) do\n+  resource = create_resource()\n+  # The resource will be automatically cleaned up when the component terminates\n+  # unless you do that earlier by calling `Membrane.Resource.cleanup/2`\n+  Membrane.ResourceGuard.register(\n+    ctx.resource_guard,\n+    fn -> cleanup(resource)\n+  end)\n+  {[], %{state | resource: resource}}\n+ end\n```\n\n```diff\n- def handle_stopped_to_prepared(_ctx, state) do\n-  {:ok, pid} = GenServer.start_link(SomeServer, options)\n-  {:ok, %{state | utility: pid}}\n- end\n- ...\n- def handle_prepared_to_stopped(_ctx, state) do\n-  GenServer.stop(state.utility)\n- end\n+ def handle_setup(ctx, state) do\n+  Membrane.UtilitySupervisor.start_link_child(ctx.utility_supervisor, SomeServer)\n+  {[], %{state | utility: pid}}\n+ end\n```\n\n```diff\n- def handle_prepared_to_stopped(_ctx, state) do\n-   do_some_work(state)\n-   {:ok, state}\n- end\n+ def handle_terminate_request(ctx, state) do\n+   # This will only be called upon graceful termination,\n+   # i.e. `Membrane.Pipeline.terminate/1`, termination of parent\n+   # or being removed by parent (`remove_child` action).\n+   # Won't be called if component crashes.\n+   do_some_work(state)\n+\n+   # You can defer component termination by not returning `:terminate`\n+   # from this callback and returning it later instead.\n+   {[terminate: :normal], state}\n+ end\n```\n\nRename `handle_other/3` into `handle_info/3` (unless your `handle_other/3` was used to receive message from parent - if so, see subsection about `handle_parent_notification/3` below)\n\n```diff \n@impl true\n- def handle_other(message, _ctx, state) do\n+ def handle_info(message, _ctx, state) do\n```\n\nRename `playback_state` to `playback` in contexts. Rely on `:stopped` instead of `:prepared`\n\n```diff\n- def handle_other(message, %{playback_state: :playing}, state) do\n+ def handle_info(message, %{playback: :playing}, state) do\n```\n\n```diff\n- def handle_other(message, %{playback_state: :prepared}, state) do\n+ def handle_info(message, %{playback: :stopped}, state) do\n```\n\nIn elements, rename `handle_caps/4` to `handle_stream_format/4`\n\n```diff\n- def handle_caps(pad_ref, caps, ctx, state) do\n+ def handle_stream_format(pad_ref, stream_format, ctx, state) do\n```\n\nIn pipelines and bins, change arguments in `handle_element_start_of_stream` and `handle_element_end_of_stream`, since till now on they accept children name and pad reference as two separate arguments instead of storing them as tuple in the first argument.\n\n```diff\nuse Membrane.Pipeline\n...\n@impl true\n- def handle_element_end_of_stream({child_name, pad_ref}, context, state)\n+ def handle_element_end_of_stream(child_name, pad_ref, context, state)\n```\n\nIn pipelines and bins, rename `handle_notification/4` into `handle_child_notification/4`\n\n```diff\n@impl true\n- def handle_notification(notification, child, context, state) do\n+ def handle_child_notification(notification, child, context, state) do\n```\n___\n\nIn elements and bins, add an implementation of `handle_parent_notification/3` in place of the `handle_other/3` that were responsible for receiving messages from the parents.\n\n```diff\n@impl true\n- def handle_other(parent_notification, context, state) do\n+ def handle_parent_notification(parent_notification, context, state) do\n```","ref":"v0-11.html#update-callbacks"},{"type":"extras","title":"Rename actions returned from callbacks - Upgrading to v0.11","doc":"In elements:\n* `:caps` -> `:stream_format`\n* `:notify` -> `:notify_parent`\n  \nIn bins:\n* `:notify` -> `:notify_parent`\n* `:forward` -> `:notify_child`\n\nIn pipelines:\n* `:forward` -> `:notify_child`\n\n\n```diff\n- {{:ok, caps: %My.Format{freq: 1}}, state}\n+ {[stream_format: %My.Format{freq: 1}], state}\n```\n\n\n```diff\nuse Membrane.Bin\n...\n- {{:ok, [forward: {child, :response}]}, state}\n+ {[notify_child: {child, :response}], state}\n```","ref":"v0-11.html#rename-actions-returned-from-callbacks"},{"type":"extras","title":"Update pads definitions - Upgrading to v0.11","doc":"Instead of using `:caps`, use `:accepted_format` option.\nOption `:accepted_format` can receive:\n\n * Module name\n\n```diff\n- caps: My.Format\n+ accepted_format: My.Format \n```\n\n * Elixir pattern\n\n```diff\n- caps: {My.Format, field: one_of([:some, :enumeration])}\n+ accepted_format: %My.Format{field: value} when value in [:some, :enumeration]\n```\n\n```diff\n- caps: :any\n+ accepted_format: _any\n```\n\n * Call to `any_of` function. You can pass there as many arguments, as you want. Each argument should be an Elixir pattern or a module name\n\n```diff\n- caps: [My.Format, My.Another.Format]\n+ accepted_format: any_of(My.Format, My.Another.Format)\n```\n\n```diff\n- caps: [My.Format, {My.Another.Format, field: :strict_value}, My.Yet.Another.Format]\n+ accepted_format: any_of(My.Format, %My.Another.Format{field: :strict_value}, My.Yet.Another.Format)\n```","ref":"v0-11.html#update-pads-definitions"},{"type":"extras","title":"Update options definitions - Upgrading to v0.11","doc":"This section applies to options defined via `def_options` macro and pad options defined in `def_input_pad` or `def_output_pad`.\n\n___\n\nRemove `:type` key and related value from the options definitions. Add `:spec` instead, if it hasn't been added before, and proper `:inspector`, if the option has a default value, that shouldn't be inspected by `inspect/1` during generating docs.\n\n```diff \n- def_options tick_interval: [\n-                 type: :time, \n-                 default: Membrane.Time.seconds(1)\n-             ],\n-             process: [\n-                 type: :pid\n-             ]\n+ def_options tick_interval: [\n+                 spec: Membrane.Time.t(),\n+                 default: Membrane.Time.seconds(1),\n+                 inspector: &Membrane.Time.inspect/1\n+             ],\n+             process: [\n+                 spec: pid()\n+             ] \n```","ref":"v0-11.html#update-options-definitions"},{"type":"extras","title":"Membrane.Time functions - Upgrading to v0.11","doc":"Rename `Membrane.Time.to_ /1` into `Membrane.Time.round_to_ /1`:\n```diff\n- rounded_time = Membrane.Time.to_seconds(time)\n+ rounded_time = Membrane.Time.round_to_seconds(time)\n```","ref":"v0-11.html#membrane-time-functions"},{"type":"extras","title":"Update the children definitions - Upgrading to v0.11","doc":"Children definitions syntax (previously known as `ParentSpec`, after the name of a structure used to define children), that was used in `Membrane.Pipeline.Action.spec_t` and `Membrane.Bin.Action.spec_t` actions, has changed. \nSince there are quite a few changes concerning children definition, we have decided to present them in the subsections below.\n\n#","ref":"v0-11.html#update-the-children-definitions"},{"type":"extras","title":"Update children names - Upgrading to v0.11","doc":"We have restricted the available names for the children. Previously, the children name could be of `any()` type, now it is specified as follows: `tuple() | atom()`.\nIn case you have used a child name that does not match the new children names specification, you can wrap that name into a tuple, i.e.:\n```diff\n- child_name = [:why_would, :i_use, :a_list, :as_a, :child_name?]\n+ child_name = {:child_name, [:why_would, :i_use, :a_list, :as_a, :child_name?]}\n```\n\n#","ref":"v0-11.html#update-children-names"},{"type":"extras","title":"Update the children and links definition - Upgrading to v0.11","doc":"Rename the following function calls:\n* `Membrane.ParentSpec.link/1` -> `Membrane.ChildrenSpec.get_child/1`\n* `Membrane.ParentSpec.link/2` -> `Membrane.ChildrenSpec.child/2`\n* `Membrane.ParentSpec.link_bin_input/1` -> `Membrane.ChildrenSpec.bin_input/1`\n* `Membrane.ParentSpec.to/2` -> `Membrane.ChildrenSpec.get_child/2`\n* `Membrane.ParentSpec.to/3` -> `Membrane.ChildrenSpec.child/4`\n* `Membrane.ParentSpec.to_bin_output/2` -> `Membrane.ChildrenSpec.bin_output/2`\n\n```diff\n- import Membrane.ParentSpec\n- children = %{source: SomeSource, filter: SomeFilter}\n- links = [link(:source) |> to(:another_filter, SomeFilter) |> to(:filter) |> to(:sink, SomeSink)]\n+ import Membrane.ChildrenSpec\n+ structure = [\n+   child(:source, SomeSource),\n+   child(:filter, SomeFilter),\n+   get_child(:source) |> child(:another_filter, SomeFilter) |> get_child(:filter) |> child(:sink, SomeSink)\n+ ]\n```\n\n#","ref":"v0-11.html#update-the-children-and-links-definition"},{"type":"extras","title":"Substitute the `Membrane.ParentSpec` structure with a tuple - Upgrading to v0.11","doc":"`%Membrane.ParentSpec{children: children, links: links, options...}` should be changed into `{structure, options...}` with the `children` and `links` defined as\ndescribed in the step above. Note, that there is no distinguishement between `children` and `links` - \nthe children and links can be concatenated, so in case you have a separate `children` and `links` lists, simply merge them.\n\n```diff\n- spec = %Membrane.ParentSpec{children: children, links: links, node: another_node, log_metadata: metadata}\n+ spec = {structure, node: another_node, log_metadata: metadata}\n```\n\nBelow there is an example that aggregates the changes that need to be done to concerning the `:spec` action preparation: \n```diff\n- children = %{source: SomeSource, filter: SomeFilter}\n- links = [link(:source) |> to(:another_filter, SomeFilter) |> to(:filter) |> to(:sink, SomeSink)]\n- spec = Membrane.ParentSpec{children: children, links: links, crash_group: {:first_group, :temporary}, node: another_node}\n+ structure = [\n+   child(:source, SomeSource),\n+   child(:filter, SomeFilter),\n+   get_child(:source) |> child(:another_filter, SomeFilter) |> get_child(:filter) |> child(:sink, SomeSink)\n  ]\n+ spec = {structure, crash_group: {:first_group, :temporary}, node: another_node}\n- {{:ok, spec: spec}, %{}}\n+ {[spec: spec], %{}}\n```","ref":"v0-11.html#substitute-the-membrane-parentspec-structure-with-a-tuple"},{"type":"extras","title":"Testing - Upgrading to v0.11","doc":"Instead of using `Membrane.Testing.Pipeline.start_link/1`, use `Membrane.Testing.Pipeline.start_link_supervised!/1`\n```diff\n- {:ok, pipeline} = Membrane.Testing.Pipeline.start_link(options)\n+ pipeline = Membrane.Testing.Pipeline.start_link_supervised!(options)\n+ # The pipeline will be shut down at the end of the test.\n```\n___\n\nThe `Membrane.Testing.Pipeline.options()` has also changed - now there is no more separate `:links` and `:children` fields. Instead, you need to\nuse a `:structure` field:\n```diff\n- {:ok, pipeline} = Membrane.Testing.Pipeline.start_link(children: children, links: links)\n+ {:ok, pipeline} = Membrane.Testing.Pipeline.start_link(structure: structure)\n```\n\n___\n\nIn order to wait for the `Membrane.Testing.Pipeline` to get into the `:playing` playback, use `assert_pipeline_play/2` instead of `assert_pipeline_playback_changed/4`:\n\n```diff\n- assert_pipeline_playback_changed(pipeline, :prepared, :playing)\n+ assert_pipeline_play(pipeline)\n```","ref":"v0-11.html#testing"},{"type":"extras","title":"Upgrading to v0.12","doc":"# Upgrading to v0.12\n\nBetween v0.11 and v0.12 some breaking changes have occurred, so here comes the guide that will help you adjust your code to the new API. See the [release notes](https://github.com/membraneframework/membrane_core/releases/tag/v0.12.1) for details.","ref":"v0-12.html"},{"type":"extras","title":"Deps upgrade - Upgrading to v0.12","doc":"Upgrade `membrane_core` to `v0.12.1`.\n\n```elixir \ndefp deps do\n   [\n     {:membrane_core, \"~> 0.12.1\"},\n     ...\n   ]\nend\n```","ref":"v0-12.html#deps-upgrade"},{"type":"extras","title":"Implement `handle_child_pad_removed/4` callback in bins and pipelines, if it is needed - Upgrading to v0.12","doc":"Now, if bin removes its pad (e.g. by removing an element linked to the bin's inner pad), bin's parent has to have implemented proper `handle_child_pad_removed/4` callback, to handle it. If there is no such a callback, default behaviour is to raise an error. \n\n```elixir\n@impl true\ndef handle_child_pad_removed(:rtp, Pad.ref(:rtp_input, _ssrc), _ctx, state) do\n  # ...\nend\n```","ref":"v0-12.html#implement-handle_child_pad_removed-4-callback-in-bins-and-pipelines-if-it-is-needed"},{"type":"extras","title":"Remove `:playback` action - Upgrading to v0.12","doc":"Now, membrane pipelines enter the playing playback by default and they don't have to return a `:playback` action to do it. \n\n```diff\n- @impl true \n- def handle_setup(_ctx, state) do\n- {[playback: :playing], state}\n- end\n```\nInstead of it, there is a new action introduced in `membrane_core` v0.12, `setup: :incomplete | :complete`. If you want to defer a moment when a component enters the playing playback, you can return `{:setup, :incomplete}` action from `handle_setup` callback. If you do that, a component will enter the playing playback only when you return `{:setup, :complete}` action from another callback, e.g. `handle_info`.\n\n```diff \n- @impl true \n- def handle_setup(_ctx, state) do\n-   Process.send_after(self(), :play, 1000)\n-   {[], state}\n- end\n-\n- @impl true \n- def handle_info(:play, _ctx, state) do\n-   {[playback: :playing], state} \n- end\n\n+ @impl true \n+ def handle_setup(_ctx, state) do\n+   Process.send_after(self(), :play, 1000)\n+   {[setup: :incomplete], state}\n+ end\n+\n+ @impl true\n+ def handle_info(:play, _ctx, state) do\n+   {[setup: :complete], state}\n+ end\n```\n\n`:setup` action is available not only in pipelines but in bins and elements as well.","ref":"v0-12.html#remove-playback-action"},{"type":"extras","title":"Upgrading to v1.0.0-rc0","doc":"# Upgrading to v1.0.0-rc0\n\nBetween v0.11 and v1.0.0-rc0 some breaking changes have occurred, so here comes the guide that will help you adjust your code to the new API. See the [changelog](https://github.com/membraneframework/membrane_core/releases/tag/v1.0.0-rc0) for detailed description of the changes.\n\n#","ref":"v1-0-0-rc0.html"},{"type":"extras","title":"Deps upgrade - Upgrading to v1.0.0-rc0","doc":"Update `membrane_core` to `v1.0.0-rc0`\n```elixir \ndefp deps do\n   [\n     {:membrane_core, \"~> 1.0.0-rc0\"},\n     ...\n   ]\nend\n```\n\n#","ref":"v1-0-0-rc0.html#deps-upgrade"},{"type":"extras","title":"Remove the pipeline's `:playback` action - Upgrading to v1.0.0-rc0","doc":"There is no more `:playback` action available, as the pipeline will start playing automatically once its setup is completed.\nIf you need to start the pipeline as soon as possible, simply do not return the `:playback` action in `c:Membrane.Pipeline.handle_init/2` or `c:Membrane.Pipeline.handle_setup/2` callbacks:\n```diff\n@impl true\ndef handle_setup(_ctx, state) do\n  ...\n- {[playback: :playing], state}\n+ {[], state}\nend\n```\nIn case you need to defer the moment the pipeline starts playing, and for this purpose you have returned `:playback` action in some other callback than `handle_init` or `handle_setup`, now you need to do two things - first, mark the setup as incomplete with `t:Membrane.Pipeline.Action.setup/0` action returned from `handle_setup`, to prevent it from automatically changing the playback to `:playing`:\n\n```diff\n+ @impl true\n+ def handle_setup(_ctx, state) do\n+   {[setup: :incomplete], state}\n+ end\n```\n\nThen, you need to return `setup: :complete` instead of `playback: :playing` action in a callback after which completion you want your pipeline to start playing:\n```diff\n@impl true\ndef handle_info(:now_start_playing, _ctx, state) do\n...\n- {[playback: playing], state}\n+ {[setup: :complete], state}\nend\n```\n\nNote that the `:setup` action is also available in elements and bins and its main purpose is to handle long, asynchronous initialization of a component. See `t:Membrane.Bin.Action.setup/0` and `t:Membrane.Element.Action.setup/0`.\n\n#","ref":"v1-0-0-rc0.html#remove-the-pipeline-s-playback-action"},{"type":"extras","title":"Rename `handle_process` and `handle_write` into `handle_buffer` - Upgrading to v1.0.0-rc0","doc":"Names of the callbacks that are used to process buffers have been unified. This applies to:\n* _Membrane.Filter.handle_process/4_ and _Membrane.Filter.handle_process_list/4_\n* _Membrane.Endpoint.handle_write/4_ and _Membrane.Endpoint.handle_write_list/4_\n* _Membrane.Sink.handle_write/4_ and _Membrane.Sink.handle_write_list/4_\n\nand they became `c:Membrane.Element.WithInputPads.handle_buffer/4` and _Membrane.Element.WithInputPads.handle_buffers_batch/4_, respectively:\n\n```diff\n  @impl true\n- def handle_process(pad, buffer, ctx, state) do\n+ def handle_buffer(pad, buffer, ctx, state) do\n    ...\n  end\n```\n\n```diff\n  @impl true\n- def handle_process_list(pad, buffers, ctx, state) do\n+ def handle_buffers_batch(pad, buffers, ctx, state) do\n    ...\n  end\n```\n\n#","ref":"v1-0-0-rc0.html#rename-handle_process-and-handle_write-into-handle_buffer"},{"type":"extras","title":"Change `mode` and `demand_mode` options to `flow_control` in pads' definitions in elements - Upgrading to v1.0.0-rc0","doc":"For input pads, change:\n\n```diff\nuse Membrane.Filter\n# or Sink or Endpoint\n\n- def_input_pad :input, mode: :push, ...\n+ def_input_pad :input, flow_control: :push, ...\n\n- def_input_pad :input, mode: :pull, demand_mode: :auto, demand_unit: :buffers, ...\n+ def_input_pad :input, ...\n# (because `flow_control: :auto` is the default whenever available - currently in Filters)\n# Note that having `flow_control: :auto` doesn't allow to pass `demand_unit`,\n# as it's determined automatically\n\n- def_input_pad :input, mode: :pull, demand_unit: :buffers, ...\n+ def_input_pad :input, flow_control: :manual, demand_unit: :buffers, ...\n```\n\nSame goes for output pads:\n\n```diff\nuse Membrane.Filter\n# or Source or Endpoint\n\n- def_output_pad :output, mode: :push, ...\n+ def_output_pad :output, flow_control: :push, ...\n\n- def_output_pad :output, mode: :pull, demand_mode: :auto,  ...\n+ def_output_pad :output, ...\n\n- def_output_pad :output, mode: :pull, ...\n+ def_output_pad :output, flow_control: :manual, ...\n```\n\nCheck `t:Membrane.Pad.element_spec/0` for details.\n\n#","ref":"v1-0-0-rc0.html#change-mode-and-demand_mode-options-to-flow_control-in-pads-definitions-in-elements"},{"type":"extras","title":"Remove `mode` and `demand_unit` from pads definitions in bins - Upgrading to v1.0.0-rc0","doc":"```diff\nuse Membrane.Bin\n\n- def_input_pad :input, mode: :pull, demand_unit: :buffers, ...\n+ def_input_pad :input, ...\n\n- def_output_pad :output, mode: :push, ...\n+ def_output_pad :output, ...\n```\n\nCheck `t:Membrane.Pad.bin_spec/0` for details.\n\n#","ref":"v1-0-0-rc0.html#remove-mode-and-demand_unit-from-pads-definitions-in-bins"},{"type":"extras","title":"Don't refer to callback contexts as to structs - Upgrading to v1.0.0-rc0","doc":"Callback contexts are now plain maps instead of structs. So, for example, if you happen to have a match like below, convert it to a match on a map:\n\n```diff\n@impl true\n- def handle_info(message, %Membrane.Element.CallbackContext.Info{pads: pads}, state) do\n+ def handle_info(message, %{pads: pads}, state) do\n  ...\nend\n```\n\nAdditionally, there's no `direction` field anymore in the callback context for `handle_pad_added` and `handle_pad_removed` - the direction can be determined by the pad name. All other fields remain the same.\n\n```diff\n@impl true\n- def handle_pad_added(_pad, %{direction: :input}, state) do\n+ def handle_pad_added(Pad.ref(:input, _id), _ctx, state) do\n  ...\nend\n```\n\nCheck `t:Membrane.Element.CallbackContext.t/0`, `t:Membrane.Bin.CallbackContext.t/0` and `t:Membrane.Pipeline.CallbackContext.t/0` for outline of what can be found in the contexts. Additionally, each callback that provides extra fields in the context, has them mentioned in its docs.\n\n#","ref":"v1-0-0-rc0.html#don-t-refer-to-callback-contexts-as-to-structs"},{"type":"extras","title":"Use `Membrane.RCPipeline` instead of `Membrane.RemoteControlled.Pipeline` - Upgrading to v1.0.0-rc0","doc":"```diff\n- pipeline = Membrane.RemoteControlled.Pipeline.start_link!()\n+ pipeline = Membrane.RCPipeline.start_link!()\n```\n\nSame goes for `Membrane.RemoteControlled.Message` -> `Membrane.RCMessage`\n\n```diff\nreceive do\n- %Membrane.RemoteControlled.Message.Notification{data: data} -> data\n+ %Membrane.RCMessage.Notification{data: data} -> data\nend\n```\n\n#","ref":"v1-0-0-rc0.html#use-membrane-rcpipeline-instead-of-membrane-remotecontrolled-pipeline"},{"type":"extras","title":"Use `Membrane.Time.as_ (time, :round)` instead of `Membrane.Time.round_to_ (time)` - Upgrading to v1.0.0-rc0","doc":"```diff\n- Membrane.Time.round_to_milliseconds(time)\n+ Membrane.Time.as_milliseconds(time, :round)\n```\n\nThere is one exception for `round_to_timebase`, which changed to `divide_by_timebase`:\n```diff\n- Membrane.Time.round_to_timebase(time, timebase)\n+ Membrane.Time.divide_by_timebase(time, timebase)\n```\n\nCheck `Membrane.Time` for details.\n\n#","ref":"v1-0-0-rc0.html#use-membrane-time-as_-time-round-instead-of-membrane-time-round_to_-time"},{"type":"extras","title":"Remove `_t` suffix when referring to Membrane types - Upgrading to v1.0.0-rc0","doc":"```diff\n- Membrane.Pad.ref_t()\n+ Membrane.Pad.ref()\n```","ref":"v1-0-0-rc0.html#remove-_t-suffix-when-referring-to-membrane-types"},{"type":"extras","title":"Upgrading to v1.0.0-rc1","doc":"# Upgrading to v1.0.0-rc1\n\n#","ref":"v1-0-0-rc1.html"},{"type":"extras","title":"Remove `handle_buffers_batch/4`/`handle_process_list/4`/`handle_write_list/4` callback - Upgrading to v1.0.0-rc1","doc":"In `v1.0.0-rc1` we removed `handle_buffers_batch/4` callback. Instead of handling list of buffer in this callback, you have to handle every single buffer separately in `handle_buffer/4` callback.\n\n```diff\n@impl true\n- def handle_buffers_batch(pad_ref, buffers_list, ctx, state) do\n+ def handle_buffer(pad_ref, buffer, ctx, state) do\n  ...\nend\n```\n\nNotice, if you are upgrading your code from `v0.12.*` directly to `v1.0.0-rc1`, omitting `v1.0.0-rc0`, there is a chance, that your elements still have callback named `handle_process_list/4` or `handle_write_list/4`, instead of `handle_buffers_batch/4`.\n\n```diff\n@impl true\n- def handle_process_list(pad_ref, buffers_list, ctx, state) do\n+ def handle_buffer(pad_ref, buffer, ctx, state) do\n  ...\nend\n```\n\n```diff\n@impl true\n- def handle_write_list(pad_ref, buffers_list, ctx, state) do\n+ def handle_buffer(pad_ref, buffer, ctx, state) do\n  ...\nend\n```\n\n#","ref":"v1-0-0-rc1.html#remove-handle_buffers_batch-4-handle_process_list-4-handle_write_list-4-callback"},{"type":"extras","title":"Implement you own `start/*`, `start_link/*` or `terminate/1` function (if you want to) - Upgrading to v1.0.0-rc1","doc":"Until `v1.0.0-rc0`, Membrane has generated `start/2`, `start_link/2`, and `terminate/1` functions in modules using `Membrane.Pipeline`, if code developer hadn't done it explicitly. Since `v1.0.0-rc1`, if you want to have these functions implemented in your pipeline module, you have to implement them on your own. Alternatively, you can always call `Membrane.Pipeline.start(YourPipelineModule, init_arg, opts)`, `Membrane.Pipeline.start_link(YourPipelineModule, init_arg, opts)`, and `Membrane.Pipeline.terminate(pipeline_pid)` from beyond `YourPipelineModule` without wrapping it into public functions.\n\n#","ref":"v1-0-0-rc1.html#implement-you-own-start-start_link-or-terminate-1-function-if-you-want-to"},{"type":"extras","title":"Rename `:structure` option passed to the `Membrane.Testing.Pipeline` into `:spec` - Upgrading to v1.0.0-rc1","doc":"```diff\nimport Membrane.ChildrenSpec\n\nspec = \n  child(:source, %Membrane.Testing.Source{some_field: some_arg})\n  |> child(:sink), %Membrane.Testing.Sink{another_filed: another_arg}\n\n- pipeline = Membrane.Testing.Pipeline.start_link_supervised(structure: spec)\n+ pipeline = Membrane.Testing.Pipeline.start_link_supervised(spec: spec)\n```","ref":"v1-0-0-rc1.html#rename-structure-option-passed-to-the-membrane-testing-pipeline-into-spec"},{"type":"extras","title":"Upgrading to v1.0.0","doc":"# Upgrading to v1.0.0\n\nComparing to v0.12, v1.0.0 introduces some minor changes in the public API of the framework. This guide was created to help you migrate to v1.0.0 of `membrane_core`.\n\n#","ref":"v1-0-0.html"},{"type":"extras","title":"Deps - Upgrading to v1.0.0","doc":"Update `membrane_core` to `v1.0.0`\n```elixir \ndefp deps do\n   [\n     {:membrane_core, \"~> 1.0.0\"},\n     ...\n   ]\nend\n```\n\n#","ref":"v1-0-0.html#deps"},{"type":"extras","title":"Rename `handle_process/4` and `handle_write/4` into `handle_buffer/4` - Upgrading to v1.0.0","doc":"Names of the callbacks that are used to process buffers have been unified. This applies to:\n* _Membrane.Filter.handle_process/4_\n* _Membrane.Endpoint.handle_write/4_\n* _Membrane.Sink.handle_write/4_\n\nand they became `c:Membrane.Element.WithInputPads.handle_buffer/4`\n\n```diff\n  @impl true\n- def handle_process(pad, buffer, ctx, state) do\n+ def handle_buffer(pad, buffer, ctx, state) do\n    ...\n  end\n```\n\n#","ref":"v1-0-0.html#rename-handle_process-4-and-handle_write-4-into-handle_buffer-4"},{"type":"extras","title":"Remove `handle_process_list/4` and `handle_write_list/4` callback - Upgrading to v1.0.0","doc":"Since `v1.0.0`, you have to handle every single buffer separately in `handle_buffer/4` callback, instead of handling whole list of buffers in this in `handle_process_list/4` or `handle_write_list/4`.\n\n```diff\n@impl true\n- def handle_process_list(pad_ref, buffers_list, ctx, state) do\n+ def handle_buffer(pad_ref, buffer, ctx, state) do\n  ...\nend\n```\n\n```diff\n@impl true\n- def handle_write_list(pad_ref, buffers_list, ctx, state) do\n+ def handle_buffer(pad_ref, buffer, ctx, state) do\n  ...\nend\n```\n\n#","ref":"v1-0-0.html#remove-handle_process_list-4-and-handle_write_list-4-callback"},{"type":"extras","title":"Change `mode` and `demand_mode` options to `flow_control` in pads' definitions in elements - Upgrading to v1.0.0","doc":"For input pads, change:\n\n```diff\nuse Membrane.Filter\n# or Sink or Endpoint\n\n- def_input_pad :input, mode: :push, ...\n+ def_input_pad :input, flow_control: :push, ...\n\n- def_input_pad :input, mode: :pull, demand_mode: :auto, demand_unit: :buffers, ...\n+ def_input_pad :input, ...\n# (because `flow_control: :auto` is the default whenever available - currently in Filters)\n# Note that having `flow_control: :auto` doesn't allow to pass `demand_unit`,\n# as it's determined automatically\n\n- def_input_pad :input, mode: :pull, demand_unit: :buffers, ...\n+ def_input_pad :input, flow_control: :manual, demand_unit: :buffers, ...\n```\n\nSame goes for output pads:\n\n```diff\nuse Membrane.Filter\n# or Source or Endpoint\n\n- def_output_pad :output, mode: :push, ...\n+ def_output_pad :output, flow_control: :push, ...\n\n- def_output_pad :output, mode: :pull, demand_mode: :auto,  ...\n+ def_output_pad :output, ...\n\n- def_output_pad :output, mode: :pull, ...\n+ def_output_pad :output, flow_control: :manual, ...\n```\n\nCheck `t:Membrane.Pad.element_spec/0` for details.\n\n#","ref":"v1-0-0.html#change-mode-and-demand_mode-options-to-flow_control-in-pads-definitions-in-elements"},{"type":"extras","title":"Adjust to change in default `flow_control` (aka. `mode`/`demand_mode`) value. - Upgrading to v1.0.0","doc":"Now, if definition of pad in element does specify type of \n\nNow, the default value of `flow_control` option in `def_input_pad` and `def_output_pad` in Elements is `:auto`. Until `v0.12.9`, pads that sepcified neither `flow_control`, `mode` nor `demand_mode` explicitly would have `:manual` `flow_control`. Therefore, `:manual` pads that haven't specified `flow_control` value, now have to do it explicitly.\n\n```diff\n- def_output_pad :output, accepted_format: %MyStruct{field: :value}\n+ def_output_pad :output, accepted_format: %MyStruct{field: :value}, flow_control: :manual\n```\n\n```diff\n- def_input_pad :input, accepted_format: %MyStruct{field: :value}, demand_unit: :buffers\n+ def_input_pad :input, accepted_format: %MyStruct{field: :value}, demand_unit: :buffers, flow_control: :manual\n```\n\nMoreover, you can remove `flow_control: :auto` from the pad definitions, if you want to.\n\n```diff\n- def_input_pad :input, accepted_format: %MyStruct{field: :value}, flow_control: :auto\n+ def_input_pad :input, accepted_format: %MyStruct{field: :value}\n```\n\n#","ref":"v1-0-0.html#adjust-to-change-in-default-flow_control-aka-mode-demand_mode-value"},{"type":"extras","title":"Remove `mode` and `demand_unit` from pads definitions in bins - Upgrading to v1.0.0","doc":"```diff\nuse Membrane.Bin\n\n- def_input_pad :input, mode: :pull, demand_unit: :buffers, ...\n+ def_input_pad :input, ...\n\n- def_output_pad :output, mode: :push, ...\n+ def_output_pad :output, ...\n```\n\nCheck `t:Membrane.Pad.bin_spec/0` for details.\n\n#","ref":"v1-0-0.html#remove-mode-and-demand_unit-from-pads-definitions-in-bins"},{"type":"extras","title":"Don't refer to callback contexts as to structs - Upgrading to v1.0.0","doc":"Callback contexts are now plain maps instead of structs. So, for example, if you happen to have a match like below, convert it to a match on a map:\n\n```diff\n@impl true\n- def handle_info(message, %Membrane.Element.CallbackContext.Info{pads: pads}, state) do\n+ def handle_info(message, %{pads: pads}, state) do\n  ...\nend\n```\n\nAdditionally, there's no `direction` field anymore in the callback context for `handle_pad_added` and `handle_pad_removed` - the direction can be determined by the pad name. All other fields remain the same.\n\n```diff\n@impl true\n- def handle_pad_added(_pad, %{direction: :input}, state) do\n+ def handle_pad_added(Pad.ref(:input, _id), _ctx, state) do\n  ...\nend\n```\n\nCheck `t:Membrane.Element.CallbackContext.t/0`, `t:Membrane.Bin.CallbackContext.t/0` and `t:Membrane.Pipeline.CallbackContext.t/0` for outline of what can be found in the contexts. Additionally, each callback that provides extra fields in the context, has them mentioned in its docs.\n\n#","ref":"v1-0-0.html#don-t-refer-to-callback-contexts-as-to-structs"},{"type":"extras","title":"Use `Membrane.RCPipeline` instead of `Membrane.RemoteControlled.Pipeline` - Upgrading to v1.0.0","doc":"```diff\n- pipeline = Membrane.RemoteControlled.Pipeline.start_link!()\n+ pipeline = Membrane.RCPipeline.start_link!()\n```\n\nSame goes for `Membrane.RemoteControlled.Message` -> `Membrane.RCMessage`\n\n```diff\nreceive do\n- %Membrane.RemoteControlled.Message.Notification{data: data} -> data\n+ %Membrane.RCMessage.Notification{data: data} -> data\nend\n```\n\n#","ref":"v1-0-0.html#use-membrane-rcpipeline-instead-of-membrane-remotecontrolled-pipeline"},{"type":"extras","title":"Use `Membrane.Time.as_ (time, :round)` instead of `Membrane.Time.round_to_ (time)` - Upgrading to v1.0.0","doc":"```diff\n- Membrane.Time.round_to_milliseconds(time)\n+ Membrane.Time.as_milliseconds(time, :round)\n```\n\nThere is one exception for `round_to_timebase`, which changed to `divide_by_timebase`:\n```diff\n- Membrane.Time.round_to_timebase(time, timebase)\n+ Membrane.Time.divide_by_timebase(time, timebase)\n```\n\nCheck `Membrane.Time` for details.\n\n#","ref":"v1-0-0.html#use-membrane-time-as_-time-round-instead-of-membrane-time-round_to_-time"},{"type":"extras","title":"Implement you own `start/*`, `start_link/*` or `terminate/1` function (if you want to) - Upgrading to v1.0.0","doc":"Until `v1.0.0-rc0` and `v0.12`, Membrane has generated `start/2`, `start_link/2`, and `terminate/1` functions in modules using `Membrane.Pipeline`, if code developer hadn't done it explicitly. Since `v1.0.0`, if you want to have these functions implemented in your pipeline module, you have to implement them on your own. Alternatively, you can always call `Membrane.Pipeline.start(YourPipelineModule, init_arg, opts)`, `Membrane.Pipeline.start_link(YourPipelineModule, init_arg, opts)`, and `Membrane.Pipeline.terminate(pipeline_pid)` from beyond `YourPipelineModule` without wrapping it into public functions.\n\n#","ref":"v1-0-0.html#implement-you-own-start-start_link-or-terminate-1-function-if-you-want-to"},{"type":"extras","title":"Rename `:structure` option passed to the `Membrane.Testing.Pipeline` into `:spec` - Upgrading to v1.0.0","doc":"```diff\nimport Membrane.ChildrenSpec\n\nspec = \n  child(:source, %Membrane.Testing.Source{some_field: some_arg})\n  |> child(:sink), %Membrane.Testing.Sink{another_filed: another_arg}\n\n- pipeline = Membrane.Testing.Pipeline.start_link_supervised(structure: spec)\n+ pipeline = Membrane.Testing.Pipeline.start_link_supervised(spec: spec)\n```\n\n#","ref":"v1-0-0.html#rename-structure-option-passed-to-the-membrane-testing-pipeline-into-spec"},{"type":"extras","title":"Adjust to possibility of receiving `end of stream` on pads, that haven't received `start of stream` yet. - Upgrading to v1.0.0","doc":"Since `v1.0.0`, Membrane will execute `c:Membrane.Element.WithInputPads.handle_end_of_stream/3` for all pads, including these having `start_of_stream?: false`.","ref":"v1-0-0.html#adjust-to-possibility-of-receiving-end-of-stream-on-pads-that-haven-t-received-start-of-stream-yet"},{"type":"extras","title":"License","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright 2018 Software Mansion\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"}],"content_type":"text/markdown"}